EntityLister = function (container, config) {
    var self = this;

    let baseURL  = null;
    let apiToken = null;

    var controls   = {};
    var tags       = {};
    var fields     = {};
    var fullList   = [];
    var entityType = null;
    var mgtZone    = null;
                 
    this.events  = {};

    var emit = function (eventName, data) {
        // Call the subscribed event handlers.
        const event = self.events[eventName] || [];
        event.forEach(fn => { fn.call(null, data); });
    };

    var subscribe = function (eventName, fn) {
        // Add the subscription (event handler) for this event.
        self.events[eventName] = self.events[eventName] || [];
        self.events[eventName].push(fn);

        // Return an 'unsubscribe' function for this subscription.
        return () => {
            self.events[eventName] = self.events[eventName].filter(eventFn => fn !== eventFn);
        };
    };

    function populateFilterControl(filterControl, fields, tags) {
        var filterOptions = [];
        
        for (var name in fields) {
            filterOptions.push({
                type:   "Field",
                name:   name,
                values: fields[name]
            });
        }
        for (var name in tags) {
            filterOptions.push({
                type:  "Tag",
                name:   name,
                values: tags[name]
            });
        }
        filterControl.options = filterOptions;
    }

    // Called whenever the filter changes.    
    function onFilterChange(ev) {
        console.log(ev.target.id + ": " + JSON.stringify(ev.value));

        filter();

        controls.availableList.specials = jobEditor.itemIds;
        controls.availableList.items    = filteredList;  
    }    
    
    function filter() {        
        var filters = controls.filterControl.value || [];

        // Index tag values for quick access.
        fullList.forEach(function (item) {
            item.tagValuesByKey = {};
            
            item.tags.forEach(function (tag) {
                tag.value = tag.value || "(no value)";
                
                item.tagValuesByKey[tag.key] = tag.value;
            });
        });
                 
        filteredList = filters.length === 0 
                      ? fullList 
                      : fullList.filter(function (item) {
            for (var i=0; i < filters.length; i++) {
                var filter = filters[i];
                var isIncluded = filter.type === "Field"
                               ? ( filter.values.length == 0
                                   ? item.hasOwnProperty(filter.name)
                                   : filter.values.includes(item[filter.name])
                                 )
                               : ( filter.values.length == 0 
                                   ? item.tagValuesByKey.hasOwnProperty(filter.name)
                                   : filter.values.includes(item.tagValuesByKey[filter.name])
                                 );
                
                if (( isIncluded &&  filter.negated) ||
                    (!isIncluded && !filter.negated))
                    return false;
            }
            return true;
        });
         
        filterMatches.innerHTML = filteredList.length + " matches";
        return filteredList;
    }
                
    function analyze() {
        var toLocaleString = function (time) { 
            var date = new Date();
            date.setTime(time);
            return date.toLocaleString();
        }
        
        /*  Based on the available fields and tag values, we will 
            create an inventory that helps us generate dropdowns easily.
         */
        tags   = {};
        fields = {};
        zones  = {};
        
        var now = (new Date()).getTime();
        var hour = 1000*60*60;
        
        filteredList.forEach(function (item, i) {
            item.id        = item.entityId;
            item.entityURL = baseURL + uiEndPoints[jobEditor.type] + item.entityId; 
            
            var firstSeenSince   = Math.round((now - item.firstSeenTimestamp) / hour);
            var lastSeenSince    = Math.round((now - item.lastSeenTimestamp)  / hour);
            
            item.firstSeenTime   = toLocaleString(item.firstSeenTimestamp) 
                                 + (firstSeenSince <=72 ? (" <span style='font-style:italic;font-size:85%;'> (" + firstSeenSince + " hours ago)</span>") : "");
            item.lastSeenTime    = toLocaleString(item.lastSeenTimestamp)
                                 + (lastSeenSince  <=72 ? (" <span style='font-style:italic;font-size:85%;'> (" + lastSeenSince  + " hours ago)</span>") : "");
            item.managementZones = item.managementZones || [];
            item.tags            = item.tags || [];
            item.tagValuesByKey  = {};
            item.zoneValuesByKey = {};
            
            // Collect tag values.
            item.tags.forEach(function (tag) {
                tag.value = tag.value || "(no value)";
                
                item.tagValuesByKey[tag.key] = tag.value;
                
                if (tags.hasOwnProperty(tag.key)) {
                    if (!tags[tag.key].includes(tag.value))
                        tags[tag.key].push(tag.value);
                }
                else {
                    tags[tag.key] = [tag.value];
                }
            });
            
            // Collect zones seen in the data.
            item.managementZones.forEach(function (zone) {
                item.zoneValuesByKey[zone.id] = zone.name;
                
                if (!zones.hasOwnProperty(zone.id)) {
                    zones[zone.id] = zone.name;
                }
            });

            // Collect field values (except common ones).
            for (var prop in item) {
                if (typeof item[prop] === "string" && 
                    !excludedFields.includes(prop)) {
                    
                    if (fields.hasOwnProperty(prop)) {
                        if (!fields[prop].includes(item[prop]))
                            fields[prop].push(item[prop]);
                    }
                    else {
                        fields[prop] = [item[prop]];
                    }
                }
                else {
                    // We don't use fields that are not contain simple strings.
                    // And we also (per the above) exclude common fields.
                }
            }            
        });
        
        // For the filter area:
        populateFilterControl(controls.filterControl, fields, tags);
    }   
        
    function processData(jsonData) {
        fullList  = jsonData.sort(function (a, b) {
            if (!a.sortName)
                a.sortName = a.displayName.trim().toLowerCase();
            if (!b.sortName)
                b.sortName = b.displayName.trim().toLowerCase();
            
            return a.sortName < b.sortName
                 ? -1
                 : a.sortName > b.sortName
                 ? 1
                 : 0;
        });
        
        filter();
        analyze();

        controls.resultCount.innerHTML  = fullList.length + " items";
        controls.availableList.specials = jobEditor.itemIds;
        controls.availableList.items    = filteredList;  
    }   

      
    function getData() {
        // Clear out old data - visually, too.
        filteredList = [];
        fullList   = [];

        controls.availableList.specials = jobEditor.itemIds;
        controls.availableList.items    = filteredList;  
        
        // Request new data.
        try {
            var xhr = new JSONHttpRequest({         
                address:      baseURL + "/api/v1/" + apiEndPoints[entityType] 
                              + '?Api-Token=' + apiToken
                              + (mgtZone != "all"
                                 ? ('&managementZone=' + mgtZone)
                                 : ''
                              ),
                onComplete:   processData,
                progressElem: controls.resultCount
            });

            xhr.send();
        }
        catch (ex) {
            alert("The Dynatrace API could not be called: " + ex.message);
        }
    }

    function init(url, token) {
        baseURL  = url;
        apiToken = token;
    }


    // Now we create the actual UI.
    container.innerHTML = "";

    var configArea = document.createElement("div");
    configArea.id = "entityFilterArea";
    container.appendChild(configArea);
 
    var listArea = document.createElement("div");
    listArea.id = "availableList";
    container.appendChild(listArea);

    var topLabel = document.createElement("span");
    topLabel.appendChild(document.createTextNode("Get "));

    var entityTypeText = document.createElement("span");
    entityTypeText.id = "entityTypeText";
    topLabel.appendChild(entityTypeText);
    topLabel.appendChild(document.createTextNode(" from "));

    configArea.appendChild(topLabel);

    // TODO: Make this plain text, following what the JobEditor says.
    var mgtZoneText = document.createElement("span");
    mgtZoneText.id = "mgtZoneText";

    configArea.appendChild(mgtZoneText);

    var refreshButton = document.createElement("input");
    refreshButton.type = "button";
    refreshButton.value = "Get";
    refreshButton.addEventListener("click", getData);

    configArea.appendChild(refreshButton);

    var resultCount = document.createElement("span");
    resultCount.id = "resultCount";

    configArea.appendChild(resultCount);
    
    var filterToggle = document.createElement("div");
    filterToggle.className = "FilterToggle";
    filterToggle.style.marginTop = "20px";

    var filterToggleControl = document.createElement("span");
    filterToggleControl.id = "filterToggleControl";
    filterToggleControl.appendChild(document.createTextNode("Filter"));

    filterToggle.appendChild(filterToggleControl);
    configArea.appendChild(filterToggle);

    var filterArea = document.createElement("div");
    filterArea.id = "filterArea";
    
    var filterMatches = document.createElement("span");
    filterMatches.id = "filterMatches";

    filterArea.appendChild(filterMatches);

    var filterBoxes = document.createElement("div");
    filterBoxes.id = "filterBoxes";

    filterArea.appendChild(filterBoxes);
    configArea.appendChild(filterArea);

    var availableToggle = document.createElement("div");
    availableToggle.className = "AvailableToggle";

    var avaiableToggleControl = document.createElement("span");
    avaiableToggleControl.className = "AvaiableToggle";
    avaiableToggleControl.innerHTML = "Available";

    availableToggle.appendChild(avaiableToggleControl);
    configArea.appendChild(availableToggle);

    var availableControls = document.createElement("div");
    availableControls.className = "AvailableControls";

    var addToJobControl = document.createElement("input");
    addToJobControl.type = "button";
    addToJobControl.value = "Add to Job";
    addToJobControl.addEventListener("click", function (evt) {
        emit("addtojob", controls.availableList.selectedItems);
    });

    availableControls.appendChild(addToJobControl);

    var rightCount = document.createElement("span");
    rightCount.id = "rightCount";

    availableControls.appendChild(rightCount);
    configArea.appendChild(availableControls);

    adjustBottomHeightOnTopChange(configArea, listArea);

    // Collect all controls on the page for quick reference later.
    controls = {
        entityTypeText:       entityTypeText,
        mgtZoneText:          mgtZoneText,
        resultCount:          resultCount,
        filterMatches:        filterMatches,
        filterToggleControl:  filterToggleControl,
        filterControl:        new FilterControl(filterBoxes),
        availableList:        new ListTable(listArea, { selectionControls: rightCount })
    };

    controls.filterControl.addEventListener("change", onFilterChange);

    // Return the interface.
    // Events to expose: 'onaddtojob' or something.
    return {
        on:    subscribe,
        init:  init,
        reset: function (scope) {
            entityType = scope.type.value;
            controls.entityTypeText.innerHTML = scope.type.label + "s";

            mgtZone = scope.zone.value;
            controls.mgtZoneText.innerHTML = scope.zone.label;

            fullList     = [];
            filteredList = [];

            controls.availableList.specials = [];
            controls.availableList.items    = filteredList;  
        },
        update: function (specials) {
            controls.availableList.specials = specials;
            controls.availableList.items    = filteredList;  
        },
        select: function (items) {
            controls.availableList.selectedItems = items;
        }
    };
    
}
