
function FilterControl (container, config) {
    config = config || {};

    var self = this;
    var menu = new Menu();
    var edit = new Edit();
    var inlineEdit = new InlineEdit();
    
    // TODO: No empty text set means don't allow empty values at all.
    this.config  = {
        multiCriteria: config.hasOwnProperty("multiCriteria") ? !!config.multiCriteria    : true, 
        minValues:     config.hasOwnProperty("minValues")     ?  Number(config.minValues) : 0,
        maxValues:     config.hasOwnProperty("maxValues")     ?  Number(config.maxValues) : 1000,
        negatable:     config.hasOwnProperty("negatable")     ? !!config.negatable        : true,
        editable:      config.hasOwnProperty("editable")      ? !!config.editable         : false,
        comparators:   config.hasOwnProperty("comparators")   ?   config.comparators      : null,
        emptytext:     config.hasOwnProperty("emptytext")     ?   config.emptytext        : "(exists)",
        allowAlias:    config.hasOwnProperty("allowAlias")    ? !!config.allowAlias       : false,
        tooltip:       config.tooltip || false
    };
    this.config.delimiter = this.config.negatable ? "=" : "";

    this.options = null;
    this.values  = null;
    
    // Whenever something changes, we raise an event to the subscriber(s).
    this.changeHandler = function () {};
    
    function onChange(noEvent) {
        if (noEvent) return;
        setTimeout(self.changeHandler, 1, {
            target: container,
            value:  getFilterValue()
        });
    }
       
    function setSelectedKeys() {
        var keys = Array
            .from(container.getElementsByClassName("CriterionControl"))
            .map(function (critCtl) {
                return critCtl.getAttribute("key");
            })
            .join(",");

        container.setAttribute("keys", keys);    
    }
    
    function getKeyOptions() {
        var keys = container.getAttribute("keys").split(",");    

        var options = self.options
            .filter(function (option){
                return !keys.includes(option.key);
            })
            .map(function (option) {
                return {
                    value: option.key,
                    name:  option.name
                };
            });
        return options;
    }       

    function getValueLabel(value) {
        for (var i=0; i < self.options.length; i++)
            if (self.options[i].value == value)
                return self.options[i].name;
        return value;
    }

    function isKeyAllowed(key) {
        for (var i=0; i < self.options.length; i++) {
            if (self.options[i].key === key) {
                return true;
            }
        }
        return false;
    }
    
    function isValueAllowed(key, value) {       
        let val = value
            .split("=")[0]      // Remove alias after '='.
            .split("|")[0];     // Get first part where "DATETIME|HH:mm|5m" for instance.

        for (var i=0; i < self.options.length; i++) {
            if (self.options[i].key === key) {
                if (!self.options[i].values)    // No constraints?
                    return true;                // Then its allowed.
                for (var j=0; j < self.options[i].values.length; j++) {
                    if (self.options[i].values[j].split("|")[0] === val) {
                        return true;
                    }
                }   
            }
        }
        return false;
    }   
    
    function setSelectedValues(critControl) {
        var values = Array
            .from(critControl.getElementsByClassName("ValueSelector"))
            .map(function (valSel) {
                return valSel.getAttribute("value");
            })
            .join(",");

        critControl.setAttribute("values", values);

        // Now let's see if the user can still add more values.
        // Make sure the '+' button is visible/hidden accordingly.
        let key = critControl.getAttribute("key");
        critControl.lastElementChild.style.display = getValueOptions(key, values).length == 0 && 
                                                     !self.config.editable && 
                                                      self.config.minValues <= values.length
                                                   ?  "none"
                                                   : "";
    }
    
    function getValueOptions(key, valuesAlreadySelected, currentValue) {
        // Make sure it's an array, and ignore aliases.
        if (!Array.isArray(valuesAlreadySelected))
            valuesAlreadySelected = valuesAlreadySelected.split(",");
        valuesAlreadySelected = valuesAlreadySelected.map(value => value.split("=")[0]);

        var values = null;
        for (var i=0; !values && i < self.options.length; i++) {
            if (self.options[i].key == key)
                values = self.options[i].values;
        }

        return (values || [])
            .map(value => value.split("|")[0])  // A value may say: DATETIME|, in which case we need to add an edit control to collect some extra info
            .filter(value => !valuesAlreadySelected.includes(value) || value === currentValue);
    }

    function extrasForValue(key, value) {
        var values = null;
        for (var i=0; !values && i < self.options.length; i++) {
            if (self.options[i].key == key)
                values = self.options[i].values;
        }
        values = values || [];

        for (var i=0; i < values.length; i++) {
            var valueOptionParts = values[i].split("|");
            if (valueOptionParts[0] == value && valueOptionParts.length > 1)
                return valueOptionParts.slice(1);
        }
        return [];
    }
     
    function getComparatorOptions(value) {
        const defaultOptions = {
            minValues: self.config.minValues,
            maxValues: self.config.maxValues
        };

        if (!self.config.comparators || value == "unknown") 
            return defaultOptions;
        
        let options = self.config.comparators;
        if (value !== undefined) {
            for (var i=0; i < options.length; i++) {
                if (options[i].value == value || options[i].icon == value)
                    return Object.assign(defaultOptions, options[i]);
            }
            return defaultOptions;
        }
        return options;
    }

    var showAliasEditor = function (evt) {
        var control = evt.currentTarget;
        if (control.classList.contains("KeySelector") &&
            control.parentElement.getElementsByClassName("ValueSelector").length > 0) {
            return;     // If there are values, there's no alias on the key.
        }

        var offset = evt.currentTarget.classList.contains("ValueSelector") ? 6 : -3;

        menu.hide(control);
        edit.hide(control);

        var editControl = document.createElement("input");
        editControl.type = "text";
        editControl.className = "AliasEditor";
        editControl.style.marginLeft = "-" + (evt.currentTarget.offsetWidth - offset) + "px"; 
        editControl.value = control.dataset.alias || "";
        editControl.setAttribute("prevAlias", editControl.value);
        editControl.addEventListener("keydown", function (evt) {
            switch(evt.key) {
                case " ":       // We don't want spaces in our aliases. Treat as "Enter".
                case "Enter":
                    if (editControl.value.trim() != "")
                        control.dataset.alias = editControl.value.trim();
                    else
                        delete control.dataset.alias;

                    control.parentElement.style.marginBottom = control.dataset.alias ? "18px" : "";
                    // Fall through
                case "Escape":
                    try { editControl.parentElement.removeChild(editControl); }
                    catch (ex) { /* NOP - blur event may have removed edit control from DOM already. */ }

                    // Undo changes if escape key hit.
                    if (evt.key == "Escape") {
                        var prevValue = editControl.getAttribute("prevAlias");
                        if (prevValue)
                            control.dataset.alias = prevValue;
                        else
                            delete control.dataset.alias;

                        editControl.setAttribute("prevAlias", null);
                    }
                    // Fall through
                case "Backspace":
                case "Delete":
                    // We handled it - or at least we don't want browser to handle it.
                    evt.cancelBubble = true;
                    evt.preventDefault = true;
                    evt.stopImmediatePropagation();
                    evt.stopPropagation();
                    break;
                default:
                    break;
            }  
        });
        editControl.addEventListener("blur", function (evt) {
            if (!editControl.parentElement) return;
            if (editControl.value.trim() != "")
                control.dataset.alias = editControl.value.trim();
            else
                delete control.dataset.alias;

            control.parentElement.style.marginBottom = control.dataset.alias ? "18px" : "";                           
            editControl.parentElement.removeChild(editControl);
        });
        control.appendChild(editControl);
        editControl.focus();
    }

    var doSelectKey = function (evt) {
        var keyControl   = evt.target;
        var critControl  = keyControl.parentElement;
 
        var onKeySelected = function (ev) {
            var key = ev.originalTarget.getAttribute("value");
            
            if (keyControl.className === "AddCritSelector") {
                critControl = addCriterionControl({ target: keyControl }, { key: key, values: [] });
            }
            else {
                critControl.setAttribute("key", key);
                if (self.config.tooltip) critControl.setAttribute("tooltip", key.split("##")[0]);
                keyControl.className = "KeySelector" + " " + key.split("##")[1];
                keyControl.innerHTML = getValueLabel(key.split("##")[0]);
                
                // Clear the list of currently selected values on the criterion control.
                // TODO: Ideally, we might only clear if the key selection changed.
                Array
                    .from(critControl.getElementsByClassName("ValueSelector"))
                    .forEach(function (valSel) {
                        critControl.removeChild(valSel)
                    });

                // If we need to have at least one value and there is none, add the first available option.
                var minValues = critControl.getAttribute("minValueCount") || 0;
                var selectedValues = [];
                var allowedValues = getValueOptions(key, selectedValues);
                for (var i=0; minValues > selectedValues.length && i < allowedValues.length; i++) {
                    selectedValues.push(allowedValues[0]); // Use the next available (unused) value.

                    addValueControl({ target: critControl.lastElementChild }, allowedValues[0]);

                    allowedValues = getValueOptions(key, selectedValues);
                }
                critControl.setAttribute("values", selectedValues.join(","));
            }
            
            // Update the list of currently selected keys in the filter control.
            setSelectedKeys();
            onChange();
            
            critControl.lastElementChild.focus();
        };
        
        var availableKeys = getKeyOptions();
        menu.show(keyControl, availableKeys, onKeySelected);             
    }
      
    var doSelectValue = function (evt) {
        var valueControl = evt.target.className === "AddValueSelector"
                         ? evt.target
                         : evt.target.parentElement.classList.contains("ValueSelector")
                         ? evt.target.parentElement
                         : evt.target.parentElement.parentElement;
        var critControl  = valueControl.parentElement;

        if (!critControl.classList.contains("CriterionControl")) 
            return; // Inadvertently percolated event - ignore.
         
        var onValueSelected = function (ev) {
            var value = ev.originalTarget.getAttribute("value");
            
            /*  Set the 'value' chosen from the dropdown on this element, so that
                this ValueControl (ValueSelector) can be queried for its value.
                If the value was selected after clicking the '+' button, we need
                to create a new value control instead.
             */
            if (valueControl.className === "AddValueSelector") {
                valueControl = addValueControl({ target: valueControl }, value);
            }
            else {
                valueControl.setAttribute("value", value);
                
                // The menu may still be present in the DOM.
                var valueText = valueControl.lastElementChild;
                if (valueText.classList.contains("MenuShield"))
                    valueText = valueText.previousElementSibling;
                     
                valueText.innerHTML = value;         // Encode??

                var key = critControl.getAttribute("key");
                inlineEdit.remove(valueText);
                var extras = extrasForValue(key, value);
                extras.forEach((extra, i) => inlineEdit.add(valueText, extra, i));
            }
            
            // Set the list of currently selected values on the criterion control.
            setSelectedValues(critControl);
            onChange();
        };
        
        var availableValues = getValueOptions(
                                critControl.getAttribute("key"),
                                critControl.getAttribute("values").split(","),
                                valueControl.getAttribute("value")
                              );

        if (availableValues.length == 0 && self.config.editable) 
            edit.show(valueControl, valueControl.getAttribute("value") || "", onValueSelected);
        else
            menu.show(valueControl, availableValues, onValueSelected);
    }
    
    function removeValueControl(evt) {
        var valueControl = evt.target.parentElement;
        var critControl  = valueControl.parentElement;
                
        // Set the focus on one of the remaining values, or the '+' button.
        var newFocusControl = valueControl.previousElementSibling.classList.contains("InvCritSelector")
                            ? valueControl.nextElementSibling
                            : valueControl.previousElementSibling;
        
        critControl.removeChild(valueControl);
        newFocusControl.focus();

        // Set the list of currently selected values on the criterion control.
        setSelectedValues(critControl);
        onChange();        
    }
    
    function addValueControl(evt, value) {
        // evt.target is always presumed to be the '+' button.
        var critControl = evt.target.parentElement;
        
        var valueControl = document.createElement("span");
        valueControl.className = "ValueSelector";

        var valueParts = value.split("=");
        var valueAspects = valueParts[0].split("|"); // In case there are extra elems of the value: DATETIME|HH:mm|5m
        valueControl.setAttribute("value", valueAspects[0]);

        if (self.config.allowAlias) {
            if (valueParts[1])
                valueControl.dataset.alias = valueParts[1]
            valueControl.addEventListener("dblclick", showAliasEditor);
        }
        var remValButton = document.createElement("span");
        remValButton.className = "RemValButton";
        remValButton.addEventListener("click", removeValueControl);
        remValButton.innerHTML =  "&#215;";
        valueControl.appendChild(remValButton);
        
        var valSelector = document.createElement("span");
        valSelector.className = "Value";
        valSelector.addEventListener("click", doSelectValue);
        valSelector.innerHTML = valueAspects[0]; // Mar 6, 2019: This is what it was (removed becasue of italics): escapeHTMLChars(valueAspects[0]);
        valueControl.appendChild(valSelector);

        var key = critControl.getAttribute("key");
        var extras = extrasForValue(key, valueAspects[0]);
        extras.forEach((extra, i) => {
            valSelector.dataset["extra" + i] = valueAspects[i+1] || "";
            inlineEdit.add(valSelector, extra, i); 
        });

        critControl.insertBefore(valueControl, evt.target);
        // We added a value control. That kills any alias on the key selector.
        delete critControl.querySelector(".KeySelector").dataset.alias;
        
        valueControl.setAttribute('tabindex', '0');  // Allow focus.
        valueControl.addEventListener("keydown", function (ev) {
            // Don't respond to keys except for the ones that should trigger the menu,
            // and the "Delete" button.
            if (![" ", "ArrowDown", "ArrowUp", "Backspace", "Delete"].includes(ev.key)) return;
            
            if (ev.key === "Backspace" || ev.key === "Delete")
                removeValueControl({target: remValButton });
            else
                doSelectValue({ target: valSelector });

            // This may be overkill, but at least one of these is necessary...
            ev.cancelBubble = true;
            ev.preventDefault = true;
            ev.stopImmediatePropagation();
            ev.stopPropagation();
        });

        // If an alias is set, claim some extra height to show the alias notch.
        valueControl.parentElement.style.marginBottom = valueControl.dataset.alias ? "18px" : "";
        return valueControl;
    }
    
    function removeCriterionControl(evt) {
        // Don't delete a criterion if there can (and must) be only one.
        if (!self.config.multiCriteria) return;
        
        var critControl = evt.target.parentElement;
        
        // Set the focus on one of the remaining values, or the '+' button.
        var newFocusControl = critControl.nextElementSibling
                            ? critControl.nextElementSibling
                            : critControl.previousElementSibling;

        critControl.parentElement.removeChild(critControl);
        newFocusControl.focus();
     
        // Update the list of currently selected keys in the filter control.
        setSelectedKeys();
        onChange();
    }
    
    function doSelectComparator(evt) {
        // Show a menu with the options available, if available.
        if (!self.config.comparators) {
            // Negate, but don't negate if it is not allowed.
            if (!self.config.negatable) return;
            
            var invCritSelector = evt.target;
            var critControl = invCritSelector.parentElement;
            
            var isNegated = critControl.getAttribute("negated") == "true";
            isNegated = !isNegated;
            critControl.setAttribute("negated", isNegated);
            
            critControl.className     = "CriterionControl"
                                    + (isNegated ? " Negated" : "");
            invCritSelector.innerHTML = isNegated
                                    ? "&#8800;"
                                    : self.config.delimiter;
            onChange();        
            return;    
        }

        var comparatorSelector = evt.target;
        var critControl = comparatorSelector.parentElement;
         
        var onComparatorSelected = function (ev) {
            var icon  = ev.originalTarget.getAttribute("icon");
            var value = ev.originalTarget.getAttribute("value");

            critControl.setAttribute("comparator", value);
            comparatorSelector.setAttribute("value", value);                
            comparatorSelector.innerHTML = icon || value;                

            // Set an attribute that says how many values we now need or allow.
            var valueObject = getComparatorOptions(value);

            critControl.setAttribute("minValueCount", valueObject.minValues);
            critControl.setAttribute("maxValueCount", valueObject.maxValues);

            // Also ensure that we have the allowed number of value controls.
            var valueSelectors = critControl.querySelectorAll(".ValueSelector");

            // Create additional ones. Disallow deletion beyond the minimum number.
            var createCount = valueObject.minValues - valueSelectors.length;
            for (var i=0; i < createCount; i++) {
                addValueControl({ target: critControl.lastElementChild }, "");
            };
            
            // Delete superfluous ones. Disallow creation beyond the maximum number.
            var deleteCount = valueSelectors.length - valueObject.maxValues;
            for (var i=0; i < deleteCount; i++) {
                removeValueControl({ target: critControl.lastElementChild.previousElementSibling.firstElementChild });
            }

            onChange();
        };
        
        var availableValues = getComparatorOptions();

        menu.show(comparatorSelector, availableValues, onComparatorSelected, true);
    }
    
    function addCriterionControl(evt, criterion) {
        var container = evt.target.parentElement;
        /*
            criterion: 
            { type: "field", 
              name: "serviceType", 
              alias: "MyAlias",
              values: ["A", "B"],
              negated: false,
              comparator: "<"
            }
         */
        var critControl = document.createElement("span");
        critControl.className = "CriterionControl"
                              + (criterion.negated ? " Negated" : "");
        
        critControl.setAttribute("key",        criterion.key);
        if (self.config.tooltip) critControl.setAttribute("tooltip", criterion.key.split("##")[0]);
        critControl.setAttribute("negated",  !!criterion.negated);
        critControl.setAttribute("comparator", criterion.comparator || "=");
                
        // Let's see how many values this comparator allows us to have.
        // Set an attribute that says how many values we now need or allow.
        var compOptions = getComparatorOptions(criterion.comparator || "unknown");

        critControl.setAttribute("minValueCount", compOptions.minValues);
        critControl.setAttribute("maxValueCount", compOptions.maxValues);

        // If we need to have at least one value and there is none, add the first available option.
        // Note that this should never happen while we are here, because we're creating a criterion
        // from an 'external' value - it's not because the user just selected a criterion value!
        // But we'll do it anyway.
        var allowedValues = getValueOptions(criterion.key, criterion.values);
        for (var i=0; compOptions.minValues > criterion.values.length && i < allowedValues.length; i++) {
            criterion.values.push(allowedValues[0]); // Use the next available (unused) value.
            allowedValues = getValueOptions(criterion.key, criterion.values);
        }
        critControl.setAttribute("values", criterion.values.join(","));

        // Delete this criterion button
        var remCritSelector = document.createElement("span");
        remCritSelector.className = "RemCritSelector";
        remCritSelector.addEventListener("click", removeCriterionControl);
        remCritSelector.innerHTML = "&#215;";
        
        // TODO: This should be handled by CSS
        if (!self.config.multiCriteria) {
            remCritSelector.style.display = "none";
        }
        // TODO: Also may an option to not being able to remove it?
        critControl.appendChild(remCritSelector);
        
        // Key for the criterion
        var keySelector = document.createElement("span");
        keySelector.className = "KeySelector" + " " + criterion.key.split("##")[1];
        keySelector.addEventListener("click", doSelectKey);

        if (self.config.allowAlias) {
            if (criterion.alias) {
                keySelector.dataset.alias = criterion.alias;
                critControl.style.marginBottom = "18px";
            }
            keySelector.addEventListener("dblclick", showAliasEditor);
        }

        keySelector.setAttribute('tabindex', '0'); // Allow focus
        keySelector.innerHTML = getValueLabel(criterion.key.split("##")[0]);

        keySelector.addEventListener("keydown", function (ev) {
            // Don't respond to keys except for the ones that should trigger the menu,
            // and the "Delete" button.
            if (![" ", "ArrowDown", "ArrowUp", "Backspace", "Delete"].includes(ev.key)) return;
            
            if (ev.key === "Backspace" || ev.key === "Delete")
                removeCriterionControl({target: remCritSelector });
            else
                doSelectKey({ target: keySelector });

            // This may be overkill, but at least one of these is necessary...
            ev.cancelBubble = true;
            ev.preventDefault = true;
            ev.stopImmediatePropagation();
            ev.stopPropagation();
        });
    
        critControl.appendChild(keySelector);

        // Comparator selector, previously called the 'Invert this criterion'-button.
        var invCritSelector = document.createElement("span");
        invCritSelector.className = "InvCritSelector";
        invCritSelector.addEventListener("click", doSelectComparator);
        invCritSelector.innerHTML = compOptions.icon
                                  ? compOptions.icon
                                  : criterion.negated
                                  ? "&#8800;"
                                  : self.config.delimiter;
        
        if (!self.config.negatable && !self.config.comparators)
            invCritSelector.classList.add("Disabled");
        
        critControl.appendChild(invCritSelector);

        var addValueSelector = document.createElement("span");
        addValueSelector.className = "AddValueSelector";
        addValueSelector.dataset.emptytext = self.config.emptytext;
        addValueSelector.innerHTML = "&#43;";
        addValueSelector.setAttribute('tabindex', '0'); // Allow focus.
        addValueSelector.addEventListener("click", doSelectValue);
        addValueSelector.addEventListener("keydown", function (ev) {
            // Don't respond to keys except for the ones that should trigger the menu.
            if (![" ", "ArrowDown", "ArrowUp", "Enter"].includes(ev.key)) return;

            doSelectValue({ target: addValueSelector });
            
            // This may be overkill, but at least one of these is necessary...
            ev.cancelBubble = true;
            ev.preventDefault = true;
            ev.stopImmediatePropagation();
            ev.stopPropagation();
        });
        
        critControl.appendChild(addValueSelector);
        
        // Add values for the criterion.
        criterion.values.forEach(function (value) {
            if (isValueAllowed(criterion.key, value))
                addValueControl({ target: addValueSelector }, value);
        });
                
        // The last child is always a '+' button - but its visibility depends on whether we have selectable options.
        container.insertBefore(critControl, container.lastElementChild);
        
        // Now let's see if the user can still add more values.
        // Make sure the '+' button is visible/hidden accordingly.
        let key = critControl.getAttribute("key");
        critControl.lastElementChild.style.display = getValueOptions(key, criterion.values).length == 0 && 
                                                     !self.config.editable && 
                                                      self.config.minValues <= criterion.values.length
                                                   ?  "none"
                                                   : "";

        return critControl;
    }

    // Getter for the complete value of the whole control.
    function getFilterValue() {
        var value = Array
            .from(container.getElementsByClassName("CriterionControl"))
            .map(function (critControl) {
                var negated    = critControl.getAttribute("negated") == "true";
                var comparator = critControl.getAttribute("comparator") || "=";
                var key        = critControl.getAttribute("key").split("##");
                var values = Array
                    .from(critControl.getElementsByClassName("ValueSelector"))
                    .map(function (valSel) {
                        var value  = valSel.getAttribute("value");
                        var extras = extrasForValue(key.join("##"), value).map((extra, i) => valSel.querySelector(".Value").dataset["extra"+i]);
                        if (extras.length > 0)
                            value = value + "|" + extras.join("|");
                        return valSel.dataset.alias 
                              ? value + "=" + valSel.dataset.alias
                              : value;
                    });
 
                return { 
                    type:       key[1],
                    name:       getValueLabel(key[0]),
                    value:      key[0],
                    alias:      critControl.querySelector(".KeySelector").dataset.alias,
                    values:     values,
                    negated:    negated,
                    comparator: comparator
                };
            });
                
        return value;
    }
    
    // Setter for the complete value of the whole control.
    function setFilterValue(values, noEvent) {
        self.values = values.map(value => {
            value.key = (value.value || value.name) + "##" + (value.type || "");
            return value;
        });
        
/*  This needs to be revisited  it's about having at least one criterion (with or without default value).
       if (!self.config.minValues > 0 && self.options && self.options.length) {
            // Only one value allowed? Then create a default value.
            if (!self.values || !self.values.length)
                self.values = [{ 
                    key: self.options[0].key,
                    values: []  // TODO: If one value for a key is allowed, then select that, too (ValueControl)
                }];
        }
 */       
        // We should basically re-create the whole control, while
        // checking that the keys and values are actually allowed.
        container.innerHTML = "";
    
        // First, create the 'add criterion' button.
        var addCritSelector = document.createElement("span");
        addCritSelector.className = "AddCritSelector";
        addCritSelector.innerHTML = "&#43;";
        addCritSelector.addEventListener("click", doSelectKey);        
        addCritSelector.addEventListener("keydown", function (ev) {
            // Don't respond to keys except for the ones that should trigger the menu,
            // and the "Delete" button.
            if (![" ", "ArrowDown", "ArrowUp", "Backspace", "Delete"].includes(ev.key)) return;

            doSelectKey({ target: addCritSelector });

            // This may be overkill, but at least one of these is necessary...
            ev.cancelBubble = true;
            ev.preventDefault = true;
            ev.stopImmediatePropagation();
            ev.stopPropagation();
        });

        if (!self.config.multiCriteria)
            addCritSelector.style.display = "none";    // Don't even show it.
        
        container.appendChild(addCritSelector);
        
        // Next, add the keys and values already selected.
        self.values.forEach(function (value) {
            if (isKeyAllowed(value.key))
                addCriterionControl({ target: addCritSelector }, value);
        });

        // Finally, update the list of currently selected keys in the filter control.
        setSelectedKeys();
        onChange(noEvent);
    }
    
    // Setter for the allowed keys and values.
    /*  [
            { name: "field1", type: "Tag", values: [] },
            ...
        ]
     */
    function setFilterOptions(options) {
        options = options || [];

        // Create a unique key per type/name combination.
        self.options = options.map(function (option) {
            option.key = (option.value || option.name) + "##" + (option.type || "");
            
            return option;
        });
        
        // We will re-render the whole control, which will validate the values as well.
        setFilterValue(getFilterValue(), true);
    }
     
    // Helper functions are defined. 
    this.changeHandler  = container.onchange || function() {};
    
    // Return an abject to the caller at which the value can be accessed
    // and event listeners registered (maybe). 
    return {
        get value() {
            return getFilterValue();
        },
        set value(vals) {
            return setFilterValue(vals || [], true);
        },
        set options(opts) {
            container.style.display = opts ? "" : "none";

            setFilterOptions(opts);
        },
        set onchange(handler) {
            self.changeHandler = handler;
        },
        addEventListener: function (evt, handler) {
            if (evt === "change") 
                self.changeHandler = handler;
            else
                container.addEventListener(evt, handler);
        }
    };
}
