FilterList = function (container, config, onChange) {
    const self = this;

    this.container = typeof container === "string"
                   ? document.getElementById(container)
                   : container;
    config = config || {};

    this.selectionControls = config.selectionControls
                           ? ( typeof config.selectionControls === "string"
                                ? document.getElementById(config.selectionControls)
                                : config.selectionControls
                           )
                           : { innerHTML: null };

    this.list    = [];
    this.options = [];
    let filters  = [];

    function selectText(node) {
        node = document.getElementById(node);

        if (document.body.createTextRange) {
            const range = document.body.createTextRange();
            range.moveToElementText(node);
            range.select();
        }
        else if (window.getSelection) {
            const selection = window.getSelection();
            const range = document.createRange();
            range.selectNodeContents(node);
            selection.removeAllRanges();
            selection.addRange(range);
        }
        else {
            console.warn("Could not select text in node: Unsupported browser.");
        }
    }

    function setFilterOptions(opts) {
        self.options = opts;

        // Apply the new options to all available filters.
        filters.forEach(filter => filter.options = self.options);
    }

    function selectFilter(evt) {
        var filterBoxes = self.container.children;
        for (var i=0; i < filterBoxes.length; i++) {
            filterBoxes[i].classList.remove("Selected");
        }
        evt.currentTarget.classList.add("Selected");

        let idx = Number(evt.currentTarget.getAttribute("idx"));
        onChange({
            target: evt.currentTarget,
            value:  isNaN(idx) ? [] : filters[idx].value
        });
    }

    function addFilter(item, i) {
        var filterBoxes = document.createElement("div");
        filterBoxes.className = "Filter";
        filterBoxes.addEventListener("click", selectFilter)
        filterBoxes.addEventListener("change", onChange);
        filterBoxes.setAttribute("idx", i);

        self.container.appendChild(filterBoxes);

        var filterControl = new FilterControl(filterBoxes,  { negatable: true,  emptytext: "(no value)", minValues: 0 });
        filterControl.options = self.options;
        filterControl.value = item || null;

        return filterControl;
    }

    function renderItems(list) {
        filters   = [];
        self.list = list || self.list;

        self.container.className = "FilterList";

        if (!document.getElementById("AddFilterToListButton")) {
            var addControl = document.createElement("input");
            addControl.id = "AddFilterToListButton"
            addControl.type = "button";
            addControl.value = "New Filter";
            addControl.style["float"] = "right";
            addControl.style.marginTop = "-23px";
            addControl.addEventListener("click", function (evt) {
                filters.push(addFilter(null, filters.length));
            });
            self.container.appendChild(addControl);

            var includeRelatedControl = document.createElement("span");
            includeRelatedControl.className = "DropdownControl Entities";
            includeRelatedControl.innerHTML = '<select id="includeWhat2" style="position:relative; top:-24px"><option value="none">Matching entities only</option><option value="flows">Include flows</option><option value="stack">Include stacks</option></select>';

            self.container.appendChild(includeRelatedControl);

            // Keep this in sync
            document.getElementById("includeWhat2").value = document.getElementById("includeWhat").value;
            document.getElementById("includeWhat2").addEventListener("change", function () {
                document.getElementById("includeWhat").value = this.value;
            });
        }

        // Remove old filters.
        let filterElements = self.container.querySelectorAll(".Filter");
        for (var i=0; i < filterElements.length; i++)
            filterElements[i].parentElement.removeChild(filterElements[i]);

        self.list.forEach(function (item, i) {
            filters.push(addFilter(item, i));
        });
    }

    function unrenderItems() {
        return filters.map(filter => filter.value).filter(value => value.length > 0);
    }

    return {
        render: renderItems,
        set options(opts) {
            setFilterOptions(opts);
        },
        set items(items) {
            renderItems(items || []);
        },
        get items() {
            return unrenderItems();
        },
        get selectedItem() {
            var selectedBox = self.container.querySelector(".Selected");
            if (!selectedBox) return null;

            let idx = Number(selectedBox.getAttribute("idx"));
            return {
                target: selectedBox,
                value:  isNaN(idx) ? [] : filters[idx].value
            };
        }
    };
}