JSONHttpRequest = function (config) {
    var method            = config.method || "GET", 
        address           = config.address,
        onCompleteHandler = config.onComplete, 
        onErrorHandler    = config.onError,
        progressElem      = config.progressElem || {};
    
    var handleError = function (ex) {
        console.log("JSONHttpRequest error: " + ex.message + ". URL: " + address);
        
        progressElem.innerHTML = ex.message;
        
        if (onErrorHandler)
            onErrorHandler(ex);
        onErrorHandler = null;
    };
    
    var xhr = new XMLHttpRequest();

    if (address.indexOf("http") !== 0) {
        xhr.open(method, address);
    }
    else if ("withCredentials" in xhr) {
        // XHR for Chrome/Firefox/Opera/Safari.
        xhr.open(method, address, true);
    } 
    else if (typeof XDomainRequest != "undefined") {
        // XDomainRequest for IE.
        xhr = new XDomainRequest();
        xhr.open(method, address);
    } 
    else {
        // CORS not supported.
        throw new Error("CORS not supported");
    }
    
    xhr.onprogress = function (evt) {
        if (evt.lengthComputable)
            progressElem.innerHTML = "Loading... " 
                + Math.round(100 * (evt.loaded / evt.total)) + "%";
        else
            progressElem.innerHTML = "Loading... ";
    };

    xhr.onload = function() {
        progressElem.innerHTML = "Analyzing... ";

        if (xhr.status == 200)
            setTimeout(onCompleteHandler, 1, JSON.parse(xhr.responseText));
        else {
            try {
                var error = (JSON.parse(xhr.responseText || "") || {}).error;
            
                handleError({ message: xhr.statusText
                            + (error && error.message ? (": " + error.message) : typeof error === "string" ? (": " + error) : "") });
            }
            catch (ex) {
                handleError({ message: xhr.statusText  + " (response is not JSON)" });
            }
        }
    };
        
    xhr.onerror = function (ex) {
        handleError({ message: "Request failed: " + xhr.status });
    };

    xhr.ontimeout = function() {
        handleError({ message: "Request timed out" });
    };

    xhr.setRequestHeader("Content-Type", "application/json");
    
    progressElem.innerHTML = "Requesting data... ";

    return xhr;
}


function setCookie(name, value, days) {
    var expires = "";
    if (days) {
        var date = new Date();
        date.setTime(date.getTime() + (days*24*60*60*1000));
        expires = "; expires=" + date.toUTCString();
    }
    document.cookie = name + "=" + (value || "")  + expires + "; path=/";
}
function getCookie(name) {
    var nameEQ = name + "=";
    var ca = document.cookie.split(';');
    for(var i=0;i < ca.length;i++) {
        var c = ca[i];
        while (c.charAt(0)==' ') c = c.substring(1,c.length);
        if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length,c.length);
    }
    return null;
}
function eraseCookie(name) {   
    document.cookie = name+'=; Max-Age=-99999999;';  
}