ListTable = function (container, config) {
    const self = this;
    
    this.container = typeof container === "string"
                   ? document.getElementById(container)
                   : container;
    config = config || {};

    this.selectionControls = config.selectionControls 
                           ? ( typeof config.selectionControls === "string"
                                ? document.getElementById(config.selectionControls)
                                : config.selectionControls
                           )
                           : { innerHTML: null };

    this.list = [];
    this.specialIds = [];
    this.selectedCount = 0;

    function selectText(node) {
        node = document.getElementById(node);
    
        if (document.body.createTextRange) {
            const range = document.body.createTextRange();
            range.moveToElementText(node);
            range.select();
        } 
        else if (window.getSelection) {
            const selection = window.getSelection();
            const range = document.createRange();
            range.selectNodeContents(node);
            selection.removeAllRanges();
            selection.addRange(range);
        } 
        else {
            console.warn("Could not select text in node: Unsupported browser.");
        }
    }

    function updateSelectedItemCount() {
        var newSelectedCount = self.container.querySelectorAll("tr.Selected").length;
        self.selectedCount   = newSelectedCount;
        self.selectedCountElem.innerHTML = newSelectedCount + " selected";
    }

    function selectAllItems() {
        Array.from(self.container.querySelectorAll("tr"))
             .forEach(tr => tr.classList.add("Selected"));

        updateSelectedItemCount();
    }

    function invertSelection() {
        Array.from(self.container.querySelectorAll("tr"))
             .forEach(tr => tr.classList.toggle("Selected"));
        
        updateSelectedItemCount();
    }

    function selectNoItems() {
        Array.from(self.container.querySelectorAll("tr"))
             .forEach(tr => tr.classList.remove("Selected"));
        
        updateSelectedItemCount();
    }

    function renderFromTemplate(html, options) {
        var re = /<%([^%>]+)?%>/g, reExp = /(^( )?(if|for|else|switch|case|break|{|}))(.*)?/g, code = 'var r=[];\n', cursor = 0, match;
        var add = function(line, js) {
            js? (code += line.match(reExp) ? line + '\n' : 'r.push(' + line + ');\n') :
                (code += line != '' ? 'r.push("' + line.replace(/"/g, '\\"') + '");\n' : '');
            return add;
        }
        while(match = re.exec(html)) {
            add(html.slice(cursor, match.index))(match[1], true);
            cursor = match.index + match[0].length;
        }
        add(html.substr(cursor, html.length - cursor));
        code += 'return r.join("");';
        //  var debugCode = code.replace(/[\r\t\n]/g, '');
        // console.log("Code: " + debugCode);
        return new Function(code.replace(/[\r\t\n]/g, '')).apply(options);
    }   
    
    function hideTooptip(evt) {
        TooltipManager.hide(); 
    }

    function showTooltip(evt) {
        const index    = evt.target.parentElement.getAttribute('data-i');
        const entityId = evt.target.parentElement.getAttribute('data-id');
        if (!entityId || !self.tooltipTemplate) return; 

        const entity = renderedList[index];
        if (entity.entityId != entityId)
            debugger;

        const rect = evt.target.getBoundingClientRect();
        const descriptor = {
            element: {
                x : rect.left,
                y : rect.top,
                h : rect.height,
                w : rect.width
            },
            offset: {
                x : this.id === "selectedList" ? 8 : -8,
                y : this.id === "selectedList" ? -6 : -6
            },
            max: {
                h : 400,
                w : 400
            },
            position: this.id === "selectedList" ? "middle-after" : "middle-before",
            contents: renderFromTemplate(self.tooltipTemplate, config.helper ? config.helper.getItem(entityId) || entity : entity),
            notch: true
        };
    
        let content = TooltipManager.show(null, descriptor); 
        if (content) {
            let tagArea = content.lastElementChild.classList.contains("TagArea") 
                        ? content.lastElementChild
                        : content.lastElementChild.previousElementSibling;
            tagArea.style.height = content.offsetHeight - tagArea.offsetTop - 20 + "px";
        }
    }

    let lastClickedRow = null;
    function selectRow(evt) {
        let thisClickedRow = evt.currentTarget;

        if (evt.shiftKey && lastClickedRow) {
            // Walk from here back to last clicked row and keep selecting.
            var thisID   = Number(thisClickedRow.getAttribute('data-i'));
            var lastID   = Number(lastClickedRow.getAttribute('data-i'));
            let iterator = thisID > lastID
                         ? "nextElementSibling"
                         : "previousElementSibling";

            while (lastClickedRow && 
                   thisID !== Number(lastClickedRow.getAttribute('data-i'))) {
                lastClickedRow.classList.add("Selected");
                lastClickedRow = lastClickedRow[iterator];
            } 
            thisClickedRow.classList.add("Selected");
        }
        else {
            thisClickedRow.classList.toggle("Selected");
        }

        lastClickedRow = thisClickedRow;
        updateSelectedItemCount();

        // Prevent text selection.
        if (window.getSelection) 
            window.getSelection().removeAllRanges();
        else if (document.selection) 
            document.selection.empty();
        else
            self.selectionControls.focus();
    }  

    function selectRows(idList) {
        Array.from(self.container.querySelectorAll("tr"))
             .forEach(tr => tr.classList.remove("Selected"));

        Array.from(self.container.querySelectorAll("tr"))
             .filter( tr => { return idList.includes(tr.getAttribute('data-id')) })
             .forEach(tr => tr.classList.add("Selected"));  

        updateSelectedItemCount();

        // Scroll the first selected row into view if it isn't already;
        let firstSelectedRow = self.container.querySelector("tr.Selected");
        if (firstSelectedRow) firstSelectedRow.scrollIntoView({ behavior: "smooth" });
    }
     
    function getSelectedRows() {
        return Array
            .from(self.container.querySelectorAll("tr.Selected"))
            .map(tr => Number(tr.getAttribute('data-i')))
            .filter(index => renderedList[index])
            .map(index => renderedList[index]);
    }

    let renderedList = [];
    function renderItems(list) {
        renderedList = list || self.list;

        self.container.innerHTML = "";
        var table = ["<table>"];
        renderedList.forEach(function (item, i) {
            table.push("<tr class='" + (self.specialIds.includes(item.entityId) ? "Included" : "")
                       + "' data-i='" + i
                       + "' data-id='" + item.entityId
                       + "' >");

            if (activeColumns.length > 1)
                activeColumns.forEach(function (colName, i) {
                    table.push("<td" + (i==0 ? " width='75%'" : "") 
                                + " title='" + item[colName]  + "'"
                                + ">" + escapeHTMLChars(item[colName]) + "</td>");
                });
            else
                activeColumns.forEach(function (colName, i) {
                    table.push("<td width='100%'" 
                                + " title='" + item[colName]  + "'"
                                + ">" + escapeHTMLChars(item[colName]) + "</td>");
                });

            table.push("</tr>");  
        });
        table.push( "</table>");
        self.container.innerHTML = table.join("");       

        // Add 'mouseover' handler. It will show a tooltip when the user moves over an entity.
        self.container.addEventListener("mouseover",  showTooltip);
        self.container.addEventListener("mouseleave", hideTooptip);

        // Add 'click' handler. It will also handle 'Ctrl' and 'Shift' keys.
        Array.from(self.container.querySelectorAll("tr"))
             .forEach(tr => tr.addEventListener("click", selectRow));

        updateSelectedItemCount();
    };
    
    // Create selection controls.
    self.selectionControls.innerHTML = "";

    self.selectedCountElem = document.createElement("span");
    self.selectedCountElem.className = "ListCount";
    self.selectedCountElem.innerHTML = "&nbsp;";
    
    self.selectionControls.appendChild(self.selectedCountElem);

    var selectAllControl = document.createElement("span");
    selectAllControl.className = "icon-radio-checked";
    selectAllControl.addEventListener("click", selectAllItems);
    selectAllControl.innerHTML = "&nbsp;";

    self.selectionControls.appendChild(selectAllControl);

    var invertSelControl = document.createElement("span");
    invertSelControl.className = "icon-contrast";
    invertSelControl.addEventListener("click", invertSelection);
    invertSelControl.innerHTML = "&nbsp;";
    
    self.selectionControls.appendChild(invertSelControl);

    var selectNoneControl = document.createElement("span");
    selectNoneControl.className = "icon-radio-unchecked";
    selectNoneControl.addEventListener("click", selectNoItems);
    selectNoneControl.innerHTML = "&nbsp;";
    
    self.selectionControls.appendChild(selectNoneControl);

/*  A beautiful enhancement for the future...
    self.searchControl = document.createElement("span");
    self.searchControl.className = "SearchControl";
    self.searchControl.innerHTML = "search";
    self.selectionControls.appendChild(self.searchControl);
 */
    return {
        render: renderItems,
        get selectedItems() {
            return getSelectedRows();
        },
        set selectedItems(items) {
            selectRows(items || []);
        },
        set specials(ids) {
            self.specialIds = ids || [];
            renderItems();
        },
        set items(items) {
            self.list = items || [];
            self.list.sort((itemA, itemB) => {
                const nameA = itemA.displayName.toLowerCase();
                const nameB = itemB.displayName.toLowerCase();
                return nameA < nameB ? -1 : nameA > nameB ? 1 : 0
            });
            renderItems();
        },
        getItem: function (itemId) {
            for (let i=0; i < self.list.length; i++) {
                if (self.list[i].entityId === itemId)
                    return self.list[i];
            }
            return null;
        },
        get items() {
            return self.list || [];
        },
        set tooltipTemplate(value) {
            self.tooltipTemplate = value;
        },
        search: function (elem) {
            value = elem.value.toLowerCase();
            renderItems(self.list.filter(item => item.displayName.toLowerCase().includes(value)));
        }
    };
}