function Menu () {

    var shield = null;
    var menu   = null;
    
    var selectElement  = null;
    var selectHandler  = null;
    var selectedOption = null;
    var currentValue   = null;
    var filterString   = null;

    var keyDownHandler = function (ev) {
        if (!menu) return;
        
        switch(ev.key) {
            case "ArrowRight":
            case "ArrowDown":
                if (selectedOption) {
                    selectedOption.classList.remove("Selected");
                    selectedOption = selectedOption.nextElementSibling || selectedOption;
                }
                else {        
                    selectedOption = menu.firstElementChild;
                }
                selectedOption.classList.add("Selected");
                break;
            case "ArrowLeft":
            case "ArrowUp":
                if (selectedOption) {
                    selectedOption.classList.remove("Selected");
                    selectedOption = selectedOption.previousElementSibling || selectedOption;
                }
                else {        
                    selectedOption = menu.firstElementChild;
                }
                selectedOption.classList.add("Selected");
                break;
            case "Enter":
                selectedOption = selectedOption || menu.firstElementChild;
                
                if (selectedOption.getAttribute("value") !== "")
                    selectHandler({ target: selectElement, originalTarget: selectedOption });
                
                hideMenu(ev);
                break;
            case "Escape":
                hideMenu(ev);
                break;
            case "Backspace":
            case "Delete":
                filterString = filterString.slice(0, -1);
                createOptions();
                break;
            default:
                if (ev.key.length == 1) {
                    filterString += ev.key;
                    createOptions();
                }
                break;
        }
        
        // Make sure the browser itself does not do anything!
        ev.cancelBubble = true;
        ev.preventDefault = true;
        ev.stopImmediatePropagation();
        ev.stopPropagation();    
        
        return false;
    }

    function createOption(value) {
        var option = document.createElement("div");
        option.className = "MenuItem";
        
        if (value) {
            var valueString = typeof(value.value) === "string" 
                            ? value.value
                            : typeof(value) === "string" 
                            ? value
                            : "";
            if (valueString === currentValue)
                option.classList.add("Preselected");
            
            var htmlValue = valueString.split("##")[0];
            if (htmlValue.charAt(0) != '<')  // Unless deliberate HTML, escape to avoid trouble.
                htmlValue = escapeHTMLChars(htmlValue);
                      
            option.setAttribute("icon", value.icon);
            option.setAttribute("value", valueString);
            option.innerHTML = (value.icon 
                                ? '<span class="MenuIcon">' + value.icon + '</span>'
                                : ''
                             )
                             + (filterString 
                                ? htmlValue.replace(filterString, 
                                    "<span class='Highlight'>" + filterString + "</span>")
                                : htmlValue
                             );
            
            option.addEventListener("mouseover", function (ev) {
                if (selectedOption)
                    selectedOption.classList.remove("Selected");
                selectedOption = ev.target;
                selectedOption.classList.add("Selected");
            });
            option.addEventListener("mouseout", function (ev) {
                if (selectedOption)
                    selectedOption.classList.remove("Selected");
                selectedOption = null;
                ev.target.classList.remove("Selected");
            });
            option.addEventListener("click", function (ev) {
                selectHandler({ target: selectElement, originalTarget: option });
                
                hideMenu(ev);
            });
        }
        else if (filterString) {
            option.classList.add("Empty");
            option.setAttribute("value", "");
            option.innerHTML = "No matches for '" + filterString + "'";
            option.addEventListener("click", hideMenu);
        }
        else {
            option.classList.add("Empty");
            option.setAttribute("value", "");
            option.innerHTML = "No values available";
            option.addEventListener("click", hideMenu);
        }
        
        return option;
    }
    
    function createOptions(noSort) {
        // When recreating the list, we reset the selected option, too.
        selectedOption = null;
        
        var values = valueList.filter(function (val) {
                val = val.name || val;
                
                return !filterString || val.includes(filterString);
                // let found = val.toLowerCase()
                // .              .includes(filterString.toLowerCase());
            });

        if (!noSort)
            values.sort(function (valA, valB) {
                var strA = typeof(valA.value) === "string" 
                         ? valA.value
                         : typeof(valA) === "string" 
                         ? valA
                         : "";
                var strB = typeof(valB.value) === "string" 
                         ? valB.value
                         : typeof(valB) === "string" 
                         ? valB
                         : "";

                return strA.toLocaleLowerCase() < strB.toLocaleLowerCase() 
                     ? -1
                     : strA.toLocaleLowerCase() > strB.toLocaleLowerCase()
                     ? 1
                     : 0;
            });

        // First a quick analysis on key types (sources).
        // If we don't find any, or only one, we can continue as usual.
        // Othwerwise, we need to create a section per type.
        var types = values.reduce(function (accum, value) {
            var valueString = typeof(value.value) === "string" 
                            ? value.value
                            : typeof(value) === "string" 
                            ? value
                            : "";

            var valParts = valueString.split("##");
            if (valParts[1] && !accum.includes(valParts[1]))
                accum.push(valParts[1]);
            return accum;
        }, []).sort();

        menu.innerHTML = "";
        
        if (values.length === 0) {
            menu.appendChild(createOption());   
        }
        else if (types.length > 1) { 
            types.forEach(function (type) {
                var header = document.createElement("div");
                header.className = "Header";
                header.innerHTML = type;
                
                menu.appendChild(header);
                
                values.forEach(function (value) {
                    var valueString = typeof(value.value) === "string" 
                                    ? value.value
                                    : typeof(value) === "string" 
                                    ? value
                                    : "";
                    if (type == valueString.split("##")[1])
                        menu.appendChild(createOption(value));
                });      
            });
        }
        else {
            values.forEach(function (value) {
                menu.appendChild(createOption(value));
            });
        }
    }
    
    let menuPending = null;

    function hideMenu(ev) {
        if (menuPending) {
            clearTimeout(menuPending);
            menuPending = null;
        }

        if (ev && ev.currentTarget) {
            // This may be overkill, but at least one of these is necessary...
            ev.cancelBubble = true;
            ev.preventDefault = true;
            ev.stopImmediatePropagation();
            ev.stopPropagation();        
        }
        
        if (shield && shield.parentElement) {
            shield.parentElement.removeChild(shield);
        }
        shield = null;
        menu   = null;
        
        selectedOption = null;
        filterString   = null;

        document.removeEventListener("keydown", keyDownHandler, true);
    }
    
    function showMenu(selector, values, onSelect, noSort) { 
        hideMenu();
        
        filterString = "";
        
        // These are object-level variables because the 'createOptions()' 
        // and 'createOption()' functions need to be able to reference them.
        selectElement = selector;
        selectHandler = onSelect;
        valueList     = values || [];
        currentValue  = selector.getAttribute("value");

        menu = document.createElement("div");
        menu.className        = "DropdownMenu";
        menu.style.top        = "-100px"; 
        menu.style.left       = "8px";
        menu.style.width      = "";      // May be adjusted later.
        menu.style.display    = "flex"; 
        menu.style.overflowX  = "auto";
        
        shield = document.createElement("div");
        shield.className      = "MenuShield";
        shield.style.position = "absolute";
        shield.style.zIndex   = 99;
        shield.style.padding  = "100px";

        shield.addEventListener("click",      hideMenu);
        shield.addEventListener("mouseleave", hideMenu);

        selector.appendChild(shield);
        
        // Fill the menu with options.
        createOptions(noSort);
        
        // Menu is ready - put in DOM.
        shield.appendChild(menu);

        // Time for positioning and sizing.
        shield.style.left = selector.offsetLeft - 100 + "px";

        if (menu.offsetWidth < menu.scrollWidth)
            menu.style.width = menu.scrollWidth + "px";

        if (shield.offsetTop + menu.offsetHeight > document.body.clientHeight - 80)
            menu.style.height = document.body.clientHeight - 80 - shield.offsetTop + "px";

        if (shield.offsetLeft + shield.offsetWidth > document.body.clientWidth + 80)
            shield.style.left = 80 + document.body.clientWidth - shield.offsetWidth + "px";

        // We want to intercept keystrokes so that we can control the menus and so on.
        document.addEventListener("keydown", keyDownHandler, true);
    }
    
    return {
        show: function() {
            hideMenu();
            menuPending = setTimeout(showMenu, 200, ...arguments); // To skip double-click;
        },
        hide: function () {
            hideMenu(arguments);
        }
    }
}