
function PicklistControl (container, config) {

    var self = this;
    var menu = new Menu();
    
    this.config  = {
        multiValue: config ? !!config.multiValue : true,
        label:      config && config.label ? config.label : null
    };
    this.options = null;
    this.values  = null;
    this.valueNames = {};
    
    // Whenever something changes, we raise an event to the subscriber(s).
    this.changeHandler = function () {};
    
    function onChange() {
        self.changeHandler({
            target: container,
            values: getPicklistValue()
        });
    /*    setTimeout(self.changeHandler, 1, {
            target: container,
            values: getPicklistValue()
        }); */
    }
    
    function isValueAllowed(value) {     
        // If we don't have any info about what is allowed,
        // allow the value by default.
        if (!self.options) return true;

        for (var i=0; i < self.options.length; i++) {
            if (self.options[i].value === value) {
                return true;
            }
        }   
        return false;
    }   
    
    function setSelectedValues() {
        var values = Array
            .from(container.getElementsByClassName("ValueSelector"))
            .map(function (valSel) {
                return valSel.getAttribute("value");
            })
            .join(",");

        container.querySelector(".PicklistControl").setAttribute("values", values);
    }
    
    function getValueOptions(valuesAlreadySelected, currentValue) {
        var options = self.options && self.options.length > 0 
                    ? self.options.filter(function (value) {
                        // Accept value as available if not yet used, or if it is current value.
                        return !valuesAlreadySelected.includes(value.value) || value.value === currentValue; 
                      })
                    : [];
        return options;
    }
      
    var doSelectValue = function (evt) {
        var valueControl = evt.target.className === "AddValueSelector"
                         ? evt.target
                         : evt.target.parentElement;
        var critControl  = valueControl.parentElement;                 
         
        var onValueSelected = function (ev) {
            var value = ev.originalTarget.getAttribute("value");
            
            /*  Set the 'value' chosen from the dropdown on this element, so that
                this ValueControl (ValueSelector) can be queried for its value.
                If the value was selected after clicking the '+' button, we need
                to create a new value control instead.
             */
            if (valueControl.className === "AddValueSelector") {
                valueControl = addValueControl({ target: valueControl }, value);
            }
            else {
                valueControl.setAttribute("value", value);
                
                // The menu may still be present in the DOM.
                var valueText = valueControl.lastElementChild;
                if (valueText.classList.contains("MenuShield"))
                    valueText = valueText.previousElementSibling;
                         
                valueText.innerHTML = escapeHTMLChars(self.valueNames[value] || value);          
            }
            
            // Set the list of currently selected values on the control.
            setSelectedValues();
            onChange();
        };
        
        var availableValues = getValueOptions(
                                critControl.getAttribute("values").split(","),
                                valueControl.getAttribute("value")
                              );

        menu.show(valueControl, availableValues, onValueSelected);
    }
    
    function removeValueControl(evt) {
        var valueControl = evt.target.parentElement;
        var critControl  = valueControl.parentElement;
        
        // Set the focus on one of the remaining values, or the '+' button.
        var newFocusControl = valueControl.previousElementSibling.classList.contains("InvCritSelector")
                            ? valueControl.nextElementSibling
                            : valueControl.previousElementSibling;
        
        critControl.removeChild(valueControl);
        newFocusControl.focus();

        // Set the list of currently selected values on the control.
        setSelectedValues();
        onChange();        
    }
    
    function addValueControl(evt, value) {
        // evt.target is always presumed to be the '+' button.
        var critControl = evt.target ? evt.target.parentElement : container;
        
        var valueControl = document.createElement("span");
        valueControl.className = "ValueSelector";
        valueControl.setAttribute("value", value);
        
        var remValButton = document.createElement("span");
        remValButton.className = "RemValButton";
        remValButton.addEventListener("click", removeValueControl);
        remValButton.innerHTML =  "&#215;";
        valueControl.appendChild(remValButton);
        
        var valSelector = document.createElement("span");
        valSelector.className = "Value";
        valSelector.addEventListener("click", doSelectValue);
        valSelector.innerHTML = escapeHTMLChars(self.valueNames[value] || value);
        valueControl.appendChild(valSelector);

        critControl.insertBefore(valueControl, evt.target);
        
        valueControl.setAttribute('tabindex', '0');  // Allow focus.
        valueControl.addEventListener("keydown", function (ev) {
            // Don't respond to keys except for the ones that should trigger the menu,
            // and the "Delete" button.
            if (![" ", "ArrowDown", "ArrowUp", "Backspace", "Delete"].includes(ev.key)) return;
            
            if (ev.key === "Backspace" || ev.key === "Delete")
                removeValueControl({target: remValButton });
            else
                doSelectValue({ target: valSelector });

            // This may be overkill, but at least one of these is necessary...
            ev.cancelBubble = true;
            ev.preventDefault = true;
            ev.stopImmediatePropagation();
            ev.stopPropagation();
        });
        
        // If we can have only one value, the '+' button must be hidden now.
        if (!self.config.multiValue) 
            evt.target.style.display = "none";

        return valueControl;
    }
          
    function addPicklistControl(criterion) {
        /*
            criterion: 
            { name: "Pick your values", 
              values: ["A", "B"]
            }
         */
        var picklistControl = document.createElement("span");
        picklistControl.className = "PicklistControl"; 
        
        picklistControl.setAttribute("values",  criterion.values.join(","));
                
        // Key for the criterion TODO: This is actually no longer a selector, 
        // but simply a label. It may be empty - depends on what we have been
        // confifgured with (options object in constructor).
        var keySelector = document.createElement("span");
        keySelector.className = criterion.name ? "Label" : "EmptyLabel";
        keySelector.innerHTML = criterion.name ? (criterion.name + ": ") : "";

        picklistControl.appendChild(keySelector);
        
        var addValueSelector = document.createElement("span");
        addValueSelector.className = "AddValueSelector";
        addValueSelector.setAttribute('tabindex', '0'); // Allow focus.
        addValueSelector.addEventListener("click", doSelectValue);
        addValueSelector.addEventListener("keydown", function (ev) {
            // Don't respond to keys except for the ones that should trigger the menu.
            if (![" ", "ArrowDown", "ArrowUp", "Enter"].includes(ev.key)) return;

            doSelectValue({ target: addValueSelector });
            
            // This may be overkill, but at least one of these is necessary...
            ev.cancelBubble = true;
            ev.preventDefault = true;
            ev.stopImmediatePropagation();
            ev.stopPropagation();
        });
        
        addValueSelector.innerHTML = "&#43;";
            
        picklistControl.appendChild(addValueSelector);
        
        // Add values for the criterion.
        criterion.values.forEach(function (value) {
            if (isValueAllowed(value))
                addValueControl({ target: addValueSelector }, value);
        });

        container.appendChild(picklistControl);
        
        return picklistControl;
    }

    // Getter for the complete value of the whole control.
    function getPicklistValue() {
        var values  = Array
            .from(container.getElementsByClassName("ValueSelector"))
            .map(function (valSel) {
                return valSel.getAttribute("value");
            });
 
        return values;
    }
    
    // Setter for the complete value of the whole control.
    function setPicklistValue(values) {
        self.values = values;
        
        container.innerHTML = "";
        
        // Next, add the values already selected.
        addPicklistControl({ name: self.config.label, values: values });

        onChange();
    }
    
    // Setter for the allowed keys and values.
    /*  [
            { name: "Field One", value: "field1", values: [] },
            ...
        ]
     */
    function setPicklistOptions(options) {
        // Create a unique key. In this case, it's the value.
        self.valueNames = {};
        self.options = options.map(function (option) {
            self.valueNames[option.value] = option.name;

            return option;
        });
        
        // We will re-render the whole control, which will validate the values as well.
        setPicklistValue(getPicklistValue());
    }
    
    // Helper functions are defined. 
    this.changeHandler  = container.onchange || function() {};
    
    // Return an abject to the caller at which the value can be accessed
    // and event listeners registered (maybe). 
    return {
        get value() {
            return getPicklistValue();
        },
        set value(vals) {
            return setPicklistValue(vals);
        },
        set options(opts) {
            return setPicklistOptions(opts);
        },
        set onchange(handler) {
            self.changeHandler = handler;
        },
        addEventListener: function (evt, handler) {
            if (evt === "change") 
                self.changeHandler = handler;
            else
                container.addEventListener(evt, handler);
        }
    };
}
