WhereLister = function (container, config) {
    var self = this;

    let baseURL  = null;
    let apiToken = null;

    var controls   = {};
    var tags       = {};
    var fields     = {};
    var fullList   = [];
    var entityType = null;
    var mgtZone    = null;
                 
    let comparators = [
        { icon: '&#x2208',    value: '=',       name: "Equals or in",     minValues: "1" },
        { icon: '&#x2209;',   value: '<>',      name: "Not equals or in", minValues: "1" },
        { icon: '=&#x2205',   value: 'IS NULL', name: "Is NULL",          minValues: "0", maxValues: "0" },
        { icon: '<span style="position:relative; top:1px;font-size:90%;padding-right:1px">&#8800;</span>&#x2205;', 
                              value: 'IS NOT NULL', name: "Is not NULL",  minValues: "0", maxValues: "0" },
        { icon: '&#x227B;',   value: '>',       name: "Larger than",      minValues: "1", maxValues: "1" },
        { icon: '&#x227A;',   value: '<',       name: "Smaller than",     minValues: "1", maxValues: "1" },
        { icon: '[&#x2217;]', value: 'BETWEEN', name: "Between",          minValues: "2", maxValues: "2" },
        { icon: '&#x39e;', value: 'LIKE', name: "Like",          minValues: "1" },
        { icon: '&#x3a0;', value: 'NOT LIKE', name: "Not Like",          minValues: "1" }
    ];

    this.events  = {};

    var emit = function (eventName, data) {
        // Call the subscribed event handlers.
        const event = self.events[eventName] || [];
        event.forEach(fn => { fn.call(null, data); });
    };

    var subscribe = function (eventName, fn) {
        // Add the subscription (event handler) for this event.
        self.events[eventName] = self.events[eventName] || [];
        self.events[eventName].push(fn);

        // Return an 'unsubscribe' function for this subscription.
        return () => {
            self.events[eventName] = self.events[eventName].filter(eventFn => fn !== eventFn);
        };
    };

    function populateFilterControl(filterControl, fields, tags) {
        // THE VALUES come from the JobEditor. We only have fields.
        // The criterion options should be added to the filteroptions.
        var filterOptions = [];
        
        for (var name in fields) {
            filterOptions.push({
                type:   "Field",
                name:   name,
                values: fields[name]
            });
        }
        for (var name in tags) {
            filterOptions.push({
                type:  "Tag",
                name:   name,
                values: tags[name]
            });
        }
        filterControl.options = filterOptions;
    }
    function processDefinitions(apiDescription) {
        function createCriteria(props) {
            return Object
                .keys(props)
                .filter(prop => props[prop].type != 'object' && 
                                props[prop].type != 'array')
                .map(prop => {
                    return {
                        value:  prop,
                        name:   prop,
                        descr:  props[prop].description,
                        values: props[prop].enum || 
                                props[prop].type == "boolean"
                              ? ["TRUE", "FALSE"]
                              : null
                    };
                });
        }
    
        /*
            WHEN FIELDS AND METRICS ARE USED AS FILTERS, WE SHOULD ACCOUNT FOR ENUMS AND BOOLEANS
         */
        const userSessionProps = apiDescription.definitions.UserSession.properties;

        let usersession  = createCriteria(userSessionProps); 

        const userActionProps  = apiDescription.definitions.UserSessionUserAction.properties;

        let useraction  = createCriteria(userActionProps); 

        controls.filterControl.options = usersession;
    }

    // Called whenever the filter changes.    
    function onFilterChange(ev) {
        console.log(ev.target.id + ": " + JSON.stringify(ev.value));

        // Makebe re-run the query? Or better done 'on-demand'?
    }     
        
    function processData(jsonData) {
        /*  jsonData looks like this:
        {
            "extrapolationLevel": 1,
            "columnNames": [
                "country",
                "city",
                "count(*)"
            ],
            "values": [
                [
                "null",
                "null",
                120
                ],
                [
                "United States",
                "Sunnyvale",
                16
                ],
                ...
            ]
        }
    */
        fullList  = jsonData.sort(function (a, b) {
            if (!a.sortName)
                a.sortName = a.displayName.trim().toLowerCase();
            if (!b.sortName)
                b.sortName = b.displayName.trim().toLowerCase();
            
            return a.sortName < b.sortName
                 ? -1
                 : a.sortName > b.sortName
                 ? 1
                 : 0;
        });

        controls.resultCount.innerHTML  = fullList.length + " items";

        controls.resultList.items    = filteredList;  
    }   

      
    function getData() {
        // Clear out old data - visually, too.
        filteredList = [];
        fullList   = [];

        // Construct query
        let query = "SELECT " + jobEditor.fields + ", " + jobEditor.metrics
                  + " FROM "  + jobEditor.type + 
                  + " WHERE " + controls.filterControl.value;
                  // TODO: GROUP BY, ORDER BY
 
        
        // Request new data.
        try {
            var xhr = new JSONHttpRequest({         
                address:      baseURL + "/api/v1/" + apiEndPoints[entityType] 
                              + '?Api-Token=' + apiToken
                              + '&query=SELECT%20country%2C%20city%2C%20COUNT(*)%20FROM%20usersession%20GROUP%20BY%20country%2C%20city',
                onComplete:   processData,
                progressElem: controls.resultCount
            });

            xhr.send();
        }
        catch (ex) {
            alert("The Dynatrace API could not be called: " + ex.message);
        }
    }

    function init(url, token) {
        baseURL  = url;
        apiToken = token;
    }

    // Now we create the actual UI.
    container.innerHTML = "";

    var configArea = document.createElement("div");
    configArea.id = "entityFilterArea";
    container.appendChild(configArea);
 
    var listArea = document.createElement("div");
    listArea.id = "availableList";
    container.appendChild(listArea);

    var topLabel = document.createElement("span");
    topLabel.appendChild(document.createTextNode("Get "));

    var entityTypeText = document.createElement("span");
    entityTypeText.id = "entityTypeText";
    topLabel.appendChild(entityTypeText);
    topLabel.appendChild(document.createTextNode(" from "));

    configArea.appendChild(topLabel);

    // TODO: Make this plain text, following what the JobEditor says.
    var mgtZoneText = document.createElement("span");
    mgtZoneText.id = "mgtZoneText";

    configArea.appendChild(mgtZoneText);

    var refreshButton = document.createElement("input");
    refreshButton.type = "button";
    refreshButton.value = "Get";
    refreshButton.addEventListener("click", getData);

    configArea.appendChild(refreshButton);

    var resultCount = document.createElement("span");
    resultCount.id = "resultCount";

    configArea.appendChild(resultCount);
    
    var filterToggle = document.createElement("div");
    filterToggle.className = "FilterToggle";
    filterToggle.style.marginTop = "20px";

    var filterToggleControl = document.createElement("span");
    filterToggleControl.id = "filterToggleControl";
    filterToggleControl.appendChild(document.createTextNode("Filter"));

    filterToggle.appendChild(filterToggleControl);
    configArea.appendChild(filterToggle);

    var filterArea = document.createElement("div");
    filterArea.id = "filterArea";
    
    var filterMatches = document.createElement("span");
    filterMatches.id = "filterMatches";

    filterArea.appendChild(filterMatches);

    var filterBoxes = document.createElement("div");
    filterBoxes.id = "filterBoxes";

    filterArea.appendChild(filterBoxes);
    configArea.appendChild(filterArea);

    var availableToggle = document.createElement("div");
    availableToggle.className = "AvailableToggle";

    var avaiableToggleControl = document.createElement("span");
    avaiableToggleControl.className = "AvaiableToggle";
    avaiableToggleControl.innerHTML = "Available";

    availableToggle.appendChild(avaiableToggleControl);
    configArea.appendChild(availableToggle);

    var availableControls = document.createElement("div");
    availableControls.className = "AvailableControls";

    var addToJobControl = document.createElement("input");
    addToJobControl.type = "button";
    addToJobControl.value = "Add to Job";
    addToJobControl.addEventListener("click", function (evt) {
        emit("addtojob", controls.availableList.selectedItems);
    });

    availableControls.appendChild(addToJobControl);

    var rightCount = document.createElement("span");
    rightCount.id = "rightCount";

    availableControls.appendChild(rightCount);
    configArea.appendChild(availableControls);

    adjustBottomHeightOnTopChange(configArea, listArea);

    // Collect all controls on the page for quick reference later.
    controls = {
        entityTypeText:       entityTypeText,
        mgtZoneText:          mgtZoneText,
        resultCount:          resultCount,
        filterMatches:        filterMatches,
        filterToggleControl:  filterToggleControl,
        filterControl:        new FilterControl(filterBoxes, { comparators: comparators, editable: true, emptytext: "" }),
        availableList:        new ListTable(listArea, { selectionControls: rightCount })
    };

    controls.filterControl.addEventListener("change", onFilterChange);

    // Return the interface.
    // Events to expose: 'onaddtojob' or something.
    return {
        on:    subscribe,
        init:  init,
        set filterOptions(options) {

        },
        set definitions(value) {
            processDefinitions(value);
        },
        reset: function (scope) {
            entityType = scope.type.value;
            controls.entityTypeText.innerHTML = scope.type.label + "s";

            mgtZone = scope.zone.value;
            controls.mgtZoneText.innerHTML = scope.zone.label;

            fullList     = [];
            filteredList = [];

            controls.availableList.specials = [];
            controls.availableList.items    = filteredList;  
        },
        update: function (specials) {
            controls.availableList.specials = specials;
            controls.availableList.items    = filteredList;  
        },
        select: function (items) {
            controls.availableList.selectedItems = items;
        }
    };
    
}
