/*** WINDOW MANAGER ***/

TooltipManager     = {};
InfoWindowManager  = {};
ContextMenuManager = {};
DateTimePicker     = {};

/*  The DialogManager needs to follow the class/prototype pattern because you can have
    sequences of dialogs (wizard-like), thus we need to preserve the context of each -
    at least, that's what I am thinking right now.
 */
DialogManager = function (eventManager, source, resolver) {
    this.source       = source;
    this.eventManager = eventManager;
    this.resolver     = resolver || { resolve: function (value) { return value; }};
    this.values       = {};
}

WindowManager = {
    shield:   null,
    frame:    null,
    vector:   null,
    instance: null,
    window:   null,

    createShield: function (underThisWindow) {
        if (!WindowManager.shield) {
            WindowManager.shield = document.createElement("div");
            WindowManager.shield.id = "shield";
            WindowManager.shield.style.position = "absolute";
            WindowManager.shield.style.left     = "0px";
            WindowManager.shield.style.right    = "0px";
            WindowManager.shield.style.top      = "0px";
            WindowManager.shield.style.bottom   = "0px";
            WindowManager.shield.style.display  = "none";         
        //    WindowManager.shield.style.backgroundColor = "yellow"; // DEBUG
        }

        if (underThisWindow) 
            document.body.insertBefore(WindowManager.shield, underThisWindow);   
        else 
            document.body.appendChild(WindowManager.shield);                   
    },
    raiseShield: function (instance, isModal) {
        WindowManager.lowerShield();    // This ensures that we don't leave event listeners behind.
        
        WindowManager.instance = instance;
        WindowManager.window   = instance.window;
        
        WindowManager.createShield(WindowManager.window);
        
        /*  When the window is modal, we create a shield that covers the whole page. However, when
            is it just a dropdown is something like that, we create a shield that extends for 100
            pixels around the window. That gives us a nice area over which the mouse can hover so
            that we know the user is still interested in interacting with the window.
            When the user moves the mouse outside this area, we assume he is no longer interested,
            and we will trigger the closure of the window.
         */
        if (isModal) {
            WindowManager.shield.style.left   = "0px";
            WindowManager.shield.style.right  = "0px";
            WindowManager.shield.style.width  = "100%";            
            WindowManager.shield.style.top    = "0px";
            WindowManager.shield.style.bottom = "0px";            
            WindowManager.shield.style.height = "100%";            
        }
        else {
            WindowManager.shield.style.left   = WindowManager.window.offsetLeft   - 100 + "px";
            WindowManager.shield.style.right  = WindowManager.window.offsetLeft   + 
                                                WindowManager.window.offsetWidth  + 100 + "px";
            WindowManager.shield.style.width  = WindowManager.window.offsetWidth  + 200 + "px";
            WindowManager.shield.style.top    = WindowManager.window.offsetTop    - 100 + "px";
            WindowManager.shield.style.bottom = WindowManager.window.offsetTop    + 
                                                WindowManager.window.offsetHeight + 100 + "px";       
            WindowManager.shield.style.height = WindowManager.window.offsetHeight + 200 + "px";
            
            WindowManager.shield.addEventListener("mousedown",  WindowManager.onMouseDown, true);
            WindowManager.shield.addEventListener("mouseleave", WindowManager.onMouseOut,  true);
        }
        WindowManager.shield.style.display  = "block";
        WindowManager.shield.style.zIndex   = "998";         
        WindowManager.window.style.zIndex   = "999";
    },
    isShieldUp: function (instance) {
        if (instance && WindowManager.instance !== instance) return false;
        return (WindowManager.shield && WindowManager.shield.style.display  == "block");       
    },
    lowerShield: function (instance) {
        if (!WindowManager.shield) return;      
        if (instance && WindowManager.instance !== instance) return;
        
        WindowManager.window   = null;
        WindowManager.instance = null;
        
        WindowManager.shield.style.display  = "none";
        WindowManager.shield.removeEventListener("mousedown",  WindowManager.onMouseDown, true);
        WindowManager.shield.removeEventListener("mouseleave", WindowManager.onMouseOut,  true);
    },
    onMouseDown: function (ev) {
        ev = ev || window.event;
        
        if (!WindowManager.window || !WindowManager.instance) return;
        
        // Did the user click _outside_ the window? That means cancel.
        if (ev.clientX <  WindowManager.window.offsetLeft    ||
            ev.clientX > (WindowManager.window.offsetLeft + 
                          WindowManager.window.offsetWidth)  ||
            ev.clientY <  WindowManager.window.offsetTop     ||
            ev.clientY > (WindowManager.window.offsetTop + 
                          WindowManager.window.offsetHeight))
        {   
            if (WindowManager.instance.cancel) 
                WindowManager.instance.cancel();     
            
            WindowManager.instance && WindowManager.instance.hide();        
        }        
    },
    onMouseOut: function (ev) {
        ev = ev || window.event;

        if (!WindowManager.instance) return;
         
        // Did the user move _outside_ the shield? That means cancel.
        if (ev.clientX <=  WindowManager.shield.offsetLeft    ||
            ev.clientX >= (WindowManager.shield.offsetLeft + 
                           WindowManager.shield.offsetWidth)  ||
            ev.clientY <=  WindowManager.shield.offsetTop     ||
            ev.clientY >= (WindowManager.shield.offsetTop + 
                           WindowManager.shield.offsetHeight))        
        {   
            if (WindowManager.instance.cancel) 
                WindowManager.instance.cancel();     
            
            WindowManager.instance && WindowManager.instance.hide();        
        }        
    },
    bestPosition: function (
        elemLeft, elemWidth, 
        elemTop, elemHeight, 
        distanceX, distanceY, 
        tooltipWidth, tooltipHeight, 
        preferredPosition, margin) 
    {
        /*
            The tooltip can be positioned on any of 12 locations relative to the element it pertains to.
            In the diagram below, we have positioned the tooltip in the center and the possible relative
            positions for the element it aligns with (EL) around it. 
            
                     AFTER  LEFT    CENTER   RIGHT  BEFORE
                            
            BELOW           EL        EL        EL    
                            ______________________  
            TOP         EL  |                    |  EL
            MIDDLE      EL  |      TOOLTIP       |  EL
            BOTTOM      EL  |____________________|  EL
            
            ABOVE           EL        EL        EL    
                                       
            We will traverse this option space clockwise around the element, starting at the preferred 
            orientation, until we find a place that keeps the tooltip inside the window boundaries.
         */
         
        margin = margin || 0;
        
        var position = {
            before: function (elemLeft, elemWidth, distanceX, tooltipWidth) {
                    return elemLeft - tooltipWidth + distanceX;
            },
            left:   function (elemLeft, elemWidth, distanceX, tooltipWidth) {
                    return elemLeft + distanceX;
            },
            center: function (elemLeft, elemWidth, distanceX, tooltipWidth) {
                    return Math.round(elemLeft + (elemWidth/2) - (tooltipWidth/2)) + distanceX;
            },
            right:  function (elemLeft, elemWidth, distanceX, tooltipWidth) {
                    return elemLeft + elemWidth - tooltipWidth - distanceX;
            },   
            after:  function (elemLeft, elemWidth, distanceX, tooltipWidth) {
                    return elemLeft + elemWidth + distanceX;
            },
            above:  function (elemTop, elemHeight, distanceY, tooltipHeight) {
                    return elemTop - tooltipHeight - distanceY;
            },
            top:    function (elemTop, elemHeight, distanceY, tooltipHeight) {
                    return elemTop + distanceY;
            },
            middle: function (elemTop, elemHeight, distanceY, tooltipHeight) {
                    return Math.round(elemTop + (elemHeight/2) - (tooltipHeight/2)) + distanceY;
            },
            bottom: function (elemTop, elemHeight, distanceY, tooltipHeight) {
                    return elemTop + elemHeight - tooltipHeight - distanceY;
            },
            below:  function (elemTop, elemHeight, distanceY, tooltipHeight) {
                    return elemTop + elemHeight + distanceY;
            }
        };
        var candidates = [
            { orientation: "below",  alignment: "left",   getX: position.left,   getY: position.below  },
            { orientation: "below",  alignment: "center", getX: position.center, getY: position.below  },
            { orientation: "below",  alignment: "left",   getX: position.left,   getY: position.below  },    /* Is a logical first alternative to the previous one */
            { orientation: "below",  alignment: "right",  getX: position.right,  getY: position.below  },
        /*  { orientation: "below",  alignment: "before", getX: position.before, getY: position.below  }, */
            { orientation: "top",    alignment: "before", getX: position.before, getY: position.top    },
            { orientation: "middle", alignment: "before", getX: position.before, getY: position.middle },
            { orientation: "bottom", alignment: "before", getX: position.before, getY: position.bottom },
        /*  { orientation: "above",  alignment: "before", getX: position.before, getY: position.above  }, */
            { orientation: "above",  alignment: "right",  getX: position.right,  getY: position.above  },
            { orientation: "above",  alignment: "center", getX: position.center, getY: position.above  },
            { orientation: "above",  alignment: "left",   getX: position.left,   getY: position.above  },
        /*  { orientation: "above",  alignment: "after",  getX: position.after,  getY: position.above  }, */
            { orientation: "bottom", alignment: "after",  getX: position.after,  getY: position.bottom },
            { orientation: "middle", alignment: "after",  getX: position.after,  getY: position.middle },
            { orientation: "top",    alignment: "after",  getX: position.after,  getY: position.top    }
        /*  { orientation: "below",  alignment: "after",  getX: position.after,  getY: position.below  } */
        ];   
        
        preferredPosition = preferredPosition || "below-left";
        preferredPosition = preferredPosition.split("-");
        
        for (var c = 0; 
                 c < candidates.length && (candidates[0].orientation != preferredPosition[0] || 
                                           candidates[0].alignment   != preferredPosition[1]); 
                 c++) {
            candidates.push(candidates.shift());    // We want the preferred position at the beginning.
        }
     
        var minX = 5,
            minY = 5,
            x    = minX,
            y    = minY,
            maxX = document.body.scrollWidth  - minX - tooltipWidth,
            maxY = document.body.scrollHeight - minY - tooltipHeight;
        
        // Found or not: we start at the beginning of our option list.   
        var position          = null;
        var goodPositionFound = false;
        for (var c = 0; c < candidates.length && !goodPositionFound; c++) {
            x = candidates[c].getX(elemLeft, elemWidth,  distanceX, tooltipWidth);
            y = candidates[c].getY(elemTop,  elemHeight, distanceY, tooltipHeight);

            x += candidates[c].alignment == "before"
               ? -margin
               : candidates[c].alignment == "after"
               ? margin
               : 0;
            y += candidates[c].orientation == "above"
               ? -margin
               : candidates[c].orientation == "below"
               ? margin
               : 0;
                        
            goodPositionFound = (x >= minX) && (x <= maxX)
                             && (y >= minY) && (y <= maxY);                        
        }

        if (goodPositionFound) {        
            // Oohh - good news! Note: c is now 1 larger than the good index we found.
            position = candidates[c-1];
        }
        else {         
            // Use preferred position as fallback - still ensure tooltip is within boudaries - 
            // but also: don't show a notch in this case, as we are no longer nicely positioned
            // according to what we should be pointing to.
            position = { orientation: "nonotch", alignment: "nonotch" };
            x = candidates[0].getX(elemLeft, elemWidth,  distanceX, tooltipWidth);
            y = candidates[0].getY(elemTop,  elemHeight, distanceY, tooltipHeight);
            
            if (x > maxX) x = maxX;
            if (x < minX) x = minX;
            if (y > maxY) y = maxY;
            if (y < minY) y = minY;
        }

        return { x: x, y: y, orientation: position.orientation, alignment: position.alignment };
    },
    onStartDrag: function (ev, titleBar) {
        ev = ev || window.event;
        
        WindowManager.frame = WindowManager.window;
        WindowManager.vector = { left: ev.clientX - WindowManager.frame.offsetLeft,
                                 top:  ev.clientY - WindowManager.frame.offsetTop  };

        titleBar.addEventListener("mousemove", WindowManager.onDrag);
        
        WindowManager.frame.classList.add("DialogDragging");
    },
    onDrag: function (ev) {
        ev = ev || window.event;
        
        // Note: A draggable window has no shield, so no reason to update it's position.
        if (!WindowManager.frame) return;
        WindowManager.frame.style.left = (ev.clientX - WindowManager.vector.left) + "px";
        WindowManager.frame.style.top  = (ev.clientY - WindowManager.vector.top)  + "px";
    },
    onEndDrag: function (ev, titleBar) {
        ev = ev || window.event;
        
        if (!WindowManager.frame) return;
        WindowManager.frame.classList.remove("DialogDragging");    
        
        titleBar.removeEventListener("mousemove", WindowManager.onDrag);

        WindowManager.frame  = null;
        WindowManager.vector = null;
    }    
};

TooltipManager.show = function (source, options) {
    /*  If somebody (including us, if we are frozen) has raised the shield, 
        it's not a good time to show a new tooltip.
     */
    if (WindowManager.isShieldUp()) return; 
    
    if (this.window) {
        if (options.notch) 
            this.window.classList.add("Notch");
        else
            this.window.classList.remove("Notch");
    }
    else {
        this.window = document.createElement("div");
        this.window.id = "popupBox";
        this.window.className = "Tooltip" + (options.notch?" Notch":"");
        this.window.style.border = "2px solid transparent";  
        
        document.body.appendChild(this.window);
        
        window.addEventListener("keydown",
            function (ev) {
                ev = ev || window.event;

                if (ev.keyCode == 113 || ev.keyCode == 192 ) {    // F2 or '`'
                    TooltipManager.freeze();
                    return false;
                }
                if (ev.keyCode == 27) {     // ESC
                    TooltipManager.hide(true);
                    return false;
                }
            },
            false
        );        
    };
    
    this.source     = source;
    this.descriptor = {
        element:  { x:0, y:0, h:20, w:20 },
        offset:   { x:0, y: 0 },
        max:      { h:400, w: 400 },
        position: "below-left",
        contents: ""
    };
    $.extend(true, this.descriptor, options);

    this.descriptor.x  = (this.descriptor.x || this.descriptor.element.x || 0); // + this.descriptor.offset.x;
    this.descriptor.y  = (this.descriptor.y || this.descriptor.element.y || 0); // + this.descriptor.offset.y;
    this.descriptor.w  =  this.descriptor.w || this.descriptor.element.w || 10;
    this.descriptor.h  =  this.descriptor.h || this.descriptor.element.h || 10;

    var sourceRect = this.source && this.source.frameElement 
                   ? this.source.frameElement.getBoundingClientRect() 
                   : this.source && this.source.contentWindow
                   ? this.source.getBoundingClientRect() 
                   : { left:0, top:0, width:0, height:0 };
    this.descriptor.x += sourceRect.left;
    this.descriptor.y += sourceRect.top;
        
    this.window.innerHTML = this.descriptor.contents;
    
    this.window.style.overflow   = "hidden";
    this.window.style.maxWidth   = this.descriptor.max.w + "px";
    this.window.style.maxHeight  = this.descriptor.max.h + "px";
    this.window.style.visibility = "visible"; 
    
    if (this.window.scrollHeight > this.window.offsetHeight) {
        this.moreIndicator = document.createElement("span");
        this.moreIndicator.id = "MoreIndicator";
        this.moreIndicator.style.position = "absolute";
        this.moreIndicator.style.bottom   = "0px";
        this.moreIndicator.style.padding  = "2px 8px 2px 8px";
        this.moreIndicator.style['float'] = "right";
        this.moreIndicator.style.right    = "0px";
        this.moreIndicator.style.color    = "#008CBD";
        this.moreIndicator.style.backgroundColor = "#000000";        
        this.moreIndicator.innerHTML = "Press '`' for more... ";
        
        this.window.appendChild(this.moreIndicator);
    }
    else {
        this.moreIndicator = null;
    }
    
    var pos = WindowManager.bestPosition(this.descriptor.x,        this.descriptor.w, 
                                         this.descriptor.y,        this.descriptor.h, 
                                         this.descriptor.offset.x, this.descriptor.offset.y, 
                                         this.window.offsetWidth,  this.window.offsetHeight, 
                                         this.descriptor.position);
                                          
    this.window.style.left = pos.x + "px";
    this.window.style.top  = pos.y + "px";                             
    this.window.setAttribute("orientation", pos.orientation);  
    this.window.setAttribute("alignment",   pos.alignment); 
    
    return this.window.classList.contains("Notch") ? this.window : undefined;
}

InfoWindowManager.show = function (eventManager, source, options) {
    TooltipManager.hide(true);  // We don't want the current tooltip, if any, to remain visible.
    
    this.hide();                // Clear any remains of a previous showing and start with a clean slate.
    
    if (!this.window) {   
        this.window = document.createElement("div");
        this.window.className = "DropDownMenu";	// Yep - same style.
		this.window.style.padding = "0px";		// Except for this.
        this.window.style.display = "none";
        
        document.body.appendChild(this.window);   
    }
    
    this.window.innerHTML = "";
   
    /*
		In order to support the use of a portlet from the porlet library, we will approach this as follows:
		- Content is loaded in an iframe.
		- Its address can come from a portlet definition from the library (if a portlet ID is passed), or
		  from the 'url' field in the descriptor. Normal substitution features apply.
		- If the content includes the DV API, the content will post messages back to the parent window.
		  They will be automatically received by the page's event handler, and it will (have to) recognize
		  them as such and forward them to us (the InfoWindowManager) for handling.
		- One of the messages we may thus receive is its registration, which is the trigger for us to give
		  it its settings. 
        - If the content definition came from a portlet in the library, we have the settings already. 
		  However, if we were given a URL AND the content does use the DV API, then we will need a 'settings' 
		  object in the descriptor that was passed to us.
		- If content does not use the DV API, nothing special happens and we simply point the iframe to the
		  provided URL.
		- If the content raises messages, the only recipient is the content that requested this info window.
		- The info window will not receive messages, except for "DATA" messages associated with any of its
		  data calls.
   
		Additionally:
		- The descriptor may give permission for us to add a menu icon in the top-right. When the user clicks
		  it, we switch to context menu mode.
    */
    this.eventManager = eventManager;
    this.source       = source;
    this.descriptor   = {
        element:   { x:0, y:0, h:0, w:0 },
        offset:    { x:0, y: 0 },
        dims:      { h:0, w: 500 },
        position:  "below-center",
        content: {
            label:     "",
            contentId: null,
            contents:  null,
            url:       null           
        }
    };
    $.extend(true, this.descriptor, options);

    this.descriptor.position = this.descriptor.position || this.descriptor.orientation;    
    this.descriptor.x  = this.descriptor.element.x || this.descriptor.element.left   || this.descriptor.x || 0;
    this.descriptor.y  = this.descriptor.element.y || this.descriptor.element.top    || this.descriptor.y || 0;
    this.descriptor.w  = this.descriptor.element.w || this.descriptor.element.width  || this.descriptor.w || 10;
    this.descriptor.h  = this.descriptor.element.h || this.descriptor.element.height || this.descriptor.h || 10;

    var sourceRect = this.source && this.source.frameElement 
                   ? this.source.frameElement.getBoundingClientRect() 
                   : this.source && this.source.contentWindow
                   ? this.source.getBoundingClientRect() 
                   : { left:0, top:0, width:0, height:0 };
    this.descriptor.x += sourceRect.left;
    this.descriptor.y += sourceRect.top;
        
    if (this.descriptor.content.url) {
        var url = this.descriptor.content.url;
        
        var windowBox = document.createElement("SPAN");
        if (this.descriptor.content.label) {
            var titleBox  = document.createElement("DIV");
            titleBox.className = "Title";
            titleBox.innerHTML = this.descriptor.content.label;
            
            windowBox.appendChild(titleBox);
        }
        var iframeElem = document.createElement("IFRAME");
        iframeElem.scrolling    = "yes";
        iframeElem.sandbox      = "allow-forms allow-same-origin allow-scripts";
        iframeElem.seamless     = "seamless";           
        iframeElem.style.height = "100%";
        iframeElem.style.width  = "100%";
        iframeElem.style.border = "0px";                    
        iframeElem.addEventListener("SelectionChange", source.onSelection);    
        iframeElem.src = url;
        
        windowBox.appendChild(iframeElem); 
        this.window.appendChild(windowBox);
    }
    else if (this.descriptor.content.contentId) {
        // TODO: Load content from library into iframne        
    }
    else if (this.descriptor.content.contents) {
        var content  = ["<span>"];
        content.push("<div class='Title'>" + this.descriptor.content.label + "</div>");
        content.push(this.descriptor.content.contents);        
        content.push("</span>");
        
        this.window.innerHTML     = content.join("");     
        this.window.addEventListener("SelectionChange", source.onSelection);    
        this.window.addEventListener("SelectionCancel", InfoWindowManager.cancel.bind(this));    
    }
    else {
        var content  = ["<span>"];
        content.push("<div class='Title'>" + this.descriptor.content.label + "</div>");
        content.push("<div>(No content provided)</div>");
        content.push("</span>");
    
        this.window.innerHTML     = content.join("");     
    }
    
	this.window.style.width   = this.descriptor.dims.w + "px";
	this.window.style.height  = this.descriptor.dims.h ?  (this.descriptor.dims.h + "px") : "";  
    this.window.style.display = "inline-block";  // Necessary here to let the height and width take hold.
        
    var pos = WindowManager.bestPosition(this.descriptor.x,         this.descriptor.w, 
                                         this.descriptor.y,         this.descriptor.h, 
                                         this.descriptor.offset.x,  this.descriptor.offset.y, 
                                         this.window.offsetWidth,   this.window.offsetHeight, 
                                         this.descriptor.position, 10); // Not sure if 10 is the best value to pass here.
                                                 
    this.window.style.left = pos.x + "px";
    this.window.style.top  = pos.y + "px";                             
    this.window.setAttribute("orientation", pos.orientation);  
    this.window.setAttribute("alignment",   pos.alignment);  
    
    WindowManager.raiseShield(this);    
}

ContextMenuManager.show = function (eventManager, source, options) {
    /*  Create a context menu if we have not yet done so. We also create a shield that overlaps the
        full page so that we can intercept any mousedown event that falls outside of the context 
        menu. If the user clicks outside the context menu, we will hide it and remove the shield.
     */
    var onClick = function (ev) {  
        ev = ev || window.event;
        var self = ContextMenuManager;

        var itemID = ev.currentTarget.id;        
        if (itemID.indexOf("CTXMNEXTRA_") == 0) {
            /*  This means: the requestor only gave extra items in addition to letting the pageflow
                engine figure out what else was available for the selected nodes. So, it one of those
                extra items was clicked, then rather than letting the pageflow engine execute it, we
                let the requestor know which extra item was clicked so that it can do its thing.
             */
            self.source.postMessage("DV.CONTEXTMENU::" 
                                    + self.nodeList + "::" 
                                    + itemID.replace("CTXMNEXTRA_", ""), "*");
        }
        else if (self.descriptor.callback) {
            self.descriptor.callback.call(self.source, self.descriptor.nodeIDs, itemID.replace("CTXMN_", ""));
        }
        else if (self.descriptor.nodeIDs) {
            /*  This means: the requestor only gave us a nodeID (or a comma-separated list of node IDs) 
                and a list of context menu items. So, now that the user has clicked one of them, we 
                let the requestor know which one so that it can do its thing - whatever that is.
             */
            self.source.postMessage("DV.CONTEXTMENU::" 
                                    + self.descriptor.nodeIDs + "::" 
                                    + itemID.replace("CTXMN_", ""), "*");
        }

        self.hide();                                     
    };    
    
    var onToggle = function (ev) {  
        ev = ev || window.event;
        var self = ContextMenuManager;

        var itemID = ev.currentTarget.id;
        var id     = itemID.replace("CTXMN_", "");
        
        if (self.descriptor.selectedIDs.indexOf(id) == -1) {
            self.descriptor.selectedIDs.push(id);
            ev.currentTarget.classList.add("selected");    
        }
        else {
            self.descriptor.selectedIDs.splice(self.descriptor.selectedIDs.indexOf(id), 1);
            ev.currentTarget.classList.remove("selected");    
        }
    };

    var onApply = function (ev) {  
        ev = ev || window.event;
        var self = ContextMenuManager;
        
        self.source.postMessage("DV.CONTEXTMENU::" 
                                + self.descriptor.selectedIDs.join(",") + "::", "*");
     
        self.hide();                                     
    };
    
    var createMenuItem = function (item, prefix, isSelected) {
        var entry = document.createElement("div");
        entry.id        = prefix + item.id;
        entry.className = "MenuItem" + (isSelected ? " selected" : "");
        entry.addEventListener("click", onClick, true);
        entry.innerHTML = "<span style='width:28px; margin-left:-4px; top:2px; position:relative;' class='" 
                        + (item.icon && item.icon != "" ? item.icon : "icon-empty")
                        + "'>&nbsp;</span>" + item.label;
                        
        return entry;
    };
    
    var createToggleItem = function (item, prefix, isSelected) {
        var entry = document.createElement("div");
        entry.id        = prefix + item.id;
        entry.className = "MenuItem" + (isSelected ? " selected" : "");
        entry.addEventListener("click", onToggle, true);
        entry.innerHTML = "<span style='width:28px; margin-left:-4px; top:2px; position:relative;' class='" 
                        + (item.icon && item.icon != "" ? item.icon : "icon-empty")
                        + "'>&nbsp;</span>" + item.label;
                            
        return entry;
    };
 
    TooltipManager.hide(true);  // We don't want the current tooltip, if any, to remain visible.
    
    this.hide();                // Clear any remains of a previous showing and start with a clean slate.
 
    if (!this.window) {    
        this.window = document.createElement("div");
        this.window.className = "DropDownMenu";
        this.window.style.display = "none";
        
        document.body.appendChild(this.window);   
    }
    this.window.innerHTML = "";
    
    this.eventManager = eventManager;
    this.source       = source;
    this.descriptor   = {
        element:   { x:0, y:0, h:0, w:0 },
        offset:    { x:0, y: 0 },
        nodeIDs    : null,
        selectedIDs: [],
        nodeList   : null,
        callback   : null,
        action     : {},
        items      : null,
        extraItems : null,
        multiselect: false,
        orientation: false
    };
    $.extend(true, this.descriptor, options);
    
    // 'position' is our preferred way to refer to orientation. We allow both.
    this.descriptor.position = this.descriptor.position || this.descriptor.orientation;    
    this.descriptor.x  = this.descriptor.element.x || this.descriptor.x;
    this.descriptor.y  = this.descriptor.element.y || this.descriptor.y;
    this.descriptor.w  = this.descriptor.element.w || this.descriptor.w || 0;
    this.descriptor.h  = this.descriptor.element.h || this.descriptor.h || 0;

    var sourceRect = this.source && this.source.frameElement 
                   ? this.source.frameElement.getBoundingClientRect() 
                   : this.source && this.source.contentWindow
                   ? this.source.getBoundingClientRect() 
                   : { left:0, top:0, width:0, height:0 };
    this.descriptor.x += sourceRect.left;
    this.descriptor.y += sourceRect.top;
    
    var items      = this.descriptor.items      || [];
	var extraItems = this.descriptor.extraItems || [];
	
	/*	It the descriptor contains 'extraItems', we will first list all the pageflow-driven actions, 
		and then the extra items (maybe with a separating line in between). Now, depending on whether
		the user clicked a pageflow-driven action or one of the 'extra' actions, the callback we need
		to call is either the pageflow engine's or the caller's. So, we need to make sure that we can
		distinguish between the two cases.
	 */
    if (items.length == 0 && extraItems.length == 0) return;    // No context menu actions available.
       
    /*  Now we have menu items - provided by the portlet that asked us, or provided by the pageflow
        engine based on the actions defined in this page. Let's create the corresponding entries.
     */
    var self = this;
    var scrollupButton = document.createElement("div");
    var scrolldnButton = document.createElement("div");
    var scrollable     = document.createElement("div");

    if (this.descriptor.multiselect) {
        scrollupButton.className = "MenuIconButton UP";
        scrollupButton.innerHTML = "&#xe607";
        scrollupButton.addEventListener("mousedown", function (ev) {
            self.scrollSpeed = 1;
            self.scrollTimer = setInterval(function () {
                var marginTop = Number(scrollable.style.marginTop.replace("px","")) + self.scrollSpeed;
                if (marginTop >= self.scrollTopMax) {    // Auto-cancel when max reached.
                    marginTop  = self.scrollTopMax;
                    clearInterval(self.scrollTimer);
                    self.scrollTimer = null;
                }
                scrollable.style.marginTop = marginTop + "px";
                self.scrollSpeed += 1;
            }, 50);
        }, true);
        scrollupButton.addEventListener("mouseup", function (ev) {
            clearInterval(self.scrollTimer);
            self.scrollTimer = null;
        }, true);
        scrolldnButton.className = "MenuIconButton DN";
        scrolldnButton.innerHTML = "&#xe604";
        scrolldnButton.addEventListener("mousedown", function (ev) {
            self.scrollSpeed = 1;
            self.scrollTimer = setInterval(function () {
                var marginTop = Number(scrollable.style.marginTop.replace("px","")) - self.scrollSpeed;
                if (marginTop <= self.scrollTopMin) {    // Auto-cancel when min reached.
                    marginTop  = self.scrollTopMin;
                    clearInterval(self.scrollTimer);
                    self.scrollTimer = null;
                }
                scrollable.style.marginTop = marginTop + "px";
                self.scrollSpeed += 1;
            }, 50);
        }, true);
        scrolldnButton.addEventListener("mouseup", function (ev) {
            clearInterval(self.scrollTimer);
            self.scrollTimer = null;
        }, true);

        this.window.appendChild(scrollupButton);
        for (var i=0; i < items.length; i++) {            
            scrollable.appendChild(createToggleItem(items[i], "CTXMN_",
                                    this.descriptor.selectedIDs.indexOf(items[i].id) != -1));
        }
        this.window.appendChild(scrollable);
        this.window.appendChild(scrolldnButton);
        
        var applyButton = document.createElement("button");
        applyButton.value     = "Apply";
        applyButton.innerHTML = "Apply";
        applyButton.addEventListener("click", onApply, false);
        
        this.window.appendChild(applyButton);   
    }
    else {
        for (var i=0; i < items.length; i++) {            // TODO: If vaue is current value, say 'selected'
			this.window.appendChild(createMenuItem(items[i], "CTXMN_", 
                                                   this.descriptor.selectedIDs.indexOf(items[i].id) != -1 || 
                                                   this.descriptor.value == items[i].id));
        } 
		for (var i=0; i < extraItems.length; i++) {            
			this.window.appendChild(createMenuItem(extraItems[i], "CTXMNEXTRA_", 
                                                   this.descriptor.selectedIDs.indexOf(items[i].id) != -1 || 
                                                   this.descriptor.value == items[i].id));
		}   
    }
    
    // First we display the menu, because otherwise it has no dimensions.
    this.window.style.display = "inline-block";

    // Check if we need to show the scroller buttons.
    if (this.window.clientHeight >= this.window.scrollHeight) {
        scrollupButton.style.display = scrolldnButton.style.display = "none";
        scrollable.style.marginTop    = "0px";
        scrollable.style.marginBottom = "28px";
    }
    else {
        scrollupButton.style.display = scrolldnButton.style.display = "block";
        scrollable.style.marginTop    = "10px";
        scrollable.style.marginBottom = "38px";
        this.scrollTopMax = 10;     // 10px is what we start with, to show 'UP' scroll button
        this.scrollTopMin = this.window.clientHeight - this.window.scrollHeight + this.scrollTopMax;
    }

    // Position the menu as best as we can.
    var pos = WindowManager.bestPosition(this.descriptor.x,        this.descriptor.w, 
                                         this.descriptor.y,        this.descriptor.h, 
                                         this.descriptor.offset.x, this.descriptor.offset.y, 
                                         this.window.offsetWidth,  this.window.offsetHeight, 
                                         this.descriptor.position, this.descriptor.position ? 10 : 0); 
                                         
    this.window.style.left = pos.x + "px";
    this.window.style.top  = pos.y + "px";                             
    this.window.setAttribute("orientation", this.descriptor.position ? pos.orientation : "nonotch");  
    this.window.setAttribute("alignment",   this.descriptor.position ? pos.alignment   : "nonotch");  
    
    WindowManager.raiseShield(this); 
}
 
DateTimePicker.show = function (eventManager, source, options) {
    TooltipManager.hide(true);  // We don't want the current tooltip, if any, to remain visible.
    
    this.hide();                // Clear any remains of a previous showing and start with a clean slate.
 
    if (!this.datetimeInput) {    
        this.datetimeInput = document.createElement("input");
        this.datetimeInput.id               = "datetimepicker";
        this.datetimeInput.className        = "clockpicker";
        this.datetimeInput.style.position   = "absolute";	
        this.datetimeInput.style.top        = "0px";		
        this.datetimeInput.style.left       = "0px";		
        this.datetimeInput.style.width      = "10px";		
        this.datetimeInput.style.height     = "10px";		
        this.datetimeInput.style.visibility = "hidden";	// It's a hidden thing with which we trigger the
                                                        // display of the time picker. It must be on the
                                                        // page, but we don't want to actually see it.

        document.body.appendChild(this.datetimeInput);           
    }
    
    this.eventManager = eventManager;
    this.source       = source;
    this.descriptor   = {
        element:   { x:0, y:0, h:0, w:0 },
        offset:    { x:0, y: 0 },
        nodeIDs    : null,
        value      : Math.floor((new Date()).getTime()/60000)*60000,  // 'Now', on minute boundary.
        callback   : null,
        timeconstraints: null,
        dateconstraints: null,
        type       : "datetime",
        orientation: "below-center"
    };
    $.extend(true, this.descriptor, options);
    
    // 'position' is our preferred way to refer to orientation. We allow both.
    this.descriptor.position = this.descriptor.position || this.descriptor.orientation;      
    this.descriptor.x  = this.descriptor.element.x || this.descriptor.x;
    this.descriptor.y  = this.descriptor.element.y || this.descriptor.y;
    this.descriptor.w  = this.descriptor.element.w || this.descriptor.w;
    this.descriptor.h  = this.descriptor.element.h || this.descriptor.h;

    var sourceRect = this.source && this.source.frameElement 
                   ? this.source.frameElement.getBoundingClientRect() 
                   : this.source && this.source.contentWindow
                   ? this.source.getBoundingClientRect() 
                   : { left:0, top:0, width:0, height:0 };
    this.descriptor.x += sourceRect.left;
    this.descriptor.y += sourceRect.top;
      
    this.datetimeInput.style.left = this.descriptor.x + (this.descriptor.w||10)/2  - 10/2 + "px";		
	this.datetimeInput.style.top  = this.descriptor.y + "px";	
  	this.datetimeInput.value      = this.descriptor.value || (new Date()).getTime(); 
    
    var self = this;
    $('#' + this.datetimeInput.id).clockpicker({
        autoclose: true,    
        beforeShow: function (picker) {
            self.picker = picker;
            self.window = picker.DateTimePicker[0];
            
            var verOrientation = self.descriptor.position.split("-")[0];
            var horOrientation = self.descriptor.position.split("-")[1];
                
            self.window.setAttribute("orientation", verOrientation);  
            self.window.setAttribute("alignment",   horOrientation);  
            
            picker.options.align = horOrientation;
            picker.options.type  = self.descriptor.type;
            picker.options.disableMinutes = self.descriptor.timeconstraints == "hours";
            picker.options.disablingRules = self.descriptor.dateconstraints;
        },
        afterShow: function (picker) {
            WindowManager.raiseShield(self); 					
        },
        afterHourSelect: function(picker) {
            console.log("after hour selected");
            // In some cases, this is enough. No minutes needed. We can close the dialog right now.
        },
        beforeDone: function(picker) {
            self.oldValue = picker.input.value;
        },
        afterDone: function(picker) {
            self.newValue = picker.input.value;
            if (self.newValue != self.oldValue) {
                if (self.descriptor.callback) 
                    self.descriptor.callback.call(self.source, self.descriptor.nodeIDs, self.newValue);
                else 
                    self.source.postMessage("DV.DATETIMEPICKER::UPDATE::" + self.newValue, "*");
            }
            self.hide();	// This closes the shield and cleans up admin.            
        }
    });			

    this.datetimeInput.style.display = "inline-block";	
    this.datetimeInput.click();
    
    var distToRight = -10 + window.document.body.offsetWidth - (this.window.offsetLeft + this.window.offsetWidth);
	if (distToRight < 0) this.window.style.left = this.window.offsetLeft + distToRight + "px";		    
}

DialogManager.prototype.show = function (descriptor, context, id, name) {
    TooltipManager.hide(true);  // We don't want the current tooltip, if any, to remain visible.

    if (!descriptor.getAttribute)
        descriptor.getAttribute = function (attr) { return descriptor[attr]; };
    
	var self = this;  
    var pos  = {};	
        
    if (!this.window) {   
        this.window = document.createElement("div");
        this.window.className = "Dialog";	
		this.window.style.padding = "0px";	
        this.window.style.display = "none";
        this.window.style.overflowY = "hidden";
        this.window.style.overflowX = "hidden";
		this.window.style.resize = "both";
        
        document.body.appendChild(this.window);   
    }
    
    this.window.innerHTML = "";
   
    this.id           = id;
    this.name         = name || id;
    this.context      = context;
    
    var titleBar = document.createElement("DIV");
    titleBar.className = "DialogHeader";
    this.window.appendChild(titleBar);
	
    titleBar.addEventListener("mousedown", function(event) { WindowManager.onStartDrag(event, titleBar); });
    titleBar.addEventListener("mouseup",   function(event) { WindowManager.onEndDrag(  event, titleBar); });

    var closeButton = document.createElement("SPAN");
    closeButton.className = "icon-radio-checked2";
    closeButton.innerHTML = " ";
    closeButton.addEventListener("click",  function(event) { self.hide(event); });
    titleBar.appendChild(closeButton);
    
    var title = document.createTextNode(descriptor.getAttribute("title"));    
    titleBar.appendChild(title);

    if (descriptor.getAttribute("from") == "html") {
        // The notification text - may be used by 'notification' action if a dialog is open.
        this.notificationArea = document.createElement("DIV");
        this.notificationArea.style.display = "none";	
        this.notificationArea.style.padding = "8px 12px 0px 12px";
        this.notificationArea.style.fontStyle = "italic";
        this.notificationArea.style.fontWeight = "bold";
        this.notificationArea.style.color = "red";
        this.window.appendChild(this.notificationArea);

        // We have html content for our dialog.
        var introArea = document.createElement("DIV");
        introArea.className = "DialogIntro Scrollbar";
        introArea.style.height = "calc(100% - 40px)";
        introArea.style.overflowY = "auto";
        this.window.appendChild(introArea);
    
        introArea.innerHTML = descriptor.getAttribute("contents");

        // We need to take the buttons from the descriptor, and let the requestor know what the user
        // clicked.
        var buttons = descriptor.getAttribute("buttons");
        if (buttons) {
            introArea.style.height = "calc(100% - 80px)";

            var buttonBar = document.createElement("DIV");
            buttonBar.className = "DialogFooter";

            buttons.split(",").forEach(title => {
                const button = document.createElement("BUTTON");
                button.type  = "button";
                button.value = title;
                button.innerHTML = title;
                button.addEventListener("click", function(e) { 
                    self.onClose(title, introArea.innerHTML); 
                });
                buttonBar.appendChild(button);
            });

            this.window.appendChild(buttonBar);   
        }

        pos = {
			left:   descriptor.getAttribute("left")   || "50%",
			top:    descriptor.getAttribute("top")    || "100px",
			width:  descriptor.getAttribute("width")  || "500px",
			height: descriptor.getAttribute("height") || "300px"
		}
    }
	else if (descriptor.getAttribute("from") == "url") {
		var dialogURL = descriptor.getAttribute("location");
		var iframeElem = document.createElement("IFRAME");
		iframeElem.scrolling = descriptor.getAttribute("scroll") || "no";
		iframeElem.sandbox   = "allow-forms allow-same-origin allow-scripts allow-popups";
		iframeElem.seamless  = "seamless";
		iframeElem.name      = "DIALOG_" + this.id;
		
		iframeElem.width        = "100%";
		iframeElem.style.width  = "100%";
		iframeElem.style.top    = "29px";
		iframeElem.style.height = "calc(100% - 29px)";
		iframeElem.style.border = "0px";
	//	iframeElem.style.position = "absolute";
		iframeElem.style.verticalAlign = "top";
		
		this.window.appendChild(iframeElem);
		
		this.eventManager.pushDialog(this);	// Now the event manager can find us when the dialog calls back.		
		iframeElem.src = dialogURL;

		pos = {
			left:   descriptor.getAttribute("left")   || "50%",
			top:    descriptor.getAttribute("top")    || "100px",
			width:  descriptor.getAttribute("width")  || "500px",
			height: descriptor.getAttribute("height") || "300px"
		}
	}
	else {
		// We have to build the dialog ourselves.
        var introArea = document.createElement("DIV");
        introArea.className = "DialogIntro";
		this.window.appendChild(introArea);
    
        var intro = document.createTextNode(descriptor.getAttribute("intro"));
        introArea.appendChild(intro);
        
        this.table = document.createElement("TABLE");
        this.window.appendChild(this.table);
        
        var fields = descriptor.getAttribute("fields");
        var items  = fields.split(";");
        // fields = assignid:User ID:list    :[dashboard.userlist]:[dashboard.userid]:true;note:Note:textarea:2000::true
        // items  = assignid:User ID:list    :[dashboard.userlist]:[dashboard.userid]:true
        //          note    :Note   :textarea:2000                :                  :true
    
        for (var i=0; i < items.length; i++) {    
            var specs = items[i].split(":");
                    
            var fieldRow = document.createElement("TR");
            var fieldId  = specs[0];
            fieldRow.setAttribute("fieldId", fieldId);
            
            var fieldCell = document.createElement("TD");
            fieldCell.appendChild(document.createTextNode(specs[1]));
            fieldRow.appendChild(fieldCell);
            
            fieldCell = document.createElement("TD");
            var inputControl = null;
            var format = specs[2];
            switch (format) {
                case "password":
                    inputControl   = document.createElement("INPUT");
                    inputControl.type  = "password";
                    inputControl.value = "";
                    break;
                case "list":
                    inputControl   = document.createElement("SELECT");
                    var options = this.resolver.resolve(specs[3]);
                    if (typeof options == "string") {
                        options = options.split(",");
                    }
                    for (var j=0; j < options.length; j++) {
                        var option = document.createElement("OPTION");
                        if (typeof options[j] == "string") {
                            option.value     = options[j];
                            option.innerHTML = options[j];                   
                        }
                        else {
                            option.value     = options[j].id;
                            option.innerHTML = options[j].label;                                       
                        }
                        
                        inputControl.appendChild(option);
                    }
                    break;
                case "number":
                    inputControl   = document.createElement("INPUT");
                    inputControl.type  = "number";
                    inputControl.value = "";
                    inputControl.min   = "0";
                    inputControl.max   = "10";
                    break;
                case "date":
                    inputControl   = document.createElement("INPUT");
                    inputControl.type  = "date";
                    inputControl.value = "";
                    break;
                case "datetime":
                    inputControl   = document.createElement("INPUT");
                    inputControl.type  = "datetime";
                    inputControl.value = "";
                    break;
                case "time":
                    inputControl   = document.createElement("INPUT");
                    inputControl.type  = "time";
                    inputControl.value = "";
                    inputControl.min   = "2000-01-01";
                    inputControl.max   = "2020-01-01";
                    break;
                case "checkbox":
                    inputControl   = document.createElement("INPUT");
                    inputControl.type  = "checkbox";
                    inputControl.value = "";
                    break;
                case "email":    
                    inputControl   = document.createElement("INPUT");
                    inputControl.type  = "email";
                    inputControl.value = "";
                    inputControl.autocomplete = "on";
                    break;
                case "file":    
                    inputControl   = document.createElement("INPUT");
                    inputControl.type  = "file";
                    inputControl.value = "";
                    inputControl.multiple = true;
                    break;
                case "textarea":
                    inputControl   = document.createElement("TEXTAREA");
                    var maxChars = specs[3] || 500;
                    inputControl.rows = Math.max(Math.round(maxChars/100), 5);
                    inputControl.maxlength = maxChars;
                    break;
                case "text":
                default:
                    inputControl   = document.createElement("INPUT");
                    inputControl.type  = "text";
                    inputControl.value = "";
                    break;
            }
            
            var defaultValue = this.values && this.values.hasOwnProperty(fieldId) 
                            ? (this.values[fieldId].hasOwnProperty("id") ? this.values[fieldId].id : this.values[fieldId])
                            : this.resolver.resolve(specs[4], this.context, true);
    
            inputControl.value = defaultValue;
            
            fieldCell.appendChild(inputControl);
            fieldRow.appendChild(fieldCell);

            this.table.appendChild(fieldRow);
        }
        
        // The notification text - may be used by 'notification' action if a dialog is open.
        this.notificationArea = document.createElement("DIV");
        this.notificationArea.style.display = "none";	
        this.notificationArea.style.paddingLeft = "12px";
        this.notificationArea.style.paddingBottom = "4px";
        this.notificationArea.style.fontStyle = "italic";
        this.window.appendChild(this.notificationArea);
        
        // And then the buttons: OK, Cancel
        var buttonBar = document.createElement("DIV");
        buttonBar.className = "DialogFooter";
        
        // We need to take the buttons from the descriptor, and let the requestor know what the user
        // clicked.
        var cancelButton = document.createElement("BUTTON");
        cancelButton.type  = "button";
        cancelButton.value = "Cancel";
        cancelButton.innerHTML = "Cancel";
        cancelButton.addEventListener("click", function(e) { self.onCancel(); });
        buttonBar.appendChild(cancelButton);
        
        var okButton = document.createElement("BUTTON");
        okButton.type  = "button";
        okButton.value = "OK";
        okButton.innerHTML = "OK";
        okButton.addEventListener("click", function(e) { self.onSubmit(); });
        buttonBar.appendChild(okButton);

        this.window.appendChild(buttonBar);
        
        pos = {
            left: "50%",
            top: "100px",
            width: "300px",
            height: ""
        }
	}
                                              
    this.window.style.left    = pos.left;
    this.window.style.top     = pos.top;                            
    this.window.style.width   = pos.width;                            
    this.window.style.height  = pos.height;                            
    this.window.style.display = "inline-block";
    
    WindowManager.raiseShield(this, true); // Modal    
}

ContextMenuManager.cancel = function () {
    if (this.source.postMessage)
        this.source.postMessage("DV.CONTEXTMENU::CLOSE", "*");    
} 

InfoWindowManager.cancel = function () {
    if (this.source.postMessage)
        this.source.postMessage("DV.INFOWINDOW::CLOSE", "*");
} 
 
DateTimePicker.cancel = function () {
    if (this.source.postMessage)
        this.source.postMessage("DV.DATETIMEPICKER::CLOSE", "*");
	if (this.picker) 
        this.picker.hide();
}   

TooltipManager.hide = function (forced) {
    if (forced) this.frozen = false;
    
    if (!this.window || this.frozen) return;
    
    if (WindowManager.isShieldUp(this))  {
        WindowManager.lowerShield(this);  
        
        this.window.classList.remove("Frozen");
    //    this.window.style.border = "";
    //    this.window.style.borderRadius = "";
    //    this.window.style.overflowY = "";
        this.frozen = false;
    }
    
    this.window.style.visibility = "hidden";
    return null;
}
 
InfoWindowManager.hide = function() {
    if (this.window) {
        this.window.style.display = "none";
        this.window.innerHTML     = "";
        /*  For now we will leave the window in existence because otherwise we'd have to
            recreate it every single time. That may not be a big deal, but we'll see.
         */
    //    document.body.removeChild(this.window);
    //    this.window = null;
        
        WindowManager.lowerShield(this);
        
        // Ensure we erase current 'context'		
        this.descriptor = null;
    }
    return null;
}

ContextMenuManager.hide = function() {
    if (this.window) {
        this.window.style.display = "none";
        this.window.innerHTML     = "";
        
        WindowManager.lowerShield(this);
        
        // Ensure we erase the current 'context'.
        this.descriptor   = null;
        this.source       = null;
        this.eventManager = null;
    }
    return null;
}

DateTimePicker.hide = function() {
    if (this.datetimeInput) {
        this.datetimeInput.style.display = "none";
        this.datetimeInput.value = "";
       
        WindowManager.lowerShield(this);
		       		
        // Ensure we erase the current 'context'.
        this.descriptor   = null;
        this.source       = null;
        this.eventManager = null;
    }
    return null;
}

DialogManager.prototype.hide = function() {
    if (this.window) {
        this.window.style.display = "none";
        this.window.innerHTML     = "";

        WindowManager.lowerShield(this);
        
        this.context = null;
        this.values  = null;
    }
    return null;
}

TooltipManager.freeze = function () {
    if (!TooltipManager.window) return;
   
    WindowManager.raiseShield(this, true);  // Modal
 
    TooltipManager.window.classList.add("Frozen");
    
    for (let i = 0; i < TooltipManager.window.children.length; i++) {
        TooltipManager.window.children[i].style.height = "";
    }
//    TooltipManager.window.style.border    = "3px solid white";
//    TooltipManager.window.style.borderRadius = "6px";
//    TooltipManager.window.style.overflowY = "auto";
    
    if (TooltipManager.moreIndicator) TooltipManager.moreIndicator.style.display = "none";
    
    TooltipManager.frozen = true;
}

DialogManager.prototype.setMessage = function (text) {
	if (!this.notificationArea) return false;
	text = text.trim();
	this.notificationArea.innerHTML = text;
    this.notificationArea.style.display = text == "" ? "none" : "block";
    this.notificationArea.scrollTop = 0;
	return true;
}
 
DialogManager.prototype.onClose = function (button, data) {  
    var event = {
        id:         this.id,
        name:       this.name,
        response:   { button: button },
		data:       data
    };
    this.eventManager.replyToPublisher(this.source, event);
//    this.hide();                     
}
 
DialogManager.prototype.onCancel = function () {  
    var event = {
        id:         this.id,
        name:       this.name,
        response:   { button: "CANCEL" }
    };
    this.eventManager.replyToPublisher(this.source, event);
//    this.hide();                     
}

DialogManager.prototype.onSubmit = function () {  
    // Peel out the values the user entered and notify the source.
    this.values = {};
    
    var rows = this.table.getElementsByTagName("TR");
    for (var i=0; i < rows.length; i++) {
        var fieldId = rows[i].getAttribute("fieldId");
        
        var cells = rows[i].getElementsByTagName("td");
        var input = cells[cells.length-1].childNodes[0];
        
        if (input.nodeName.toLowerCase() == "select") 
            this.values[fieldId] = { id: input.value, label:input.options[input.selectedIndex].innerHTML };
        else
            this.values[fieldId] = input.value;
    }
    
    var event = {
        id:         this.id,
        name:       this.name,   
        response:   { button: "OK",
                      fields: this.values }
    };
    this.eventManager.replyToPublisher(this.source, event);
//    this.hide();        
}
