# Interface: DatabaseAPI


## DatabaseAPI

**This is the interface that database connectors must implement.**





### Constructor


#### new DatabaseAPI<span class="signature">(config, logger)</span>


Connect to the database using the options specified in the 'config'
object, and initialize a connection pool.






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`config`|*Object*|The configuration settings for this adapter.|
|`logger`|*Object*|The logger at which can be logged.|























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L6">line 6</a>























---------------











### Members


#### <span class="type-signature"></span>activeConnections<span class="type-signature"></span>


Return the number of currently active connections.






















*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L47">line 47</a>











#### <span class="type-signature"></span>totalConnections<span class="type-signature"></span>


Return the maximum number of available connections.






















*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L52">line 52</a>













### Methods






#### <span class="type-signature">(async) </span>acquire<span class="signature">(connection, onAcquired<span class="signature-attributes">opt</span>)</span><span class="type-signature"></span>


Acquire a connection to the database for the provided <a href="/doc/index.html#/apis/DatabaseConnection">DatabaseConnection</a>.

First set the <a href="/doc/index.html#/apis/DatabaseConnection?id=state">DatabaseConnection#state</a> value to "Connecting".
Next, acquire a connection ('conn') from the pool.
Once available, call <a href="/doc/index.html#/apis/DatabaseConnection?id=initialize">DatabaseConnection#initialize</a> with that connection (i.e. 'conn').
After that, call <a href="/doc/index.html#/apis/DatabaseConnection?id=resolve">DatabaseConnection#resolve</a> with the provided <a href="/doc/index.html#/apis/DatabaseConnection">DatabaseConnection</a>.
Finally, call 'onAcquired()' (if specified).

If a connection cannot be acquired,
call <a href="/doc/index.html#/apis/DatabaseConnection?id=cleanup">DatabaseConnection#cleanup</a> with the provided <a href="/doc/index.html#/apis/DatabaseConnection">DatabaseConnection</a>, and
call <a href="/doc/index.html#/apis/DatabaseConnection?id=reject">DatabaseConnection#reject</a> with the error.






##### Parameters:

|Name|Type|Argument|Description|
|----|----|--------|-----------|
|`connection`|*<a href="/doc/index.html#/apis/DatabaseConnection">DatabaseConnection</a>*|  |The connection object created by <a href="/doc/index.html#/apis/DatabaseAPI?id=createConnection">DatabaseAPI#createConnection</a>.|
|`onAcquired`|*function*|*optional*  |The function to be called once the connection has been acquired.|























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L42">line 42</a>




























#### <span class="type-signature">(async) </span>close<span class="signature">(resolve)</span><span class="type-signature"></span>


Close the connection pool. Call resolve() afterwards.






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`resolve`|*function*||























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L59">line 59</a>




























#### <span class="type-signature"></span>createConnection<span class="signature">(name, resolve, reject, cleanup)</span><span class="type-signature"> &rarr; {<a href="/doc/index.html#/apis/DatabaseConnection">DatabaseConnection</a>}</span>


Creates and returns a <a href="/doc/index.html#/apis/DatabaseConnection">DatabaseConnection</a> object, initialized with the parameters below.
The object's <a href="/doc/index.html#/apis/DatabaseConnection?id=state">DatabaseConnection#state</a> value is initialized to "Queued".






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`name`|*string*|The name of the connection. This name will be used by the DatabaseTracker to serialize requests for a database connection by the same task.|
|`resolve`|*function*|(Private) Function to be called when a connection has been acquired.|
|`reject`|*function*|(Private) Function to be called when a connection cannot be acquired.|
|`cleanup`|*function*|(Private) Function to be called when a connection has been released.|























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L24">line 24</a>



















##### Returns:



    *<a href="/doc/index.html#/apis/DatabaseConnection">DatabaseConnection</a>*