# Interface: DatabaseConnection


## DatabaseConnection

**A connection object that exposes standardized methods to interact with the database.
An instance of this class can only be created by <a href="/doc/index.html#/apis/DatabaseAPI?id=createConnection">DatabaseAPI#createConnection</a>.
The <a href="/doc/index.html#/apis/DatabaseConnection?id=state">DatabaseConnection#state</a> value is initialized to "Queued".

This is the interface that database connections must implement.**





### Constructor


#### new DatabaseConnection<span class="signature">(adaptor, config, logger)</span>









##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`adaptor`|*Object*|An adaptor that inherits from <a href="/doc/index.html#/apis/DatabaseAPI">DatabaseAPI</a>.|
|`config`|*Object*|The configuration settings for this adapter.|
|`logger`|*Object*|The logger at which can be logged.|























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L70">line 70</a>























---------------











### Members


#### <span class="type-signature"></span>cleanup<span class="type-signature">: $ {
        types.join('|')
    }
</span>


Function to be called when a connection has been released.


##### Type:
_*function*_





















*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L91">line 91</a>











#### <span class="type-signature"></span>name<span class="type-signature">: $ {
        types.join('|')
    }
</span>


The name of the connection. This name will be used by
the DatabaseTracker to serialize requests for a database connection by the same task.


##### Type:
_*string*_





















*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L81">line 81</a>











#### <span class="type-signature"></span>reject<span class="type-signature">: $ {
        types.join('|')
    }
</span>


Function to be called when a connection cannot be acquired.


##### Type:
_*function*_





















*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L89">line 89</a>











#### <span class="type-signature"></span>resolve<span class="type-signature">: $ {
        types.join('|')
    }
</span>


Function to be called when a connection has been acquired.


##### Type:
_*function*_





















*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L87">line 87</a>











#### <span class="type-signature"></span>state<span class="type-signature">: $ {
        types.join('|')
    }
</span>


The state of the connection. Valid values are "Queued",
"Connecting" and "Connected".


##### Type:
_*string*_





















*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L85">line 85</a>













### Methods






#### <span class="type-signature">(async) </span>begin<span class="signature">()</span><span class="type-signature"></span>


Start a new transaction. The 'commit()' or 'rollback()' method will be called later.


























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L108">line 108</a>




























#### <span class="type-signature">(async) </span>commit<span class="signature">(onDone, onError)</span><span class="type-signature"></span>


Commit transaction and release the connection. Call 'onDone()' or 'onError(err)' when done.






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`onDone`|*function*||
|`onError`|*function*||























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L134">line 134</a>




























#### <span class="type-signature"></span>initialize<span class="signature">(conn)</span><span class="type-signature"></span>


This method should only be called by <a href="/doc/index.html#/apis/DatabaseAPI?id=acquire">DatabaseAPI#acquire</a>. The 'conn'
argument is the value passed by 'acquire()', and for which the methods below
will be called. it is recommended to save this value as 'this.conn'.
Upon completion of this method the <a href="/doc/index.html#/apis/DatabaseConnection?id=state">DatabaseConnection#state</a> value must
be set to "Connected". Note that the other methods only need to be available
after this method has been called.






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`conn`|*object*||























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L102">line 102</a>




























#### <span class="type-signature">(async) </span>query<span class="signature">(statement, values<span class="signature-attributes">opt</span>)</span><span class="type-signature"></span>


Return a promise to execute the statement, using the values if specified.






##### Parameters:

|Name|Type|Argument|Description|
|----|----|--------|-----------|
|`statement`|*string*|  |SQL statement, optionally with '?' placeholders to be
resolved against the provided values.|
|`values`|*Array.&lt;string>*|*optional*  |Values for the placeholders.|























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L117">line 117</a>




























#### <span class="type-signature">(async) </span>release<span class="signature">(onReleased, noCleanup<span class="signature-attributes">opt</span>)</span><span class="type-signature"></span>


Release the connection. Call 'cleanup()' if noCleanup not 'true'. Call onReleased() when done.






##### Parameters:

|Name|Type|Argument|Default|Description|
|----|----|--------|-------|-----------|
|`onReleased`|*function*|  |||
|`noCleanup`|*boolean*|*optional*  |false||























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L150">line 150</a>




























#### <span class="type-signature">(async) </span>rollback<span class="signature">(onDone, onError)</span><span class="type-signature"></span>


Rollback transaction and release the connection. Call 'onDone()' or 'onError(err)' when done.






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`onDone`|*function*||
|`onError`|*function*||























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L142">line 142</a>




























#### <span class="type-signature">(async) </span>stream<span class="signature">(statement, values<span class="signature-attributes">opt</span>)</span><span class="type-signature"></span>


Execute the statement, using the values if specified, and stream the result back.






##### Parameters:

|Name|Type|Argument|Description|
|----|----|--------|-----------|
|`statement`|*string*|  |SQL statement, optionally with '?' placeholders to be
resolved against the provided values.|
|`values`|*Array.&lt;string>*|*optional*  |Values for the placeholders.|























*Source:*
<a href="/doc/index.html#/apis/DatabaseManager.js">DatabaseManager.js</a>, <a href="/doc/index.html#/apis/DatabaseManager.js?id=L126">line 126</a>