# Interface: DestinationAPI


## DestinationAPI

**This is the interface that destination adaptors must implement.**





### Constructor


#### new DestinationAPI<span class="signature">(config)</span>


Create an instance of the DestinationAPI. During the contruction the instance
may prepare to connect to its output (if appropriate). The instance has to
emit a 'ready' event (asynchronously) once it has fully initialized and is ready
to accept calls to <a href="/doc/index.html#/apis/DestinationAPI?id=open">DestinationAPI#open</a>.






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`config`|*object*|The configuration object as extracted for this adaptor
from config.json. Note that the config object will also include a 'formatter'
property. This property needs to be exposed at a public getter of this instance.|























*Source:*
<a href="/doc/index.html#/apis/DestinationAPI.js">DestinationAPI.js</a>, <a href="/doc/index.html#/apis/DestinationAPI.js?id=L6">line 6</a>























---------------











### Members


#### <span class="type-signature"></span>formatter<span class="type-signature"></span>


The formatter as provided in the config object used to construct this instance.






















*Source:*
<a href="/doc/index.html#/apis/DestinationAPI.js">DestinationAPI.js</a>, <a href="/doc/index.html#/apis/DestinationAPI.js?id=L40">line 40</a>











#### <span class="type-signature"></span>produce<span class="type-signature"></span>


Holds the strategy used by this adaptor when a job contains related entities.
Some implementations of this class are able to handle graphs (nodes and edges),
while others can only deal with arrays of tuples containing 'source' and 'target'
properties, and yet others only support arrays of messages.
The preferred value comes from the 'config' object passed to the constructor,
but it is up to the adaptor to set this property to the best supported value.
Valid values are "entities" (no relationships accepted), "linkages" (tuples only)
and "graphs" (all nodes and edges).






















*Source:*
<a href="/doc/index.html#/apis/DestinationAPI.js">DestinationAPI.js</a>, <a href="/doc/index.html#/apis/DestinationAPI.js?id=L35">line 35</a>













### Methods






#### <span class="type-signature"></span>on<span class="signature">(event, listener)</span><span class="type-signature"></span>


Subscribe to an event.






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`event`|*string*|Valid values are 'error' and 'ready'.|
|`listener`|*function*|The listener to this event.|























*Source:*
<a href="/doc/index.html#/apis/DestinationAPI.js">DestinationAPI.js</a>, <a href="/doc/index.html#/apis/DestinationAPI.js?id=L23">line 23</a>




























#### <span class="type-signature">(async) </span>open<span class="signature">(job, fieldGetter, onConnectionOpen)</span><span class="type-signature"></span>


Opens a connection to whatever destination the subclass supports. Upon completion
this method must call 'onConnectionOpen(error, destConn)' (asynchronously) with a
<a href="/doc/index.html#/apis/DestinationConnection">DestinationConnection</a> object at which messages can be sent.






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`job`|*object*|The job. Notable properties in the job are 'name', 'cycle' and 'tenant'.|
|`fieldGetter`|*function*|A function that will return an object with a 'headers'
property and a 'columns' property. This is useful for tabular outputs where headers need
to be written first, or for any output where the names of the available fields need to be
known in advance. The headers constitute all headers that need to be written to the output
in scenarios where entity relationships are involved. The columns represent just the names
of the fields found/created for entities while generating messages.
generating the messages.|
|`onConnectionOpen`|*function*|The function that needs to be called upon completion.
It must pass a <a href="/doc/index.html#/apis/DestinationConnection">DestinationConnection</a> object at which messages can be sent.|























*Source:*
<a href="/doc/index.html#/apis/DestinationAPI.js">DestinationAPI.js</a>, <a href="/doc/index.html#/apis/DestinationAPI.js?id=L58">line 58</a>