# Class: DynatraceAPI


## DynatraceAPI

**Provides load balancing and rate-limited access to a pool of hosts that
expose the Dynatrace API.**





### Constructor


#### <span class="type-signature"></span>new DynatraceAPI<span class="signature">(config, limits)</span><span class="type-signature"></span>


Creates a pool of connections to the Dynatrace API for multiple tenants and clusters.






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`config`|*object*|The configuration for the tenants.|
|`limits`|*object*|The rate limits, timeouts and queue sizes.|























*Source:*
<a href="/doc/index.html#/apis/DynatraceAPI.js">DynatraceAPI.js</a>, <a href="/doc/index.html#/apis/DynatraceAPI.js?id=L116">line 116</a>























---------------













### Methods






#### <span class="type-signature"></span>getHealth<span class="signature">(frequency<span class="signature-attributes">opt</span>, callback<span class="signature-attributes">opt</span>)</span><span class="type-signature"></span>


Report the health of the connection pool. If no parameters are specified, the health
report will be returned. If both parameters are specified, teh callback will be called
with the health report as its only parameter, per the specified frequency.






##### Parameters:

|Name|Type|Argument|Description|
|----|----|--------|-----------|
|`frequency`|*number*|*optional*  |The requency with which the health of the
connection pool should be reported.|
|`callback`|*function*|*optional*  |The function that consumes the health report.|























*Source:*
<a href="/doc/index.html#/apis/DynatraceAPI.js">DynatraceAPI.js</a>, <a href="/doc/index.html#/apis/DynatraceAPI.js?id=L147">line 147</a>




























#### <span class="type-signature"></span>request<span class="signature">(requestOptions, callback)</span><span class="type-signature"> &rarr; {<a href="/doc/index.html#/apis/CancellableEventEmitter">CancellableEventEmitter</a>|<a href="/doc/index.html#/apis/CancellablePromise">CancellablePromise</a>}</span>


Submit a request to the Dynatrace API. The DynatraceAPI will find the best
host to handle the request, or queue the request if necessary.






##### Parameters:

|Name|Type|Description|
|----|----|-----------|
|`requestOptions`|*<a href="/doc/index.html#/apis/global?id=RequestOptions">RequestOptions</a>*|The parameters for the request.|
|`callback`|*<a href="/doc/index.html#/apis/global?id=RequestCallback">RequestCallback</a>*|(Optional) The callback that handles the response. If specified, this method returns a <a href="/doc/index.html#/apis/CancellableEventEmitter">CancellableEventEmitter</a>. If not specified, this method returns a <a href="/doc/index.html#/apis/CancellablePromise">CancellablePromise</a>.|























*Source:*
<a href="/doc/index.html#/apis/DynatraceAPI.js">DynatraceAPI.js</a>, <a href="/doc/index.html#/apis/DynatraceAPI.js?id=L127">line 127</a>



















##### Returns:

Either (1) an object at
which the progress of the request can be monitored and at which the request can
be canceled, or (2) a Promise(-like) object at which the request can be canceled.

    *<a href="/doc/index.html#/apis/CancellableEventEmitter">CancellableEventEmitter</a>*|*<a href="/doc/index.html#/apis/CancellablePromise">CancellablePromise</a>*











#### <span class="type-signature"></span>restart<span class="signature">()</span><span class="type-signature"></span>


Cancel all running and/or queued requests and reset the connection pool.


























*Source:*
<a href="/doc/index.html#/apis/DynatraceAPI.js">DynatraceAPI.js</a>, <a href="/doc/index.html#/apis/DynatraceAPI.js?id=L132">line 132</a>




























#### <span class="type-signature"></span>shutdown<span class="signature">()</span><span class="type-signature"></span>


Cancel all running and/or queued requests and delete the connection pool.


























*Source:*
<a href="/doc/index.html#/apis/DynatraceAPI.js">DynatraceAPI.js</a>, <a href="/doc/index.html#/apis/DynatraceAPI.js?id=L137">line 137</a>