Fault Tolerance
=====

## Data Loss Prevention
Workers work off their assigned jobs, handed to each individual Worker at each cycle. If a Worker cannot finish the work it was assigned within the allocated time, it will get three additional minutes before it is forcefully terminated, and the remainder of its metric requests discarded. This condition will result in metric value gaps for some entities in the eventual data set in the target repository. 

When the collection of scheduled jobs is too big to be executed in the allotted time, additional Workers will be created. If the maximum number of Workers is reached, jobs are selected for demotion to a lower frequency (largest jobs first). 

A Worker may also occasionally encounter failing requests. Where that happens, it will inform the StreamerService. This condition will be logged, and metric values may be lost.

## Endpoint Failover
If a Worker has trouble connecting to an endpoint, it moves on to the next candidate. If this leads to capacity problems, the granularity will be reduced (adding Workers cannot compensate for a reduction in endpoints). It will be up to a human operator to examine and remediate the problem.

## Worker Failover
The StreamerService continuously observes its Workers to see if they are still up and operational. If one dies, it will create new ones as needed. Such conditions will be logged. The availability of the StreamerService is essential as it provides the heartbeat for each cycle. While the StreamerService and the Web App run in the same process, a ‘watchdog’ process may be added in the future.
 
## StreamerService Failover
The StreamerService itself may go down. While a PM2 process manager, if used, would automatically restart the StreamerService when that happens, it is also possible to launch a failover instance of the StreamerService on the same or a different machine. This failover instance periodically (every 10 seconds) checks the health of the primary instance and takes over if it fails to get an affirmative answer 5 consecutive times. When the primary instance comes back up, the failover instance will return to its ‘passive’ mode.
The failover instance receives its configuration and jobs from the primary instance, and is automatically kept up to date when configuration, jobs, tuning information and schedules change.

## LibraryService Failover
When the StreamerService starts the LibraryService process(es) autonomously, it will automatically restart failing LibraryService processes. If LibraryService processes are started ‘externally’, the StreamerService will periodically attempt to check if a failing LibraryService process has been restarted yet (either manually or by a NodeJS process manager like PM2). In the meantime, it will reuse previously received answers from a failing LibraryService until it can successfully query that LibraryService again. 

TODO: Run LibraryService failover instance in passive mode.

## Data Duplication Prevention
Since every cycle starts on a strict 1-minute boundary and absolute timeframes (with a 1-minute delay, based on the timestamp of each cycle) are used to request data from the Dynatrace API, requesting (and processing) data twice for the same timeframe is avoided. When the implementation starts to ‘stagger’ requests, absolute timestamps ensure consistency of the requested timeframes.

## Dynatrace API Overload Protection
The highly optimized implementation of the StreamingAPI carries the very real risk of overloading the Dynatrace API and affecting the overall performance of the Cluster nodes. To prevent this, all access to the Cluster is throttled and guaranteed not to exceed both (1) a user-configurable maximum number of requests per minute and (2) any request rate limits communicated by the Dynatrace API itself. Additionally, the LibraryService will continuously adjust its refresh cycle to stay within the configured rate limits as well as observed database access limits.
