Throughput Optimization
=====

To maximize throughput and resilience of the solution, the retrieval and posting of metrics is performed by a variable number of Worker processes. A StreamerService process is responsible for determining and instantiating the optimal number of Workers and distribute the load evenly across them on an ongoing basis. The load is defined in terms of the job configuration: source and a destination, a set of implicitly or explicitly referenced entities (or user sessions), any number of metrics, and a preferred frequency (schedule). 

Each Worker is responsible for maximizing its own throughput, based on the assigned work and the endpoints at its disposal (which it utilizes in a round-robin fashion). If it nonetheless falls behind, the StreamerService will take notice, create extra Workers, and redistribute the load. When the StreamerService fails to observe increasing throughput as it creates Worker processes, or if it reaches the maximum number of Workers, it will start to decrease the granularity with which metrics are retrieved, starting with the largest jobs in the offending schedule.

The LibraryService component is responsible for ensuring that the appropriate entities are included in each job, every time that job executes. There are two reasons for its existence: (1) the properties of each entity in a job change over time, and (2) the entities included in a criteria/filter-based job will vary as entities come into existence or get retired by Dynatrace. Querying the Dynatrace API for each job would place an impossible burden on the Cluster nodes. Additionally, the LibraryService supports criteria that the Dynatrace API does not support, like negative criteria, criteria based on properties, criteria that specify multiple options, and criteria that leverage more advanced operators. 

A web application, hosted by the same NodeJS process that runs the StreamerService, exposes the UI in which a user can select entities (or criteria), user sessions, metrics and frequencies. It will save the selections in a configuration file (/config/jobs.json). The UI leverages the LibraryService to know which filter options are available to the user.

## StreamerService
The StreamerService starts with a collection of jobs, each constituting a set of entity IDs or filter criteria, one or more metric IDs, and a desired granularity (i.e. minute-level or coarser aggregation). It starts by assigning all jobs to a single Worker process. Every minute it assigns new jobs to all Workers deemed necessary to perform the work within the allotted time) to start the next cycle. It also keeps track of Workers that have not yet sent all their "Job Done" messages for their previous cycle so that it can adjust its capacity. Note that jobs are split into smaller jobs where necessary to obtain an even spread among the available Workers.

When a Worker cannot handle the load (i.e. it has not yet published all its "Job Done" messages by the time the next cycle needs to start), it asks those Workers to finish up and retire while creating replacement Workers, plus one extra. The StreamerService then redistributes the work among the new (and now grown) collection of Workers.

If the percentage of completed work stops growing on addition of new Workers (up to a configured maximum), the StreamerService will start to decrease the granularity of individual jobs (selected in order of size). When the time to complete all work successfully falls under 30 seconds, the StreamerService will start to reduce the number of Workers. This procedure continues until the ‘goldilocks’ zone is found for each schedule. These tuning values are stored in the /config/tuning.json file so that they survive restarts. Note that any update to the /config/jobs.json file will reset the assigned granularities of all jobs so that previously demoted jobs are given a chance to run at their assigned schedule again.

Additionally, the "Health" monitoring page allows users to make ad-hoc adjustments to schedules.

## Workers
Every Worker receives a collection of jobs from the StreamerService. The Worker executes its jobs in parallel up to a configurable limit. Upon completion of a thread it takes the next pending job and executes it. The maximum number of threads can be configured in the /config/config.json file. On completion of a job, the Workers sends a "Job Done" message to the StreamerService process.

## LibraryService
The LibraryService contains a complete accounting of all entities in Dynatrace. It keeps itself up-to-date by scheduling a series of ‘pipelines’ that run in a rotating fashion, at intervals that collectively cover a complete (configurable) ‘refresh cycle’. To obtain a complete set of entities, the LibraryService supports any of the following strategies: 
1. Retrieve all entities of a specific type at once (including or excluding relationship info), or 
2. Iterate over all provided values of a specified tag or management zone, or 
3. Discover new entities by examining dependencies with already-retrieved entities and retrieving them in chunks, or
4. Don’t retrieve entities of a specific type at all.

The LibraryService subsequently indexes every entity by all tags, zones, groups and stringify-able properties. The entities themselves are only kept in memory when they have been recently requested by any client of the LibraryService (like the StreamerService). Whenever it is asked for a collection of entities by a set of criteria, it ‘AND’s the matching index lists and retrieves the matching entities from its database (or cache). The database is always kept up-to-date with respect to what is indexed. This also enables the LibraryService to start up and ‘remember’ what it previously discovered and indexed.
