config.json
=====
This is the primary configuration file for the Dynatrace ETL Service. It controls all critical functionality for the application.

If deployed using the Dynatrace Solution Server, this file can be updated via the configuration UI.

| Key | Type | Description |
|--|--|--|
| Tenants | tenants.json | The Dynatrace tenants/environments the DETL Service should connect to. By default it references the included [tenants.json](/esa-etl-service/Configuration/tenants.json) file but can be modified. The `tenants.json` object can be embedded here to support legacy configurations. |
| Templates | templates.json | Extraction templates for Dynatrace data. By default this references the included [templates.json](/esa-etl-service/Configuration/templates.json) file but can be modified. The `templates.json` object can be embedded here to support legacy configurations. |
| Destinations | destinations.json | The destinations to which the Dynatrace ETL may send metrics. By default this references the included [destinations.json](/esa-etl-service/Configuration/destinations.json) file but can be modified. The `destinations.json` object can be embedded here to support legacy configurations. |
| HTTPScripts | httpscripts.json | Any custom HTTP scripts available to the ConfigService. By default this references the included [httpscripts.json](/esa-etl-service/Configuration/httpscripts.json) file but can be modified. The `httpscripts.json` object can be embedded here to support legacy configurations. |
| library | Array[Object<library>] | This entry can consist of a single object, or an **array of objects**. Each object represents a **named library instance** that handles a subset of the configured tenants. |
| (library) name | String<LibraryName> | The name of the library instance. Default "LibraryService". |
| (library) host | String<hostname> | The host on which the instance runs. Default "localhost". |
| (library) port | Integer | The port at which this instance will expose its REST interface. Default 8081. |
| (library) launch | auto \| manual \| embedded" | Specifies whether the `StreamerService` should automatically start a `LibraryService` instance (as a separate process or embedded) for this entry. If not "auto" or "embedded", the `LibraryService` must be manually (eg. node LibraryService.js LibraryName). |
| (library) offline | Boolean | This setting is for debugging purposes. If ‘true’, the LibraryService will not synchronize with Dynatrace through the Dynatrace API. This may be useful for debugging. Default 'false'. |
| (library) debug | Boolean | This setting is for debugging purposes. If ‘true’, LibraryService instances launched by the StreamerService will attach to the Visual Studio Code debugger. Default 'false'.  |
| (library) tenants | Array[String<tenant>] | A list of tenants (name keys from "Tenants" from [tenants.json](/esa-etl-service/Configuration/tenants.json)) that should be handled by this library instance. eg. [ "qa", "prod_asda" ]|
| (library) refreshRate | String<timespan> | The preferred timespan the StreamingAPI is allowed to spend on refreshing all entities. Default "_hour". |
| (library) timeToPurge | String<timespan> | The timespan after "last seen by" entities are deleted from the DETL's admin. Default "_2hours". |
| (library) cachePeriod | String<timespan> | The timespan for keeping entities in memory since they have been last used in a query. Default "_15mins". |
| (library) savePeriod | String<timespan> | The timespan for persisting the entity and index administration. Upon exit, the library will persist all indexes before terminating. This setting however allows that to happen periodically as well, while the library runs. Mote that the only purpose of saving the indexes is to allow the library to start up faster (i.e. avoid initializing from the database). Default "_6hours". |
| (library) retryLimit | Integer | The number of times a Dynatrace API call should be retried before it fails. Serves as a default for the per-tenant setting with the same name. Also limits the number of retries to the database, per transaction. Default 1. |
| database | Object<database> | Configuration of the database in which the DETL stores the entities it indexes. |
| (database) class | String<ClassName> | Name of a Javascript class that implements the DatabaseAPI (i.e. a database adaptor). Currently supported are `MariaDBAdaptor`, `MySQLAdaptor` and `SQLiteAdaptor`. You can write our own classes for other databases. Default "SQLiteAdaptor" |
| (database) host | String<hostname> | Database hostname. If using SQLiteAdaptor this is not used. Default "localhost". |
| (database) port | Integer | Database port. If using SQLiteAdaptor this is not used. Default 3309. |
| (database) user | String<user> | Configuration options used by the database adaptor. Generally, this consists of the host, port and account at which the database can be accessed. There is no constraint on what configuration options are allowed here. |
| (database) password | String<password> | Database password. If using SQLiteAdaptor this is not used. |
| (database) database | String<databasename> | Name of the database in which the DETL will create its "Entities_*" tables |
| (database) connectionLimit | Integer | Maximum number of open connections to the database. Default 5. |
| (database) connectionTimeout | Integer | Timeout (in ms) before the attempt to connect to the database fails. Default 10000 |
| (database) connectionTTL | Integer | Connection Time to Live in ms. Default is 30. |
| (database) maxQueueTime | Integer | Maximum time in a query can stay in queue in seconds. Default 20. |
| (database) retryLimit | Integer | The number of times a query will be retried. Default 1. |
| failover | Object<failover> | (Optional) This section configures the contract between the primary StreamerService instance and the failover instance.  Failover instances can be started with a config.json file that contains only this entry (or use the command line argument config=myfile.config). It will receive its configuration from the primary instance. Note that the "logging" entry must be in the initial config file for the failover instance in order to get any logging. |
| (failover) primary| String | This is the address and port at which the primary instance listens. Default "localhost:8080". |
| (failover) standby| String | This is the IP address from where the failover instance will contact the primary instance to check for its health. Default "127.0.0.1". |
| (failover) secret | String | This is the key that the failover instance must provide in order to receive any updated configuration information from the primary instance. Only if both the standby IP address and the secret match will this information be included in the ‘health’ response. Default "12345". |
| diagnostics| Integer | Activate internal resource diagnostics gathering by setting value to 1. Default is 0. |
| reqRateLimit | Integer | Maximum number of requests per minute the DETL is allowed to make at each host at which the DynatraceAPI can be accessed. Default 600. |
| requestLimit | Integer | Maximum number of concurrently running requests per host. Default 20. |
| maxQueueTime | Integer | Maximum number of seconds a request may be queued when the request rate limit is reached. Default 10. |
| retryLimit | Integer | The number of times a Dynatrace API call should be retried before it fails. Serves as a default for the per-tenant setting with the same name. Also limits the number of retries to the database, per transaction. Default 1.  |
| maxWorkers | Integer | Maximum number of worker processes before the DETL Service starts to reduce the granularity of jobs. Default 8. |
| maxThreads | Integer | Maximum number of parallel jobs being processed in a Worker. Default 8. |
| maxJobSize | Integer | Maximum number of entities in a job before the DETL Service starts to split a job into smaller jobs. Default 10000. |
| metricTimeout| Integer | The timeout in seconds when attempting to retrieve data from a `tenant`. Default 20. |
| streamerPort | Integer | The port at which the DETL Service exposes its UI. Default 8080. |
| configPort| Integer | The port at which the DETL Config Service exposes its UI. Default 8085. |
| debugWorker | Boolean | Specifies whether Workers should connect to the Visual Studio Code debugger. Default false. |
| logging | Array[String<type>] | Specifies where the logging output should go. Running under PM2, the console output is captured for viewing automatically and no file logging is required. When "file" is included, log files are stored in the "../log" directory. To run in silent mode, omit this entry or specify an empty array. Default [ "file", "console" ]. |
| jobLatency | Integer | The number of minutes the DETL Service waits before it retrieves the metrics for a particular interval. Default 2. |
| libTimeout | Integer | The timeout in ms after which an attempt to connect to a `LibraryService` instance fails. Default 60000. |
| allowedUsers | Array[String<user>] | If specified, the list of user IDs that are allowed to use the DETL editor. |
| adminAccount | String<user\|password> | Define an admin account for accessing the DETL editor. |

