destinations.json
=====

The `destinations.json` file defines all the possible destinations for DETL streaming jobs. The file is referenced in the Destinations key of the [config.json](/esa-etl-service/Configuration/config.json.md) file. Destinations refer to the Javascript class and configuration parameters that describe where Workers need to send the output of a job to.

Out of the box you can push data to Apache Kafka, CSV to file, JSON to file, JSON to REST Endpoint, JSON to Azure Blob Storage. You can also create your own custom connector to push data to a different destination (See [Destination Extensibility](/esa-etl-service/Extensibility/DestinationAPI.md)).

See [Included Destinations](/esa-etl-service/Configuration/destinations.json/Included-Destinations.md) for more details on how to configure the out of the box destination classes.

If deployed using the Dynatrace Solution Server, this file can be updated via the configuration UI.

| Key  | Type | Description |
|--|--|--|
| name | String<destinationclass> | The name of the destination class to be used to connect to the destination’s endpoint. Currently supported values are "KafkaBrokerAPI", "CSVRendererAPI", "JSONRendererAPI", "JSONSenderAPI", "JSONSendToGCPAPI", "AzureBlobUploader", and "HTTPSenderAPI"  |
| host | Array[String<hostname>]| One or more hosts at which the destination can be reached, plus the port to be used to connect to any of them. Example ```["kafka-213355567.mycompany.com", "kafka-213355561.mycompany.com", "kafka-213355552.mycompany.com"]``` |
| port | Integer | Destination port, if applicable. |
| address | String<destinationaddress> | For the KafkaBrokerAPI this represents the topic at which the data should be published. For other adapters this might represent something else. Example `"tf.medusa.metric.1"`. |
| defaults | Object<defaults> | (Optional) In addition to the built-in template macros documented below, you may create your own macros and the values that they should return when used in templates. The values can be complex objects, in which case you can use the [default.subprop.subsubprop] notation to select the value. This also implies that you can add your own default tag and prop properties. Note: this can only be configured by directly modifying the `destinations.json` file. <br />Example: <br /> ```"default" : { "qa": "value", "prod": "value" }``` |
| template | String<template> | The name of the `template` (from the [templates.json](/esa-etl-service/Configuration/templates.json.md) section) that specifies the format of the message before it is sent to the destination. |
| delim | String<args> | For the CSVRendererAPI class this specifies the cell delimiter to be used in the export. If none specified, it will default to ","  |
| produce | String<type> | For supported classes this specifies to type of object graph to produce. Options could be "graphs" or "linkages", depending on the destination class used. |
| storagefolder | String<folder> | For the AzureBlobUploader class this specifies the blob storage folder on Azure. |