Included Destinations
=====
Out of the box you can use DETL push data to multiple destinations using the following destination classes.


| Destination Class | Description |
|--|--|
| AzureBlobUploader | JSON to Azure Blob Storage |
| CSVRendererAPI | Save as a local CSV file |
| DBRendererAPI | Data to a database table |
| HTTPSenderAPI | Data to an HTTP Endpoint |
| JSONRendererAPI | Save as a local JSON file |
| JSONSendToGCP | JSON to Google Cloud Storage |
| KafkaBrokerAPI | JSON to Kafka Broker |


You can also create custom destination classes to push data to a places that aren't supported out of the box (See [Destination Extensibility](/esa-etl-service/Extensibility/DestinationAPI.md)).

Reference these classes when configuring output destinations in [destinations.json](/esa-etl-service/Configuration/destinations.json.md). See below for more details on each destination class.

[[_TOC_]]

# Destination Classes
Descriptions of the destination classes and all appropriate json keys when configuring in `destinations.json`.

## AzureBlobUploader
This class is defined by `AzureBlobUploader.js` and can be used to save data to an Azure Blob Storage location. It is utilized by the ACE Services Portal to feed data from tenants.
| Key | Type | Description |
|--|--|--|
| address | String | Filename of the destination. Ex. "\$jobName-$timeStamp.json". |
| template  | String | For Portal use "DynatracePortal" |
| produce | String | "graphs" or "linkages" |
| storagefolder | String | Azure Blob storage folder path |

## CSVRendererAPI
This class is defined by `CSVRendererAPI.js` and can be used to save data to a local CSV file.
| Key | Type | Description |
|--|--|--|
| address | String | Filename of the destination. Ex. "\$jobName-$timeStamp.json". |
| template  | String | Default "TableOutput" |
| produce | String | Default "entities" |

## DBRendererAPI
This class is defined by `DBRendererAPI.js` and can be used to send data to a database.
The class utilizes the same [DatabaseAdaptor](/esa-etl-service/Extensibility/DatabaseAPI) classes that are used to interact with DETL's internal database.

| Key | Type | Description |
|--|--|--|
| class | String | The DatabaseAdaptor class to be used. |
| connectionTTL | Integer | Connection Time to Live in ms. Default is 30. |
| produce | String | "graphs" or "linkages" |

## HTTPSenderAPI
This class is defined by `HTTPSenderAPI.js` and can be used to send data in json format to a HTTP REST endpoint.
| Key | Type | Description |
|--|--|--|
| address | String | Filename of the destination. Ex. "\$jobName-$timeStamp.json". |
| template  | String | Default "TableOutput" |
| produce | String | Default "entities" |

## JSONRendererAPI
This class is defined by `JSONRendererAPI.js` and can be used to save data in json format to a local .json file.
| Key | Type | Description |
|--|--|--|
| address | String | Filename of the destination. Ex. "\$jobName-$timeStamp.json". |
| template  | String | Default "TableOutput" |
| produce | String | Default "entities" |

## JSONSendToGCP
This class is defined by `JSONSendToGCP.js` and can be used to send data in json format to Google Cloud Storage.

## KafkaBrokerAPI
This class is defined by `KafkaBrokerAPI.js` and can be used to save data to a local CSV file.
| Key | Type | Description |
|--|--|--|
| hosts | Array[String<host>] | An array of destination Kafka Broker hostnames. |
| port | Integer | Kafka broker port. Defaults to 9092. |
| connect | Object | Optional connection parameters as a json object. |
| address | String | Kafka topic name. |
| template  | String |  |
| produce | String | Always "entities" |

