httpscripts.json
=====

The ‘httpscripts.json’ file is an alternative to defining your HTTP scripts in ‘config.json’. HTTP scripts are series of parameterized REST calls to be made by the ConfigService to the Dynatrace UI.


| Structure of httpscripts.json  | Description |
|--|--|
| { | The scripts (or simple requests) that the ConfigService exposes. |
| "<RequestName>": { | Symbolic name of the request. The value is a "request" object with the properties specified below. |
| "description": "(DynatraceToken)", | (Optional) A description that will be shown in the API Explorer (in which this request may be tested). |
| "method": "POST|PUT|GET|DELETE", | (Optional) The HTTP method to be used. The default is "GET" unless the ‘data’ property is defined – then it is "POST". |
| "params": { <br>"age": "[request.years]", <br> "greeting": "[response.message]", <br> "fullName": "[request&#46;name]" <br> } | An arbitrary object that specifies the query parameters for the request. Each value may contain one or more macros and/or literals. Macros may refer to ‘request’ (taken from any query parameter provided by the client/user or created in a previous request) or to ‘response’ (taken from any previous response – ‘.’-separated paths into a response object are allowed). Not that the use of ‘\|’ characters are allowed to provide for alternative sources for the information to be collected.<br>For more information on macros, see the section "templates.json" (next). |
| "data": { … },|  (Optional) An arbitrary object of data to be posted. The data will be posted as JSON. Alternative encodings can be supported if necessary. |
| "next": { … } | (Optional) The next request to be invoked. The value is again a "request" object with the properties specified above. |
| }, |  |
| "<RequestName>": "/a/url/into/dynatrace", | The alternative to the above is to specify a simple URL into Dynatrace. It may contain query parameters. Whatever query parameters are provided by a client/user when this request is invoked will be added to it. Parameters with the name replace any defaults specified here. |
| ... | Define any number of named request scripts. |
| } |  |

## Example
```json
{ 
  "scripttest": {
    "url": "/test/first",
    "description": "Provide your age ('a') and 
                    a password ('pass').",
      "params": {
        "age":  "[request.a]",
        "name": "Bart"
      },
      "next": {
        "url": "/test/second",
        "params": {
          "years": "[request.age]",
          "name":  "[response.userName]",
          "pwd":   "[request.pass]"
        }, 
        "next": {
          "url": "/test/third",
          "params": {
            "age":      "[request.years]",
            "greeting": "[response.message]",
            "fullName": "[response.name.full]"
          }
        }
      }
    }
  }
} 
```
