templates.json
=====

The ‘templates.json’ file is an alternative to defining all templates in ‘config.json’. Templates describe how to create messages from a job’s entities and their respective metrics. 


| Structure of templates.json  | Description |
|--|--|
| { |  |
| "MyFirstTemplate": { | Name of the template. |
| "_metadata_": [ "col1","col2", … ] | (Optional) Name of file containing metadata. This can be YAML or a JSON file. |
| "_columns_": "KafkaBrokerAPI", | (Optional) Array of column names. |
| "default": { … }, | Default template for entities of a type not explicitly defined. |
| "HOST": { … }, | Template for entities of type ‘HOST’. Note that it is not mandatory to have a template for HOSTS, specifically. You can define templates for any type(s) (e.g. APPLICATION, SERVICE, PROCESS_GROUP, PROCESS_GROUP_INSTANCE, HOST) for which you need a specifically structured output. |
| ... |  |
| }, |  |
| "MySecondTemplate": { … }, | You can define as many templates as you need. |
| ... |  |
| } |  |

A template for a specific type of entity (as well as the "default" template) describes a simple or complex object with properties and fields. This is an example of a simple object template:

```json
    "default": {
        "format": "[job.type] message",
        "id": "[prop.entityId]",
        "name": "[prop.displayName]",
        "*1": "[tag.ENVIRONMENT.*]",
        "*2": "[tag.CONTEXTLESS*]",
        "*3": "[metric.*]"
    }

```

Values can be strings (e.g. "[job.type] message"), or objects with properties, or arrays. String values can (and typically do) contain macros, encapsulated in square brackets (e.g. ‘[a.macro]’). In the output, macros are replaced with the value(s) found at the path they reference. Macros follow the following syntax: [scope.path.to.value]. 

- A value can contain multiple macros (e.g. "This is a [job.type] named [prop.displayName]"). When that is the case, the output is always a string. If an individual macro resolves to an object or array, it will be stringified before replacing the macro.
- A value can contain macro alternatives, separated by a ‘|’ character (e.g. "This [job.type] is a [tag.CONTEXTLESS.TagThatNormallyExists]|[tag.CONTEXTLESS.OtherwiseThisTag]|(N/A)").
- A value can be surrounded by ‘[‘ and ‘]’ characters. This affects into what data type the value will be resolved. More on that later. A value can also be an object itself, with properties and macro-ed values.
