Column Order
=====

Finally, a template can also contain a "_columns"_ property. This property is useful for destination adaptors that create tabular outputs. The ‘_columns_’ property provides a means to control the order in which the fields are to be written to the output. Consider this example:
```
"MyTableFormat": {
    "_columns_": ["id", "name", "format", "CONTEXTLESS.*", "ENVIRONMENT.*", "*" ],
    "default": {
        "format": "[job.type] message",
        "id": "[prop.entityId]",
        "name": "[prop.displayName]",
        "*1": "[tag.ENVIRONMENT.*]",
        "*2": "[tag.CONTEXTLESS*]",
        "*3": "[metric.*]"
    }
}
```
Whenever a message is produced, the Destination adaptor receives a list of columns names that reflects the order specified in the "_columns" property. The Worker (specifically, the FormatterFactory) will track all fields added to the output across all messages that are produced as part of the job. This means that the destination adaptor can use a single, consistent set of columns, even if some messages did not receive (values for) some of the referenced fields. 

The column names may contain ‘*’ characters. Fields with matching prefixes are matched against the best matching column names and inserted there in a consistent order. It is recommended to always include a ‘*’ name at the end to catch any fields that did not match any of the other wildcarded column names.

Note that the ‘ledger’ that is used internally to track all fields is a complex (i.e. hierarchical) object. Thus, non-tabular destinations may also leverage this ledger. For column-based outputs the template should ideally only contain one level –the ledger will then contain a simple list of column names. 
