Context Switches
=====

In addition to the use of ‘ENTITY_TYPE’ in macros (e.g. "[scope.ENTITY_TYPE.path.to.value]"), it is also supported in ‘context switches’, like this: "<ENTITY_TYPE>…[macro]…". 
- Context switches are surrounded by ‘<’ and ‘>’ characters. They specify that any macros following the ‘switch’ should be resolved against each individual related entity of the specified type. Thus, using a ‘switch’ immediately duplicates the value into as many values as there are matching related entities. 
- You can have multiple context switches in a value. Each starts at the current context and multiplies from there. Thus, for a host with two process groups and two related services in each, you can do this:
  - "myprop": "[prop.displayName] running <PROCESS_GROUP>[prop.displayName] doing <SERVICE>[prop.displayName]"
```
"myprop": "this.host.name running mysite.company.com doing orderService,this.host.name running mysite.company.com doing confirmService,this.host.name running app-backend doing checkCredit,this.host.name running app-backend doing storeOrder" 
```
  - "myprop": [ "[prop.displayName] running <PROCESS_GROUP>[prop.displayName] doing <SERVICE>[prop.displayName]" ]
```
"myprop": [ 
        "this.host.name running mysite.company.com doing orderService", 
        "this.host.name running mysite.company.com doing confirmService", 
        "this.host.name running app-backend doing checkCredit", 
        "this.host.name running app-backend doing storeOrder" 
    ]
```
- When a value that uses context switches is assigned to a wildcarded property name, the result adds properties to the containing object. The property names are constructed by concatenating the IDs of each context branch (using a ‘:’ character). The IDs can be mapped if so desired.
  - "*": "… running <PROCESS_GROUP>[prop.displayName] doing <SERVICE>[prop.displayName]" 
"PROCESS_GROUP-1234:SERVICE-7890": "… running mysite.company.com doing orderService",
```
               "PROCESS_GROUP-1234:SERVICE-7891": "… running mysite.company.com doing confirmService",
               "PROCESS_GROUP-5678:SERVICE-1230": "… running app-backend doing checkCredit",
               "PROCESS_GROUP-5678:SERVICE-1231": "… running app-backend doing storeOrder"
```
  - "*=>displayName": "… <PROCESS_GROUP>[prop.displayName] doing <SERVICE>[prop.displayName]" 
```
"mysite.company.com:orderService": "… mysite.company.com doing orderService",
               "mysite.company.com:confirmService": "… mysite.company.com doing confirmService",
               "app-backend:checkCredit": "… app-backend doing checkCredit",
               "app-backend:storeOrder": "… app-backend doing storeOrder"
```
- For flow-based jobs, where each entity is related to one or more entities of the same type, in a potentially long chain, each context switch goes one level deeper than the previous one.
- Note that macros that refer to metrics are always resolved against the ‘original’ entity. Tags and properties however will honor the context switches. 
