Formal Template Specifications
=====
[[_TOC_]]

## Grammar

The specifications below describe the grammar of the template language. Note that templates must be valid JSON objects.

| Color Code Legend |  | Syntax Character Legend |  |
|--|--|--|--|
| Black | Syntax character	→ | := |  Is defined as |
| <span style="color:blue">Blue</span>| Literal character | \| | Or |
| <span style="color:orange">Orange</span> | Symbol, syntax of which is defined elsewhere | ... | Any number of the preceding |
| <span style="color:green">Green</span> | Comment | () | Group |
|  |  | ? | Zero or one |
|  |  | * | Zero or more |

![image.png](/.attachments/image-dffa81f9-f723-425b-987c-8eb6fa7b14c8.png)
![image.png](/.attachments/image-04e5d02c-bc23-4025-98bd-4e6862e0dcbb.png)
![image.png](/.attachments/image-c189e0ff-158b-4b56-940a-f2e8ad1fcd45.png)
![image.png](/.attachments/image-1a49afa6-eb08-4e74-ba7a-1cddf1e4e19a.png)
![image.png](/.attachments/image-dba54835-6397-41ec-ba6c-b103d4b3701f.png)
![image.png](/.attachments/image-7a30f61b-2a19-4b38-b014-984ebe8922e5.png)


## Resolving values
For each entity, an output object is produced from an applicable template. The structure of a template has been described above. At the heart of the template engine lies the concept of resolvableValue. It is defined as follows (in simplified notation, with more advanced constructs removed for clarity – more on those later):

```resolvableValue:= <literal><macro><resolvableValue>```

This means that a resolvableValue is a string that contains any mix of static text and macros. A macro refers to a property, tag, metric or other piece of information that should be retrieved from the entity at hand or from somewhere else. It is possible to specify alternative sources for the desired information by listing several macros, separated by ‘|’ characters. After a value is found, the other alternatives are discarded.

```macro:= 	<contextSwitch>?[<macroString>](|[<macroString>])*```

The context switch is useful for changing the current context (i.e. entity) to another, related entity. This affects from where ‘prop.’ and ‘tag.’ macros are resolved (but not ‘metric.’ macros!).

The resulting information can be a primitive value like a string, number or boolean, a compound value like an object or an array. The result of resolving a resolvableValue depends on the following:

1. If a macroString has no wildcard (i.e. no ‘*’) and thus refers to a specific value, the result is that specific value (e.g. a string, number, boolean, object or array). 
- Metrics only: If a macroString also has a mapping, the metric’s ID is used to map to a specific field in the metadata (or to ‘id’, ‘name’ or ‘value’). The result is whatever the metadata contains for that field. 
2. If a macroString has a wildcard, the result is a collection of values that may end up being rendered as a comma-separated string, an object or an array. 
- Metrics only: If a macroString also has a mapping, the metrics’ IDs are used to map to a specific field in the metadata (or to ‘id’, ‘name’ or ‘value’). The result is a collection of whatever the metadata contains for that field. 
3. If a macroString contains one or more context switches (e.g. <PROCESS_GROUP>[prop.displayName]), the result is a collection of values that may end up being rendered as a comma-separated string, an object or an array.
- Each time a context switch is encountered in a ‘still-resolving’ resolvableValue, the resolvableValue, as resolved up till that point, is immediately duplicated into as many copies as related entities of the specified type are found, starting from the current context (which is initially the entity at hand – the one that contains the metrics). 
- This means that a resolvableValue that has one or more context switches resolves to a collection of resolved values, one for each distinct branch of context switches.
4. If a macroString refers to a related entity type (e.g. [prop.PROCESS_GROUP.displayName]), the result is a collection of values (one for each related entity of the specified type) that may end up being rendered as a comma-separated string, an object or an array. 
- This notation does not change the context away from the current entity and has no multiplying effect beyond the macroString itself.
5. If a resolvableValue contains more than one macroString or has static text in it as well (e.g. [tag.CONTEXTLESS.*]_tags), the result is always converted into a string and replaced as such into the ‘still-resolving’ value.

## Resolving objects

For each entity, an output object is produced from an applicable template. The structure and syntax of a template has been described above. The relevant part for this section is this:
```
objectValue:= 	{
	"<propertyName>(=><mapping>)?": <propertyValue>, 
	…
}
propertyName:= 	<literal>?*<literal>?	
propertyName:= 	<literal>

propertyValue:= 	<objectValue>
propertyValue:= 	"<resolvableValue>"
propertyValue:= 	{ "<resolvableValue>" }
propertyValue:= 	[ <propertyValue>, … ]
```
The top-level element of a ‘default’ or type-specific template is an objectValue. The formatter generates a message for a given entity (and its metrics) by starting with the creation of an empty output object that corresponds to that top-level objectValue.

For each property listed an objectValue it creates one or more corresponding properties in the output object.

1. If the propertyName is a literal value (e.g. "myprop"), a single property with that name is created in the output object and its value is whatever propertyValue resolved to.
2. If the propertyName contains a wildcard (e.g. "*" or "k8s_*"), the properties will be obtained from whatever the propertyValue resolves to. 
- If the propertyValue resolves to an object, the properties  in that object (and their values) are copied to the current output object.
- If the propertyValue resolves to a collection, the indexes are used as property names and copied into the current output object (with their respective values).
- If the propertyValue resolves to a collection because of references to other entities, the entity IDs that identify each branch or match are used as property names (e.g. PROCESS_GROUP-1234:SERVICE-4567) and copied into the current output object (with their respective values).
- If the propertyValue resolves to a primitive value (i.e. a string, number, boolean), the propertyName is kept as-is.
- If a prefix is specified, each copied property is prefixed with it. Postfixes are ignored – they are merely supported in order to allow for multiple wildcarded properties (multiple "*" properties are invalid in JSON).
3. If the propertyName contains a wildcard and a mapping, the property names will be obtained by mapping their values, using the field specified in the mapping, to the corresponding metadata.
- For entityIds, available mapping fields are any string-property available for one or all of the corresponding entity types (typically displayName).
 

The resolved propertyValue is assigned to the propertyName as follows:

4. If the propertyValue is an objectValue itself (e.g. "{ … }"), that embedded object is resolved recursively according to the usual rules. This of course creates a nested output object.
5. If the propertyValue resolves to a string, that string becomes the value of the specified propertyName. Note that a wildcard in the propertyName would not make sense because there is nothing to derive a name from. 
6. If the propertyValue resolves to an object, either (if no ‘\*’ in play) that object is assigned to the propertyName, 
OR (if it contains a ‘*’) its properties are copied into the object that contains the propertyName. 
7. If the propertyValue resolves to an array, either (if no ‘\*’ is in play) that array is assigned to the propertyName, 
OR (if it contains a ‘\*’) its indexes are copied into the object that contains the propertyName.
8. If the propertyValue is embedded in an array (e.g. [ "prop.technologies.*", "anotherElement ]), its resolution will be inserted in that array at the position of that propertyValue. An object will be inserted as an object. An array will be ‘spliced’ into the array.
9. If the propertyValue contains a macro that references related entities of a specific type (e.g. prop.PROCESS_GROUP.displayName or "<PROCESS_GROUP>[prop.displayName]"), then the result is a collection of resolutions, one for each matching entity (or distinct chain of entities if multiple context switchers are used). That collection can be assigned to the propertyName as follows:
- If the propertyValue is expressed as a string (i.e. "prop.PROCESS_GROUP.displayName"), 
the collection resolves into a single, comma-separated string of stringified values.
- If the propertyValue is embedded in an array (e.g. [ "prop.PROCESS_GROUP.displayName" ]), 
the collection resolves as individual elements in that array.
- If the propertyValue is embedded in an object (e.g. { "*": "prop.PROCESS_GROUP.displayName"} ),
 the collection resolves as individual properties in that object.
- The usual rules for translating wildcards into the property names apply (see above).

Note that rule (6) works recursively upwards. That has the following implication:

This objectValue in a template:		
```
"myprop": {
    "name": "[prop.displayName]",
    "*": { 
       "app": "Orders",
       "*": "[tag.CONTEXTLESS*]"
    }
}
```
does not resolve into:
```
"myprop": {
    "name": "my.host.name",
     "*": { // Wildcard here!
        "app": "Orders",
        "tag1": "value1",
        "tag2": "value2",
        "tag3": "value3"
    }
}
```
but rather, into:
```
"myprop": {
    "name": "my.host.name",
    "app": "Orders",
    "tag1": "value1",
    "tag2": "value2",
    "tag3": "value3"
}
```
TODO: diagrams for value resolution. Separately, for object construction.
