Mappings
=====

We have seen that macros follow the following syntax: [scope.path.to.value]. However, a variant on this syntax is supported as well: [scope.ENTITY_TYPE.path.to.value]. This is only available for jobs that contain flow or stack relationships, where each entity to be output is (typically) related to other entities. Macros that contain an ‘ENTITY_TYPE’ work as follows:
- For stack-based jobs where for each entity we have its stack dependencies, the macro will find all related entities of the given type and resolve the macro for each. The result will therefore be an array, and the usual rules apply for resolving an array into an output:
  - "myprop": "[prop.PROCESS_GROUP.softwareTechnologies.0.type]"	(assume the entity is a HOST)
```"myprop": "ASP_NET, WCF"```
  - "myprop": [ "[prop.PROCESS_GROUP.softwareTechnologies.*]" ]	(assume the entity is a HOST)
```
"myprop": [
                    [ { "type": "ASP_NET", "edition": ".NET", "version": "4.7.2.0" },
                      { "type": "Java",    "edition": "8",    "version": "1.8"     }
                    ],
                    [ { "type": "WCF",     "edition": null,   "version": "4.7.2.0"    },
                      { "type": "MSSQL",   "edition": null,   "version": "4.7.3260.0" }
                    ]
                ]
```

- It is possible to force the macro to resolve to an object instead of an array, and use the usual rules:
  - "myprop": { "*": "[prop.PROCESS_GROUP.softwareTechnologies.0.type]" }	(value in ‘{…}’)
```"myprop": { "PROCESS_GROUP-1234": "ASP_NET", "PROCESS_GROUP-5678": "WCF" }```
  - "*": { "*": "[prop.PROCESS_GROUP.softwareTechnologies.0.type]" } (property name is ‘*’, value in ‘{…}’)
```"PROCESS_GROUP-1234": "ASP_NET",```
```"PROCESS_GROUP-5678": " WCF"```
  - "*": { "*": "[prop.PROCESS_GROUP.softwareTechnologies.*]" }   (property name is ‘*’, value in ‘{…}’)
  ```
               "PROCESS_GROUP-1234": [
                    { "type": "ASP_NET", "edition": ".NET", "version": "4.7.2.0" },
                    { "type": "Java",    "edition": "8",    "version": "1.8"     }
                ],
               "PROCESS_GROUP-5678": [ 
                    { "type": "WCF",     "edition": null,   "version": "4.7.2.0"    },
                    { "type": "MSSQL",   "edition": null,   "version": "4.7.3260.0" }
                ]
  ```
  - "*=>displayName": "[prop.PROCESS_GROUP.softwareTechnologies.*]"	(name is mapped!)
```
"mysite.company.com": [    (Note: result is similar to the previous due to the rules re. wildcarded properties!)
                    { "type": "ASP_NET", "edition": ".NET", "version": "4.7.2.0" },
                    { "type": "Java",    "edition": "8",    "version": "1.8"     }
                ],
               "app-backend": [ 
                    { "type": "WCF",     "edition": null,   "version": "4.7.2.0"    },
                    { "type": "MSSQL",   "edition": null,   "version": "4.7.3260.0" }
                ]
```
- For flow-based jobs, where each entity is related to one or more entities of the same type, you can only refer to that type. The macro will find all related entities and resolve the macro for each. There may not be a good use case for this, but it is supported.
- Note that using ‘ENTITY_TYPE’ is not supported for macros that reference metrics.
