Using Scopes
=====

The rules for how macros resolve depend on the "scope" (i.e. ‘prop’, ‘tag’, ‘metric’, ‘job’ or ‘default’).

- ‘prop’ signals that the ‘path.to.value’ part will be a ‘.’-separated path to an entity’s property. Examples are ‘prop.displayName’ and ‘prop.softwareTechnologies.0.type’. The Dynatrace API Explorer describes all properties that entities of various types can have. Some entities have properties that are complex objects or arrays. Specific values can be ‘harvested’ from them by describing the path to the desired field. When arrays are in play, you can use numbers to refer to a value at a particular index.
- ‘tag’ signals that that the ‘path.to.value’ part is a ‘.’-separated path to a specific entity tag. Tags use the following naming convention: "CONTEXT.tagName". Commonly available contexts are "CONTEXTLESS", "ENVIRONMENTS", "K8S", etc.
- ‘metric’ signals that the ‘path.to.value’ part is a metric ID. Metric IDs look like ‘.’-separated paths but are in fact ‘just’ IDs. A metric ID is always followed by a ‘#’ character and an aggregation mnemonic (i.e. AVG. MAX, MIN, SUM). For metrics that have dimensions, an additional ‘#’ may follow, describing how to aggregate the values across all dimensions (i.e. AVG. MAX, MIN, SUM, NUM). The default value for such scenarios is "AVG".
- ‘job’ signals that the ‘path.to.value’ part is a job property. Available properties are: 
  - "name": Name of the job.
  - "source": Origin of the entities. This is the name of one of the tenants defined in config.json.
  - "destination": The destination of the entities (as defined in config.json or destinations.json).
  - "zone": The management zone used to constrain the set of entities selected in this job.
  - "TYPE": The original uppercase type of the entities selected in the job. 
  - "type": That same type in a lowercase form, with the underscores replaced by spaces.
  - "granularity": The granularity of the metrics in the output. Valid values are "_min", "_5mins", etc.
  - "timestamp": The epoch value (ms) of the end of the timeframe for which the metrics have been collected.
  - "timestring": The string representation of the timestamp in simplified extended ISO format (YYYY-MM-DDTHH:mm:ss.sssZ). The time zone is always zero UTC offset, as denoted by the suffix "Z".
- ‘default’ signals that the ‘path.to.value’ part refers to a value in the ‘defaults’ section in the destination (see config.json). The values can be complex objects because you can use the ‘[default.path.to.value]’ notation to select the desired property inside it. Note that this scope serves two use cases: 
  - To include data in the output that is not present in the job or entities themselves;
  - To provide defaults when properties or tags are missing in an entity 
(e.g. "[tag.CONTEXTLESS.mytag]|[default.mytag]").
- If the scope is missing, the macro is presumed to refer to one of the predefined ‘globals’:
  - "format": The name of the template used to produce the message.
  - "tenant": The name of the tenant as configured in the "Tenants" section. Is an alias for [job.source].
  - "timestamp": Is an alias for [job.timestamp].
  - "timestring": Is an alias for [job.timestring].
