Using Wildcards
=====

In all cases, the ‘path.to.value’ part can contain a ‘*’ somewhere in the path (only one is allowed). Examples are: ‘metrics.*’, ‘prop.softwareTechnologies.*.type’,  ‘tag.CONTEXTLESS.*’, etc. A ‘*’ matches any property, index, tag or metric at that point in the path. The macro gets replaced with all matching values. The resulting output depends on a number of factors. Consider the following template:
```
    "default": {
        "myprop": "[mymacro]"
    }
```

- If the property name is a ‘fixed’ name (i.e. "myprop") and the macro resolves to one or more strings, the output can be a comma-separated list of strings, or an array of strings, depending on whether the value is specified as an array or as a string: 
  - "myprop": "[prop.softwareTechnologies.*.type]" 	(value specified as string)
```"myprop": "DotNet,Java" ```
  - "myprop": [ "[prop.softwareTechnologies.*.type]" ] 	(value specified as array)
```"myprop": [ "DotNet", "Java" ]```
- If the property name is a ‘fixed’ name and the macro resolves to one or more objects, the output will be an object or an array of objects (never a string). Examples: 
  - "myprop": "[prop.softwareTechnologies.0]"	(value specified as string)
```"myprop": { "type": "DotNet" , "version": "5.1" }```
  - "myprop": [ "[prop.softwareTechnologies.0]" ]	(value specified as array)
```"myprop": [ { "type": "DotNet" , "version": "5.1" } ]```	
  - "myprop": "[prop.softwareTechnologies.*]" 	(value specified as string)
```"myprop": [ { "type": "DotNet" , "version": "5.1" }, { "type": "Java" , "version": "8" } ]```
  - "myprop": [ "[prop.softwareTechnologies.*]" ] 	(value specified as array)
```"myprop": [ { "type": "DotNet" , "version": "5.1" }, { "type": "Java" , "version": "8" } ]```
- When a macro will resolve to an object or an array, the property name can also contain a ‘*’. The ‘*’ property will be replaced with the actual property names found in the resulting object or array (note: therefore, using a ‘*’ for macros that resolve to a string would be meaningless). The replacement then goes as follows:
  - "*": "[prop.softwareTechnologies.0]"	(macro resolves to an object)
```"type": "DotNet", "version": "5.1"```
  - "*": "[prop.softwareTechnologies.0.*]" 	(macro resolves to an object’s properties)
```"type": "DotNet", "version": "5.1"```
  - "*": "[prop.softwareTechnologies.*]" 	(macro resolves to an array of objects)
```
"0": { "type": "DotNet", "version": "5.1" }, 
"1": { "type": "Java",   "version": "8"   }
```
  - "*": "[prop.softwareTechnologies.*.type]" 	(macro resolves to an array of strings)
```
"0": "DotNet", 
"1": "Java"
```
  - "*": "[tag.CONTEXTLESS.*]"	(resolves to a tag object’s properties)
```
"MyFirstTag": "SomeValue", 
"MySecondTag": "AnotherValue"
```
  - "*": [ "[tag.CONTEXTLESS.*]" ] 	(resolves to a tag object’s properties)
(Don’t know what this will or should do yet)
  - "*": "[metric.*]"	(resolves to a metric object’s properties)
```
"metric1": "1.234", 
"metric2": "2.345"
```
  - "*": [ "[metric.*]" ] 	(resolves to a metric object’s properties)
(Don’t know what this will or should do yet)
- Property names with a ‘*’ may have a prefix or a postfix, so that multiple ‘*’-properties can be used in the same containing object. Prefixes will be placed in front of each found property name (e.g. "ABC_*": "[tag.CONTEXTLESS.*]" creates properties "ABC_MyFirstTag" and "ABC_MySecondTag"). Postfixes will be omitted – they only exist to enable multiple ‘*’ properties that each will expand into sets of actual properties.
