tenants.json
=====

The `tenants.json` file is an alternative to defining your tenants in `config.json`. Starting with Dynatrace ETL Service version 6.0.0, this is the default method for defining tenants. Tenants refer to environments or tenants in your Dynatrace cluster(s). Any number of tenants can be defined. They are used by the LibraryService(s) to know from where to extract SmartScape data, and by the Workers to know from where to obtain metric values.

If deployed using the Dynatrace Solution Server, this file can be updated via the configuration UI.


| Key | Type | Description |
|--|--|--|
| name | String | Symbolic name of the tenants. Refer to this name when configuring tenants in the [Library Service](/esa-etl-service/Configuration/config.json.md) |
| host | String<Hostname> | A single host or an array of hosts at which the StreamingAPI can access the Dynatrace APIs such as "mycluster.dynatrace-managed.com" |
| url | String<URL> | The tenant/environment ID for Dynatrace Managed tenants, such as "/e/xxxx-xxx-xxx-xxx-xxxxxx" |
| token | String<DynatraceToken> | The [API token](https://www.dynatrace.com/support/help/dynatrace-api/basics/dynatrace-api-authentication/) that should be used when requesting data from the Dynatrace APIs. |
| account | String<user>\|<password> | The user ID and password of a service account that the configuration API should use. Note that the configuration API actually logs on to Dynatrace as an end-user to make configuration changes. This setting is also used when the tenant is a SaaS tenant. In that scenario, the DETL Service is not able to log in to Dynatrace as an actual user, so to validate user credentials when a user logs in to the Editor, these are the credentials that provide that user access. <br/> e.g. "serviceuser&#124;servicepwd" |
| retryLimit | Integer | The number of times a Dynatrace API call should be retried before it fails. Default 1. |
| pageSize | Integer | The maximum number of entities the library is allowed to retrieve per request against the Dynatrace API. If there is more data, the StreamingAPI will retrieve the data in groups of ‘pageSize’ entities. Default 1000. |
| chunking | Object<Chunking> | (Optional) Retrieval strategies for the various types of entities. `APPLICATIONS` and `SERVICES` are retrieved by default in single requests. All other entities are by default retrieved as they are discovered by following other entities’ relationships. This configuration allows users to override this per type of entity. |
| Object<Chunking> | <Type>: "<criterion>=a,b,c" |  See below for _available chunking strategies_ |
| storagefolder | String<folder> | (Optional) Blob storage folder used for destinations such as AzureBlobUploader. |
| environment | String<environment id> | (Optional) Explicit environment ID used for destinations such as AzureBlobUploader. |


**Available Chunking Strategies** 
`HOST` is an example here: 

- "HOST": "[ENVIRONMENT]Cloud=cloud1,cloud2,cloud3" - Use individual requests to retrieve hosts, one for each tag value in the list. 
- "HOST": "ZONE=mz1,mz2,mz3" - Use individual requests to retrieve hosts, one for each  management zone in the list. 
- "HOST": "HOSTGROUP=group1,group2,group3" - Use individual requests to retrieve hosts, one for each  host group in the list. 
- "HOST": "ALL" – Retrieve all hosts at once, including relationships. 
- "HOST": "LIST" – Retrieve all hosts at once, excluding relationships.
- "HOST": "NONE" – Don’t load hosts at all.