DatabaseAPI
=====

The DatabaseAPI is an interface that should be implemented by a NodeJS module when there is no available implementation for the selected database (MariaDB and MySQL are supported out of the box). It is responsible for handling the interactions with a database. For more details see the MariaDB and MySQL adaptors. Also refer to the JSDoc-generated documentation at /doc/www/index.html.

Below is a boilerplate implementation.

```javascript
class MyDatabaseAdaptor {
    constructor(config, cleaner, logger) {
        // Connect to the database using the options specified in the 'config' object. Initialize a connection pool.
    }
    createConnection(taskName, resolve, reject) {
	// Return an object that implements the DatabaseConnection interface:
        return {
            name:    taskName,
            resolve: resolve,
            reject:  reject,
            state:   "Queued",
            conn:    null,      // Recommended. All methods after ‘initialize()’ must work on this actual connection.
            initialize:   function (conn) {
                // This is the ‘connection.initialize()’ method call by ‘acquire()’. Set ‘this.state’ to "Connected".
            },
            // The methods below only need to exist after 'initialize()' is called, must an act on the ‘conn’ object.
            begin: function () {
                // Start a new transaction. The 'commit' or 'rollback' method will be called as appropriate.
            },
            query: function (statement, values) {   
                // Return a promise to execute the statement, using the values if specified.
            },
            stream: function (statement, values) {
                // Execute the statement, using the values if specified, and stream the result back.
            },
            commit: function (onDone, onError) { 
                // Commit transaction and release the connection. Call onDone() or onError(err) when done.
            },
            rollback: function (onDone, onError) { 
                // Rollback transaction and release the connection. Call onDone() or onError(err) when done.
            },
            release: function (onReleased, noCleanup) {
                // Release connection. Call 'cleanup()' if noCleanup not 'true'. Call onReleased() when done.
            }
        };
    }
    acquire(connection, onAcquired) {
        /*  Set ‘this.state’ to "Connecting". Create a connection ('conn') from the pool. Once available, call
            connection.initialize(conn). After that, call connection.resolve(connection) and, if specified, 
            call onAcquired(). If a connection cannot be created, call connection.cleanup(c) and connection.reject(err).
         */
    }
    get activeConnections() {
        // Return the number of currently active connections.
    }
    get totalConnections() {
        // Return the maximum number of available connections.
    }
    close(resolve) {
        // Close the connection pool. Call resolve() afterwards.
    }
}
module.exports = MyDatabaseAdaptor;
```