DestinationAPI
=====

The DestinationAPI is an interface that should be implemented by a NodeJS module when there is no implementation available for the selected output (CSV-file, JSON-file, HTTP and Kafka are supported out of the box). It is responsible for handling the communication with the targeted destination technology. Each implementation should return a singleton instance from its constructor that implements the DestinationAPI. Also refer to the JSDoc-generated documentation at /doc/www/index.html.

Below is a boilerplate implementation.

```javascript
const MyDestinationAPI = function (config) {
    this.config = {
        produce:   config.produce, // The provided value is the ‘ask’. Translate it into what this adaptor can support.
        // Other, implementation-specific settings. The config comes straight from the ‘destination’ in config.json.
    };
    const subscribe = function (eventName, fn) {  
        // Add the subscription for this event.
    };
    const emit = function (eventName, data) {     
        // Call the subscribed event handlers.    
    };
    // Initialize. When ready emit a "ready" (or "error") event (asynchronously!).
    setImmediate(emit, "ready");
    // Return an object that implements the DestinationConnection interface.   
    return {
        on:        subscribe,               // Accept subscriptions to "ready" and "error" events.
        produce:   self.config.produce,     // This is the output this adaptor can produce for graphs.
        formatter: config.formatter,        // The formatter as provided in the 'config' object.
        open:      function (name, cycle, fieldGetter, onConnectionOpen) { 
            // Open a connection to the destination and call onComplete() when done (asynchronously!).
            setImmediate(() => onConnectionOpen(null, {
                // Send/write messages and call onComplete() when done (asynchronously!).
                send:  function (messages, onComplete) { … },
                // Close whatever connections were opened.
                close: function () { … }
            }));
        }
    };
};
module.exports = MyDestinationAPI;

```