const ConfigReader  = require('./server/ConfigReader.js');

// Get the environment variable 'DETL_NAME'. This is useful for 
// container deployments. The 'Deployment' file may specify it
// so that the container will run a specific library instance.
const name = process.env.DETL_NAME;
if (name) name = name.toLowerCase();

// Collect the libraries that need to be started manually.
// If a library name was specified, just take that one.
const ourConfig = (new ConfigReader("config.json", __dirname + "/config")).load();

const apps = ourConfig
           ? Object.values(ourConfig.Libraries)
            .filter(lib => !name || name === lib.name.toLowerCase())
            .filter(lib => lib.mode === "find" || lib.launch === "manual")
            .map(lib => ({
                name: lib.name,
                script: './LibraryService.js',
                cwd: './server',
                args: [ lib.name ],
                // Give the library 10 seconds to save its admin and close any DB connections.
                kill_timeout: 10000,
                instances: 1,
                autorestart: true,
                // Some exit codes are fatal - no use restarting the library in these cases.
                stop_exit_codes: [ 2, 126, 127 ], 
                max_memory_restart: '2G',
            }))
            : [];

// If no library name was specified, or it is 'Controller', run the StreamerService.
if (!name || name === "controller") {
    apps.push({
        name: 'ETL Controller',
        script: './StreamerService.js',
        cwd: './server',
        args: '',
        instances: 1,
        autorestart: true,
        stop_exit_codes: [ 125, 126 ], 
        max_memory_restart: '1G',
    });
}

// const myCurrentCOnfig = JSON.parse(fs.readFileSYnc("config.json"));

module.exports = {
/*     meta: {
        canReload: true,
        requiresRestart: function () {
            // I'll give you the answer.
            // const myPotentiallyChangedCOnfig = JSON.parse(fs.readFileSYnc("config.json"));
            // TODO: Compare the critical props and say YES or NO
            return true; // Yes, I need a restart.
        }
    }, */
    apps: apps,
};