const BlackHoleAPI = function (config) {
    config = config || {};

    this.config = config;
    this.config = {
        produce:   config.produce,
        formatter: config.formatter
    };

    this.events = {};

    var emit = function (eventName, data) {
        // Call the subscribed event handlers.
        const event = self.events[eventName] || [];
        event.forEach(fn => { fn.call(null, data); });
    };

    var subscribe = function (eventName, fn) {
        // Add the subscription (event handler) for this event.
        self.events[eventName] = self.events[eventName] || [];
        self.events[eventName].push(fn);
    };

    const self = this;

    console.log("Creating black hole for unloadable class '" + config.class + "'");

    setTimeout(emit, 1, "ready");

    return {
        on: subscribe,
        produce:   self.config.produce,
        formatter: self.config.formatter,
        open: function (job, fieldGetter, onConnectionOpen) {
            console.log("Opening black hole for job " + job.name);

            setTimeout(onConnectionOpen, 1, null, {
                send:    function (messages, onComplete) {
                    setTimeout(onComplete, 1);
                },
                close:   function () {
                    return true;
                }
            });
        }
    };
};

module.exports = BlackHoleAPI;
