const Diagnostics = function (config, library, logger) {
    let timer    =  null;
    let interval = config.diagnostics ? Number(config.diagnostics) * 1000 : null;

    const diagnose = function () {
        let memoryUsage   = process.memoryUsage();
        let memoryStats   = [
            "Resident set=" + (memoryUsage.rss / 1024 / 1024).toFixed(2) + "MB",
            "heap total=" + (memoryUsage.heapTotal / 1024 / 1024).toFixed(2) + "MB",
            "heap used=" + (memoryUsage.heapUsed  / 1024 / 1024).toFixed(2) + "MB",
            "external=" + (memoryUsage.external / 1024 / 1024).toFixed(2) + "MB",
            "array buffers=" + (memoryUsage.arrayBuffers / 1024 / 1024).toFixed(2) + "MB"
        ];

        logger.info("Memory Usage");
        logger.info("    " + memoryStats.join(", "));
        logger.info("");
    };

    this.start = function () {
        timer = clearInterval(timer);
        if (!interval) return;
        timer = setInterval(diagnose, interval);
    };

    this.stop = function () {
        timer = clearInterval(timer);
    };
};

module.exports = Diagnostics;