
if (!Array.prototype.getIndexByField) 
    Array.prototype.getIndexByField = function (field, value) {
        for (var i=0; i < this.length; i++)
            if (this[i][field] === value)
                return i;
        return -1;
    };

if (!Array.prototype.replace) 
    Array.prototype.replace = function (oldElem, newElem) {
        let idx = this.indexOf(oldElem);
        if (idx < 0) return;
        this.splice(idx, 1, newElem);
    };

if (!Array.prototype.remove)
    Array.prototype.remove = function (value) {
        for (var i=0; i < this.length; i++)
            if (this[i] === value)
                return this.splice(i, 1)[0];
        return null;
    };    

if (!Array.prototype.delete)
    Array.prototype.delete = function (value) {
        for (let i=0; i < this.length; i++)
            if (this[i] === value)
                return this.splice(i, 1)[0];
        return null;
    };

if (!Array.prototype.unique) 
    Array.prototype.unique = function () {
        const seen = {};
        return this.filter(item => seen.hasOwnProperty(item) ? false : (seen[item] = true));
    };

if (!Array.prototype.without)
    Array.prototype.without = function (unwanted) {
        unwanted = unwanted || [];
        return this.filter(item => !unwanted.includes(item));
    }; 

if (!Array.prototype.flat)
    Array.prototype.flat = function () {
        return this.reduce((acc, val) => acc.concat(val), []);
    };

if (!Array.prototype.flatUnique)
    Array.prototype.flatUnique = function (field) {
        const result = [];
        const seen   = {};

        let item = null;
        for (let i=0; i < this.length; i++) {
            for (let j=0; j < this[i].length; j++) {
                item = this[i][j];
                if (!seen.hasOwnProperty(item[field])) 
                    seen[item[field]] = result.push(item);
            }
        }
        return result;
    };

if (!String.prototype.padStart) 
    String.prototype.padStart = function (targetLength, padString) {
        targetLength = targetLength>>0; 
        padString = String((typeof padString !== 'undefined' ? padString : ' '));

        if (this.length > targetLength) {
            return String(this);
        }
        else {
            targetLength = targetLength - this.length;
            if (targetLength > padString.length) {
                padString += padString.repeat(targetLength/padString.length); 
            }
            return padString.slice(0, targetLength) + String(this);
        }
    };

if (!String.prototype.padEnd) 
    String.prototype.padEnd = function (targetLength, padString) {
        targetLength = targetLength>>0; 
        padString = String((typeof padString !== 'undefined' ? padString : ' '));

        if (this.length > targetLength) {
            return String(this);
        }
        else {
            targetLength = targetLength - this.length;
            if (targetLength > padString.length) {
                padString += padString.repeat(targetLength/padString.length); 
            }
            return String(this) + padString.slice(0, targetLength);
        }
    };
    
// Utility function to throttle access to a function.
// Usage:  throttledFunction = myFunction.throttle(this, minInterval, curriedArgs);   
if (!Function.prototype.throttle)
    Function.prototype.throttle = (context, limit, ...curriedArgs) => {
        const func = this;
        let inThrottle = false;
        return function() {
            const extraArgs = [...arguments];
            if (!inThrottle) {
                inThrottle = true;
                func.apply(context, curriedArgs.concat(extraArgs));
                setTimeout(() => inThrottle = false, limit);
            }
        }
    };
