'use scrict'

class OpenAPIHelper {

    constructor(config) {
        this.config = config;
    }

    getServiceFlowAPI(request, response) {
        const contentType = 'application/json';

        // Get that thing but place the available libraries in the "servers" property.
        // Then return serialized.
        const libraryAPIFile = require("../doc/serviceflowapi.json");
        let libInfo = null;

        libraryAPIFile.servers = [];

        if (Array.isArray(this.config.library)) {
            for (let i = 0; i < this.config.library.length; i++) {
                libInfo = this.config.library[i];
                libraryAPIFile.servers.push({
                    url: "http://" + (libInfo.host || request.hostname) + ":" + libInfo.port,
                    description: libInfo.name + " (" + libInfo.tenants.join(", ") + ")"
                });
            }
        }
        else {
            libInfo = this.config.library;
            libraryAPIFile.servers.push({
                url: "http://" + (libInfo.host || request.hostname) + ":" + libInfo.port,
                description: libInfo.name + " (" + libInfo.tenants.join(", ") + ")"
            });
        }
        response.statusCode = 200;
        response.setHeader('Content-Type', contentType);
        response.write(JSON.stringify(libraryAPIFile));
        response.end();
    }

    getLibraryAPI(request, response) {
        const contentType = 'application/json';

        // Get that thing but place the available libraries in the "servers" property.
        // Then return serialized.
        const libraryAPIFile = require("../doc/libraryapi.json");
        let libInfo = null;

        libraryAPIFile.servers = [];

        if (Array.isArray(this.config.library)) {
            for (let i = 0; i < this.config.library.length; i++) {
                libInfo = this.config.library[i];
                libraryAPIFile.servers.push({
                    url: "http://" + (libInfo.host || request.hostname) + ":" + libInfo.port,
                    description: libInfo.name + " (" + libInfo.tenants.join(", ") + ")"
                });
            }
        }
        else {
            libInfo = this.config.library;
            libraryAPIFile.servers.push({
                url: "http://" + (libInfo.host || request.hostname) + ":" + libInfo.port,
                description: libInfo.name + " (" + libInfo.tenants.join(", ") + ")"
            });
        }
        response.statusCode = 200;
        response.setHeader('Content-Type', contentType);
        response.write(JSON.stringify(libraryAPIFile));
        response.end();
    }

    getConfigAPI(request, response) {
        const contentType = 'application/json';

        // Return a configapi.json that contains the custom requests.
        function findParams(customRequest, requestParams = [], autoParams = []) {
            // const requestParamMatch = RegExp('(?:\[request\.)(.*?)(?=\])', 'g');
            const macroPrefix = "[request.";
            const macroPostFix = "]";

            const paramNames = customRequest.params ? Object.getOwnPropertyNames(customRequest.params) : [];

            for (const param of paramNames) {
                // Examine every property to see if we can find [request.*] macros.
                // If the macro is not yet seen as a request parameter, its public.
                let value = customRequest.params[param];
                // const matches = value.matchAll(requestParamMatch);
                let matches = [];
                let name = null;
                let start = -1;
                let end = -1;
                while ((start = value.indexOf(macroPrefix)) !== -1) {
                    end = value.indexOf(macroPostFix);
                    name = value.substring(start + macroPrefix.length, end);
                    value = value.replace(macroPrefix + name + macroPostFix, "");
                    matches.push(name);
                }
                // Body of this can go into body of the above.
                for (const match of matches) {
                    if (!requestParams.includes(match) && !autoParams.includes(match))
                        requestParams.push({
                            name: match,
                            description: ""
                        });
                }
                // From now on, the current param can be referenced in "next" requests,
                // and its existence should be noted so that future references to it are
                // not interpreted as query parameters for the customRequest as a whole.
                if (!autoParams.includes(param))
                    autoParams.push(param);
            }

            if (customRequest.next)
                return findParams(customRequest.next, requestParams, autoParams);
            else
                return requestParams;
        }

        // Get that thing but place the available libraries in the "servers" property.
        // Then return serialized.
        const configAPIFile = require("../doc/configapi.json");
        const apiTemplate = require("../doc/apitemplate.json");
        const scriptNames = Object.getOwnPropertyNames(this.config.HTTPScripts);

        for (const name of scriptNames) {
            const customRequest = this.config.HTTPScripts[name];

            const entry = configAPIFile.paths["/custom/{tenant}/" + name] = {};
            entry.get = JSON.parse(JSON.stringify(apiTemplate));
            entry.get.summary = customRequest.description || entry.get.summary;
            entry.get.operationId = "ID_" + Math.random();

            const params = typeof customRequest === "string" ?
                [{
                    name: "query",
                    description: "The complete query string, with the values encoded. For instance, a '#' character needs to be provided as '%23'."
                }] :
                findParams(customRequest);

            for (const param of params) {
                entry.get.parameters.push({
                    name: param.name,
                    in: "query",
                    required: true,
                    description: param.description,
                    schema: {
                        type: "string"
                    }
                });
            }
        }

        configAPIFile.servers = [];

        configAPIFile.servers.push({
            url: "http://" + request.hostname + ":" + this.config.configPort,
            description: "ConfigService"
        });


        response.statusCode = 200;
        response.setHeader('Content-Type', contentType);
        response.write(JSON.stringify(configAPIFile));
        response.end();
    }
}

module.exports = OpenAPIHelper;
