## Getting Started

- Prerequisites
  - nodeJS 10+
  - NPM package manager
  - MariaDB database
  - (Optionally) HeidiSQL to manage MariaDB databases and users.

- Installation
  - Download the source code to your local system.
  - Navigate to the /config directory and open 'config.json' in a text editor.
    - Under "Tenants" configure your tenants (including the Dynatrace API token).
    - Under "library" ensure that your tenant is referenced. Also, set the "launch" property to "embedded".
    - Under "database" configure the location, port, user and password.
  - In MariaDB create a database named "StreamingAPI". 
    - It is easiest to use HeidiSQL for this task.
    - Use the "utf8-general-ci" collation strategy. 
    - Ensure that the user with which you connect to that database has "CREATE TABLE" permissions for the "StreamingAPI" database.
  - Navigate to the /server directory.
    - Run 'nmp install'. This will install libraries required by the StreamingAPI.
    - Check that no error messages appear.
    - Run "node StreamerService.js".
    - Check that no error messages appear.
  - Once the StreamerService is running, open your browser and navigate to "http://localhost:8080".
    - Log in as "myuser" and password "mypassword". You can change this in the config.json.
    - Click "Continue".
    - In the editor, click "New Job". 
    - You should now see the hosts in the selected tenant.

## Objective

Dynatrace customers have expressed a desire to be able to receive a continuous stream of metrics for selected Dynatrace entities onto enterprise message infrastructures like Kafka. Additionally, there is also a need to stream user session metrics. The Streaming API enables this functionality, both for entity metrics as well as user session metrics.

Metric streams can be consumed by 3rd parties and/or persisted. It may subsequently be used for various purposes:

- Custom reporting in 3rd-party dashboard products like Grafana;
- Consolidation of key performance data points, collected by various APM technologies, of relevant entities and transactions;
- Automated remedial actions based on metric behavior.

## Goals

The Streaming API collects and streams, for subsets of entities and user sessions, minute-level metric values (or lower granularities, where desired or necessary) on an ongoing basis. It should be noted that there are inherent limits to the volume of data that can be retrieved from the Dynatrace API within a given timeframe. Likewise, there are inherent limitations to the number of messages that can be posted to, for instance, a Kafka message bus in the allocated timeframes. The solution therefore aims to automatically adapt its deployment and parallelization strategies to maximize its throughput while not exceeding rate limits placed upon it. Every effort is made to utilize multiple, concurrent endpoints to the Dynatrace API and the Kafka API.

## Solution

The solution consists of the following components:

- A web application to expose a UI in which the user can select entities, sessions, metrics and timeframes. The UI also reports on key deployment, performance and behavioral characteristics to help users make informed decisions with respect to the stress their selections impose on the Dynatrace API.
- A Scheduler that determines which jobs are due for execution at each minute, and how many workers are required to do the work at hand.
- A Controller that assigns the work and controls the automated deployment of worker processes to maximize throughput.
- A variable collection of Worker processes that round-robin across available API endpoints to process their part of the overall workload.
- A Library that analyzes and indexes all Dynatrace entities for fast filtering and retrieval. It exposes a public query interface that is (also) used by the Controller to select the appropriate entities for each job that is due.


Any number of metrics can be added. By default, the average value for a metric will be collected. The user can select any of the available aggregators on each metric from a menu. Also, above the selected list of metrics is a control that allows users to configure the metric granularity for the job.

Below are the three variants of the UI for (1) entity-based jobs with explicitly selected entities, (2) entity-based jobs with a criteria-based entity selection, and (3) user-session/action-based jobs defined in terms of a USQL query.

Note that a &quot;Test&quot; panel is always available (expandable via a handle at the bottom of the page) at which the current job can be tested and validated. This panel can also be used for ad-hoc queries (as temporary jobs). The rows are selectable and can be copied directly into Excel or a text file for further analysis and reporting.
