const DBRendererAPI = function (config) {
    const self = this;

    const DatabaseAdaptor = require("./" + (config.class || "MariaDBAdaptor") + ".js");
    delete config.class;         // The database adaptor itself doesn't need this property.

    const connectionTTL = (config.connectionTTL || 30) * 1000;
    delete config.connectionTTL; // The database adaptor itself doesn't need this property.

    const database = new DatabaseAdaptor(config, logger);

    config = config || {};
    this.config = {
        produce:   config.produce === "graphs" ? "linkages" : config.produce,
        formatter: config.formatter
    };

    this.events = {};

    var emit = function (eventName, data) {
        // Call the subscribed event handlers.
        const event = self.events[eventName] || [];
        event.forEach(fn => {
            fn.call(null, data);
        });
    };

    var subscribe = function (eventName, fn) {
        // Add the subscription (event handler) for this event.
        self.events[eventName] = self.events[eventName] || [];
        self.events[eventName].push(fn);
    };

    const createTable = (session, tableName) => {
        return new Promise((resolve, reject) => {
            logger.info("    Looking for '" + tableName + "' table");

            const statement = "CREATE TABLE `Entities` ( " +
                " `id` varchar(255) NOT NULL, " +
                " `tenant` varchar(100) NOT NULL, " +
                " `entityType` enum('HOST','SERVICE','PROCESS_GROUP','PROCESS_GROUP_INSTANCE','APPLICATION','MOBILE_APPLICATION','CUSTOM_APPLICATION') NOT NULL, " +
                " `lastSeen` bigint(20) unsigned NOT NULL, " +
                " `lastUpdated` bigint(20) unsigned NOT NULL, " +
                " `displayName` varchar(" + maxDisplayNameSize + ") NOT NULL, " +
                " `checksum` binary(32) NULL DEFAULT NULL, " +
                " `entity` longtext NOT NULL, " +
                " PRIMARY KEY (`id`,`tenant`) " +
                ") ENGINE=InnoDB DEFAULT CHARSET=utf8";

            session
            .query(statement)
            .then(result => {
                connection.release(() => {
                    logger.info("    Created table 'Entities' in database");
                    resolve();
                });
            })
            .catch(error => {
                connection.release(() => {
                    if (error.code = "ER_TABLE_EXISTS_ERROR") {
                        logger.info("    Found table 'Entities' in database");
                        resolve();
                    } else {
                        reject(error);
                    }
                });
            });
        });
    }

    const prepare = (session, produce, fieldGetter, tableName, cycle) => {
        return new Promise((resolve, reject) => {
            const fields = produce === "linkages"
                         ? fieldGetter(["source", "target"])
                         : fieldGetter();

            // Check if there is a table with this job name that has the right columns.
            // Note that it is the responsibility of the person creating the template
            // to ensure the columns are always the same. Otherwise the table will get
            // recreated on every job execution.
            const statement = "SELECT WHEN (" + fields.headers.length + " = "
                            +  " (SELECT COUNT(*) Names FROM sys.columns"
                            +     " WHERE OBJECT_ID = OBJECT_ID('" + tableName + "')"
                            +     " AND Name IN ('" + fields.headers.join("', '") + "')"
                            +  " ) THEN 'YES' ELSE 'NO' END";
            session
            .query(statement)
            .then(result => {
                // TODO: Create an insert statement;
                const writeFunction = () => {
                    // This needs to be aware of the 'produce' strategy.
                    // take the fields in the message, or use source and
                    // target and get the fields there.
                    "INSERT INTO..."; // Will include column names. Order?
                }

                if (result === "YES")
                    return resolve(writeFunction);

                // Create the table. Issue: What are the data types?
                // I wonder if the can let the formatter return a lookup table
                // with data types that we cna use here to properly create the table.
                // TODO
                createTable(session, tableName, columnsAndTypes)
                .then(() => {
                    resolve(writeFunction);
                })
                .catch(err => {
                    reject(err);
                });
            })
            .catch(err => {
                reject(err);
            });
        });
    }

    // Create a DestinationConnection object (see DestinationAPI.js).
    const createDestinationConnection = (connection, writeFunction) => ({
        error: null,
        sent: 0,
        send: function (messages, onComplete) {
            // Start a transaction by calling the write function without a message.
            writeFunction()
            .then(() => {
                // Serialize writing potentially many messages to the database.
                messages
                .reduce(async (prevPromise, message) => {
                    try {
                        // Wait until the previous message has been written to DB.
                        await prevPromise;
                        this.sent++;
                    } catch (ex) {
                        // Could not insert a record - take note of the first error.
                        this.error = this.error || ex.message;
                    } finally {
                        // Continue trying to write data even if some messages fail.
                        return writeFunction(message);  // Becomes the next promise.
                    }
                }, Promise.resolve())   // Use a resolved promise as starting point.
                .then(() => {           // Now the last promise has resolved - Done.
                    connection.commit(() => {
                        onComplete(this.error, this.sent);
                    });
                })
                .catch(err => {
                    onComplete("Could not persist messages: " + err);
                });
            })
            .catch(err => {
                onComplete("Could not start transaction: " + err);
            });
        },
        close: function () {
            if (connection) {
                connection.release();
            }
            connection = null;
            return true;
        }
        /*  Alternative approach considered to write multiple rows:
            INSERT (or REPLACE) INTO table_name (colA, colB, ...)
            VALUES
                ('valA1', 'valB1', ...),
                ('valA2', 'valB2', ...),
                ...
                ('valAn', 'valBn', ...);
            Concern: Query may get too big if there are many messages.
        */
    });

    // Create a DestinationAPI object (see DestinationAPI.js).
    const instance = {
        on:        subscribe,
        produce:   self.config.produce,
        formatter: self.config.formatter,
        open: function (job, fieldGetter, onConnectionOpen) {
            const name = job.name;
            const cycle = job.cycle;
            
            const connection = database.createConnection(
                // Use the job name to name the connection. It doesn't affect anything in any way.
                name,
                // Called when connection has been acquired - triggered by 'database.acquire(connection)'.
                session => {
                    // Ensure the table exists (with the correct columns), and then return an object that
                    // exposes 'send()' and 'close()' methods, per the "Destination.js" specifications.
                    prepare(session, instance.produce, fieldGetter, name, cycle)
                    .then(writeFunction => {
                        // The writeFunction created by 'prepare()' knows how to write messages to the DB.
                        onConnectionOpen(null, createDestinationConnection(connection, writeFunction));
                    })
                    .catch(err => connection.release(() => {
                        onConnectionOpen(err)
                    }));
                },
                // Called when connection cannot be acquired - triggered by 'database.acquire(connection)'.
                err => {
                    onConnectionOpen(err);
                },
                // Called after connection has been released - triggered by 'connection.release()'.
                () => {}
            );

            // Note that 'createConnection()' does not establish a session - it just prepares and returns
            // an object at which we can issue queries and so on. The below establish an actual session.
            database.acquire(connection, sameConnection => {
                // We set a timer to auto-release the connection in case that somehow doesn't happen.
                setTimeout(connection.release.bind(connection), connectionTTL);
            });
        }
    };

    // We must raise this event asynchronously.
    setTimeout(emit, 1, "ready");

    // Return the DestinationAPI instance.
    return instance;
};

module.exports = DBRendererAPI;
