const http = require('http');
const https = require('https');
const axios = require('axios');

const HTTPSenderAPI = function (config) {
    config = config || {};
    this.config = {
        url: config.url,
        produce: "entities",
        formatter: config.formatter
    };

    this.events = {};

    var emit = function (eventName, data) {
        // Call the subscribed event handlers.
        const event = self.events[eventName] || [];
        event.forEach(fn => fn.call(null, data));
    };

    var subscribe = function (eventName, fn) {
        // Add the subscription (event handler) for this event.
        self.events[eventName] = self.events[eventName] || [];
        self.events[eventName].push(fn);
    };

    const self = this;

    const tokenConfig = require('./config/idanywhere.json');
    const params = {
        grant_type: tokenConfig.grant_type,
        client_id: tokenConfig.client_id,
        username: `AD\\${tokenConfig.username}`,
        password: tokenConfig.password,
        resource: tokenConfig.resource_id
    };
    const encQuery = encodeURI(Object.keys(params).map(key => `${key}=${params[key]}`).join('&'));

    const options = {
        headers: {
            'accept': 'application/x-www-form-urlencoded',
            'Content-Type': 'application/x-www-form-urlencoded'
        }
    };

    let bearerToken = null;

    axios.post(config.tokenUrl, encQuery, options)
        .then(token => {
            bearerToken = token.data ? token.data.access_token : "Unknown";

            data.decoded = {
                oauthkey: JSON.parse(Buffer.from(token.data.access_token.split('.')[0], 'base64').toString('utf8')),
                oauth: JSON.parse(Buffer.from(token.data.access_token.split('.')[1], 'base64').toString('utf8')),
            };
            emit("ready");
        })
        .catch(err => {
            emit("error", err);
        });

    return {
        on: subscribe,
        produce: self.config.produce,
        formatter: self.config.formatter,
        open: function (job, fieldGetter, onConnectionOpen) {
            const name = job.name;
            const cycle = job.cycle;

            setTimeout(onConnectionOpen, 1, null, {
                send: function (messages, onComplete) {
                    const reqData = {
                        port: 443,
                        method: 'post',
                        path: config.url,
                        headers: {
                            'Authorization': 'Bearer ' + bearerToken,
                            'Content-Type': 'application/json',
                            'Accept': 'application/json'
                        },
                        timeout: 10000,
                        rejectUnauthorized: false // We may bypass load balancers and stuff, so certs may mismatch.
                    };

                    let completed = 0;
                    for (let i = 0; i < messages.length; i++) {
                        const request = https.request(reqData, result => {
                            // TODO: Check if result.statusCode is 200
                            result.setEncoding('utf8');
                            result.on('data', chunk => {
                                data.push(chunk); // Collect response in chunks.
                            });
                            result.on('end', () => {
                                var responseData = data.join("");

                                completed++;
                                if (completed >= messages.length)
                                    setTimeout(onComplete, 1); // We let caller know we sent everything
                            });
                        });
                        request.write("[" + JSON.stringify(messages[i]) + "]");
                        request.end();
                    }
                },
                close: function () {
                    return true;
                }
            });
        }
    };
};

module.exports = HTTPSenderAPI;
