const fs = require('fs');
const { BlobServiceClient, StorageSharedKeyCredential } = require("@azure/storage-blob");

// Load the .env file if it exists
//require("dotenv").config();

const addPrefixToProps = (obj, prefix) =>
    Object.keys(obj).reduce(
        (acc, key) => ({
            ...acc,
            ...{ [prefix + key]: obj[key] }
        }),
        {}
    );

const JSONSenderAPI = function (config, tenants) {
    const self = this;

    this.events = {};

    var emit = function (eventName, data) {
        // Call the subscribed event handlers.
        const event = self.events[eventName] || [];
        event.forEach(fn => { fn.call(null, data); });
    };

    var subscribe = function (eventName, fn) {
        // Add the subscription (event handler) for this event.
        self.events[eventName] = self.events[eventName] || [];
        self.events[eventName].push(fn);
    };

    setTimeout(emit, 1, "ready");

    const instance = {
        on: subscribe,
        produce:   config.produce,
        formatter: config.formatter,
        open: function (job, fieldGetter, onConnectionOpen) { 
            // ALso: job.startTime, job.endTime
            const name = job.name;
            const cycle = job.cycle;
            const tenant = job.source;
            const originatingTenantConfig = tenants[tenant];

            setTimeout(() => {
                onConnectionOpen(null, {
                    send: async function (messages, onComplete) {

                        if (instance.produce === "linkages") {
                            // Merge the source and target into one object, whith prefixed property names.
                            messages = messages.map(message => ({
                                ...addPrefixToProps(message.source, "source"),
                                ...addPrefixToProps(message.target, "target")
                            }));
                        }
        
                        // Send to BLOB storage account.
                        // Enter your storage account name and shared key
                        const account = process.env.ACCOUNT_NAME || config.azureAccount;
                        const accountKey = process.env.ACCOUNT_KEY || config.azureAccountKey;

                        const sharedKeyCredential = new StorageSharedKeyCredential(account, accountKey);

                        const blobServiceClient = new BlobServiceClient(
                            // When using AnonymousCredential, following url should include a valid SAS or support public access
                            `https://${account}.blob.core.windows.net`,
                            sharedKeyCredential
                        );

                        const containerName = originatingTenantConfig.storagefolder
                        const containerClient = blobServiceClient.getContainerClient(containerName);
                        //const blobName = "hosts" + new Date().getTime();
                        const blobName = originatingTenantConfig.environment + "_" + job.startTime + ".json";
                        const blockBlobClient = containerClient.getBlockBlobClient(blobName);
                        const hostContent = JSON.stringify(messages, null, 4);

                        const uploadBlobResponse = await blockBlobClient.upload(hostContent, Buffer.byteLength(hostContent));
                
                        // setTimeout(onComplete, 1);
                        onComplete();
                    },
                    close:  function () { 
                        return true; 
                    }
                });
            }, 1);
        }
    };
    return instance;
};

module.exports = JSONSenderAPI;
