export declare type SessionStoreOptions = {
    instanceId?: string;
    cacheOnDisk?: boolean;
    cachePath?: string;
    maxStoreTime?: number;
};
/**
 * Run on "primary"
 */
export declare class SharedSessionStore<T = unknown> {
    private app;
    private server;
    private db;
    /**
     *
     * useful if you intend to use multiple session stores.
     * @param port port that the store will listen on.
     */
    constructor(port?: number, options?: SessionStoreOptions);
    createInstance(options?: any): void;
    /**
     * Method to release all handles and flush all writes to disk.
     * If the program terminates without this called, in rare cases data could be lost.
     */
    destroy(): void;
    private launchServer;
}
/**
 * Run in worker instances
 */
export declare class SessionStoreClient<T = unknown> {
    private port;
    private instanceId;
    /**
     * Create a new Session Store Client instance.
     * @param port Port of Level cluster to listen on
     * @param instanceId Instance ID for this client. If you have multiple instances that you need to interface, create multiple client objects.
     */
    constructor(port?: number, instanceId?: string);
    /**
     * Get an item from the store
     * @param key Key to look for item at
     * @returns stored item OR `undefined`. Getting a response of `undefined` means that there was no value for the given key.
     *
     */
    get(key: string): Promise<T>;
    /**
     * Put an item into the store.
     * @param key   Key to store value at.
     * @param value Value to store at key.
     * @returns true/false. Returns true if operation was successful.
     */
    put(key: string, value: T): Promise<boolean>;
    /**
     * Delete an item from the store.
     * @param key Key of item to delete
     * @returns true/false. Returns true if operation was successful.
     */
    delete(key: string): Promise<boolean>;
    /**
     * @experimental
     * @param entries
     * @returns
     */
    batch(entries: Array<{
        type: "put" | "del" | "get";
        key: string;
        value: T;
    }>): Promise<unknown>;
    /**
     * Make the request to the target store.
     *
     * @param storeId string ID of cache within store.
     * @param key     which element in the store to work on
     * @param action  "get" "put" "del" -- do we delete the key, set the value for the key, or get the value?
     * @param value   when action is "put", what to store for the specified key.
     * @returns
     */
    private request;
}
