/**
 * Assert if a string is a Dynatras SaaS url, Managed URL, or ActiveGate URL (incl. localhost).
 *  - https://abc012345.live.dynatrace.com/
 */
export declare const dynatraceUrlRegex: RegExp;
/**
 * Assert if a string is a Dynatrace SaaS url.
 *  - https://abc012345.live.dynatrace.com/
 *
 * Groups:
 * | Index | Item |
 * | ---   | ---  |
 * | 1 | Protocol |
 * | 2 | Tenant Id |
 * | 3 | Stage  |
 * | 4 | Domain |
 */
export declare const dynatraceSaaSUrlRegex: RegExp;
/**
 * Assert if a string is a Dynatrace SaaS host.
 *  - https://abc012345.live.dynatrace.com/
 *
 * Groups:
 * | Index | Item |
 * | ---   | ---  |
 * | 1 | Tenant Id |
 * | 2 | Stage  |
 * | 3 | Domain |
 */
export declare const dynatraceSaaSHostRegex: RegExp;
/**
 * Assert if a string is either a V1 or V2 API token format.
 *
 * Groups (V2 format only):
 * | Index | Item | Description |
 * | ---   | ---  | --- |
 * | 1 | Unique Prefix | Unique prefix identifying a Dynatrace API token |
 * | 2 | Public Portion of token | This is a 24-character public identifier of the token. This value can be safely displayed in the UI and can be used for logging purposes. |
 * | 3 | Secret Portion of token | This is a 64-character secret portion of the token, which can be treated like a password and therefore doesn’t need to be displayed in the Dynatrace web UI (following initial creation) or stored in log files. |
 */
export declare const dynatraceTokenRegex: RegExp;
/**
 * Assert if a string matches the Dynatrace V1 Token format.
 *  - <...>21
 *
 * @deprecated The V1 token format is deprecated. Thus newly created tokens will _never_ have this format.
 * https://www.dynatrace.com/news/blog/further-enhance-security-by-easily-automating-your-api-token-protection/
 */
export declare const dynatraceV1TokenFormatRegex: RegExp;
/**
 * Assert if a string matches the Dynatrace V2 Token format.
 *  - dt0c01.<...>24.<...>64
 *
 * Groups:
 * | Index | Item | Description |
 * | ---   | ---  | --- |
 * | 1 | Unique Prefix | Unique prefix identifying a Dynatrace API token |
 * | 2 | Public Portion of token | This is a 24-character public identifier of the token. This value can be safely displayed in the UI and can be used for logging purposes. |
 * | 3 | Secret Portion of token | This is a 64-character secret portion of the token, which can be treated like a password and therefore doesn’t need to be displayed in the Dynatrace web UI (following initial creation) or stored in log files. |
 */
export declare const dynatraceV2TokenFormatRegex: RegExp;
